@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -50,7 +49,6 @@ struct Algorithm
     char          **infiles; /* an array of the input files listed on the command line */
     int             filenum; /* number of input files */
     char            rootname[FILENAME_MAX];
-    int             weight; /* weighting method */
     int             verbose;  /* lots of output */
     double          precision; /* requested relative precision to converge to */
     int             iterations; /* max # of iterations allowed for outer loop of MultiPose() */
@@ -73,7 +71,6 @@ struct Algorithm
     int             tenberge;
     int             morph;
     int             stats; /* calculate moment stats */
-    double          constant; /* minimum variance allowed */
     int             info; /* just calculate stats for given pdb file and quit */
     int             princaxes; /* flag to align final superposition with principle axes of mean structure */
     int             nullrun;
@@ -118,6 +115,10 @@ struct Algorithm
     int             scaleanchor; /* model index for relative scaling -- scale of this model = 1 */
     int             randgibbs; /* Randomly initialize GibbsMet */
     double          covnu;
+    double          ndf;
+    int             crush;
+    int             mg_crush;
+    int             parallel;
 };
 
 
@@ -176,7 +177,7 @@ struct StCdsArray
     Cds            *avecds;  /* average Cds of all in CdsArray */
 
     double         *evals;
-    double         *samplevar3N; /* atomic sample variances */
+    double         *samplevar; /* atomic sample variances */
     double        **CovMat;      /* the atomic, row-wise covariance matrix */
 };
 
@@ -187,7 +188,6 @@ struct CdsParams
 
     double         *prvar;      /* prior variances */
 
-    double         *residual_x, *residual_y, *residual_z;
     double         *covx, *covy, *covz; /* covariance matrix weighted x,y,z cds */
 
     double        **matrix;      /* 3x3 rotation matrix */
@@ -214,17 +214,17 @@ struct Params
     int             vlen;       /* number of coordinates */
     int             cnum;       /* number of Cds in array */
 
-    CdsParams    **cdsp;     /* array of coords parameters */
+    CdsParams    **cdsp;        /* array of coords parameters */
 
-    Cds            *avecds;  /* average Cds of all in CdsArray */
-    Cds            *tcds;    /* target Cds */
+    Cds            *avecds;     /* average Cds of all in CdsArray */
+    Cds            *tcds;       /* target Cds */
 
-    double         *w;           /* diagonal atomic weights */
-    double         *var;         /* atomic variance estimates */
+    double         *w;          /* diagonal atomic weights */
+    double         *var;        /* atomic variance estimates */
     double         *evals;
-    double         *samplevar3N; /* atomic sample variances */
-    int            *df;          /* degrees of freedom for variances, for incomplete data alignments */
-    double         *S2;          /* theoretical NMR order parameters */
+    double         *samplevar;  /* atomic sample variances */
+    int            *df;         /* degrees of freedom for variances, for incomplete data alignments */
+    double         *S2;         /* theoretical NMR order parameters */
 
     double         *residuals;  /* 3 x vlen x cnum vector of normalized residuals */
 
@@ -268,25 +268,23 @@ struct Cds
     char           *chainID; /* chain ID */
     int            *resSeq;  /* residue number */
 
-    double        **wc; /* 3 x K matrix matrix of working coordinates, aliased to x,y,z below */
+    double        **wc; /* 5 x K matrix matrix of working coordinates, aliased to x,y,z below */
     double         *x, *y, *z; /* x,y,z atomic coordinates */
     double         *o;         /* occupancy */
     double         *b;         /* B-factor */
 
-    int            *nu, *mu; /* binary flag vectors for present and missing data, respectively */
+    int            *nu; /* binary flag vectors for present and not missing data */
 
-    double        **sc; /* 3 x K matrix matrix of static coordinates, aliased to sx,sy,sz below */
+    double        **sc; /* 5 x K matrix matrix of static coordinates, aliased to sx,sy,sz below */
     double         *sx, *sy, *sz; /* x,y,z atomic coordinates */
     double         *so;           /* occupancy */
     double         *sb;           /* B-factor */
 
-    double        **cc; /* inv covariance weighted coordinates */
+    double        **cc; /* 3 x K matrix, inv covariance weighted coordinates */
     double         *covx, *covy, *covz; /* inv covariance matrix weighted x,y,z cds */
 
     double         *prvar;      /* prior variances */
 
-    double         *residual_x, *residual_y, *residual_z;
-
     double        **matrix;      /* 3x3 rotation matrix */
     double        **last_matrix; /* temp 3x3 rotation matrix */
     double        **last_outer_matrix; /* temp 3x3 rotation matrix */
@@ -319,12 +317,12 @@ struct CdsArray
     struct PDBCdsArray  *pdbA;     /* associated PDBCdsArray */
     struct CdsArray     *scratchA; /* associated scratch array of Cds */
 
-    char           outfile_name[FILENAME_MAX];
-    int            vlen;       /* number of coordinates */
-    int            cnum;       /* number of Cds in array */
-    char          *anchorf_name;
-    char          *mapfile_name;
-    char          *msafile_name;
+    char            outfile_name[FILENAME_MAX];
+    int             vlen;       /* number of coordinates */
+    int             cnum;       /* number of Cds in array */
+    char           *anchorf_name;
+    char           *mapfile_name;
+    char           *msafile_name;
 
     Cds           **cds;     /* pointer to an array of pointers to Cds */
     Cds            *avecds;  /* average Cds of all in CdsArray */
@@ -334,8 +332,9 @@ struct CdsArray
 
     double         *w;           /* diagonal atomic weights */
     double         *var;         /* atomic variance estimates */
+    double         *invvar;
     double         *evals;
-    double         *samplevar3N; /* atomic sample variances */
+    double         *samplevar; /* atomic sample variances */
     int            *df;          /* degrees of freedom for variances, used for incomplete data alignments */
     double         *S2;          /* theoretical NMR order parameters */
 
