@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -71,16 +70,16 @@ FragDist(CdsArray *cdsA, int fraglen, int center_ca)
                 for (k = 0; k < fraglen; ++k)
                 {
                     tmp = i + k - offset;
-    
+
                     frag1->x[k] = cdsA->cds[coord1]->x[tmp];
                     frag1->y[k] = cdsA->cds[coord1]->y[tmp];
                     frag1->z[k] = cdsA->cds[coord1]->z[tmp];
-    
+
                     frag2->x[k] = cdsA->cds[coord2]->x[tmp];
                     frag2->y[k] = cdsA->cds[coord2]->y[tmp];
                     frag2->z[k] = cdsA->cds[coord2]->z[tmp];
                 }
-    
+
                 if (center_ca == 1)
                 {
                     CenterFragCA(frag1);
@@ -91,7 +90,7 @@ FragDist(CdsArray *cdsA, int fraglen, int center_ca)
                     CenterFrag(frag1);
                     CenterFrag(frag2);
                 }
-    
+
                 var = QuarticHornFrag((const FragCds *) frag1, (const FragCds *) frag2, coeff);
 
                 fprintf(distfile, "%-16.8e \n", var);
