@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -34,6 +33,7 @@
 #include <gsl/gsl_errno.h>
 #include <gsl/gsl_math.h>
 #include <gsl/gsl_min.h>
+#include "DLTutils.h"
 #include "CovMat.h"
 #include "Error.h"
 #include "Cds.h"
@@ -89,10 +89,10 @@ dlt_min_fminimizer_set_with_values (gsl_min_fminimizer *s, gsl_function *f,
     s->f_upper = f_upper;
     s->f_minimum = f_minimum;
 
-    if (f_minimum >= f_lower && f_minimum >= f_upper) // DLT
+    if (f_minimum >= f_lower && f_minimum >= f_upper)
     {
-        GSL_ERROR("guess is worse than both endpoints", GSL_EINVAL); // DLT
-    } // DLT
+        GSL_ERROR("guess is worse than both endpoints", GSL_EINVAL);
+    }
 
     return(s->type->set)(s->state, s->function,
                          x_minimum, f_minimum,
@@ -227,33 +227,57 @@ InvGammaAdjustVar(double *newvar, const int vlen, const int cnum,
         // this is the conditional maximization algorithm
         //newvar[i] = (3.0*cnum*var[i] + 2.0*b) / (3.0*cnum + 2.0*c);
         // this is required for an EM algorithm of the variances
-        //printf("%3d %26.6f\n", i, var[i]);
 }
 
 
+// this is required for an EM algorithm of the variances
 static void
-InvGammaAdjustVar3N(double *newvar, const int vlen, const int cnum,
-                    double *var, const double phi, const double nu)
+InvGammaAdjustVarPhi(double *newvar, const int vlen, const int cnum,
+                     double *var, const double phi, const double ndf)
 {
     int         i;
+    double      fact = 1.0 / (3.0 * cnum + ndf);
 
     for (i = 0; i < vlen; ++i)
-        newvar[i] = (var[i] + phi) / (3.0*cnum + nu);
-        // this is required for an EM algorithm of the variances
+    {
+        newvar[i] = fact * (3.0 * cnum * var[i] + phi);
         //printf("%3d %26.6f\n", i, var[i]);
+    }
+}
+
+
+// this is required for an EM algorithm of the variances
+void
+InvGammaAdjustVarPhiNu(Cds **cds, double *newvar, const int vlen, const int cnum,
+                       double *var, const double phi, const double ndf)
+{
+    int         i, j;
+    double      ri;
+
+    for (i = 0; i < vlen; ++i)
+    {
+        // could use CalcDf() here
+        ri = 0;
+        for (j = 0; j < cnum; ++j)
+            ri += cds[j]->nu[i];
+
+        newvar[i] = (3.0 * ri * var[i] + phi) / (3.0 * ri + ndf);
+        //printf("%3d %26.6e %26.6f %26.6f %26.6f %26.6f\n", i, var[i], phi, ri, ndf, newvar[i]);
+    }
+    //printf("\n");
 }
 
 
 void
 InvGammaAdjustEvals(double *newevals, const int vlen, const int cnum,
-                    double *evals, const double phi, const double nu)
+                    double *evals, const double phi, const double ndf)
 {
     int         i;
 
     for (i = 0; i < vlen; ++i)
-        newevals[i] = (3.0*cnum*evals[i] + phi) / (3.0*cnum + nu);
+        newevals[i] = (3.0*cnum*evals[i] + phi) / (3.0*cnum + ndf);
         // this is for expected inverse
-        // newevals[i] = (3.0*cnum*evals[i] + phi) / (3.0*cnum + nu - vlen - 1.0);
+        // newevals[i] = (3.0*cnum*evals[i] + phi) / (3.0*cnum + ndf - vlen - 1.0);
         // this is for expected covariance matrix (not inverse)
 
         //printf("%3d %26.6f\n", i, var[i]);
@@ -273,7 +297,7 @@ InvGammaAdjustVarNu(Cds **cds, double *newvar, const int vlen, const int cnum,
         for (j = 0; j < cnum; ++j)
             df += cds[j]->nu[i];
 
-        df *= 3;
+        df *= 3.0;
 
         newvar[i] = (df*var[i] + 2.0*b) / (df + 2.0*(1.0 + c));
     }
@@ -284,23 +308,32 @@ double
 LogPrPhi(double phi, void *params)
 {
     CdsArray       *cdsA = (CdsArray *) params;
-    const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
-    double          nu = 1.0;
-    double         *var3N = cdsA->samplevar3N;
-    double          fact = 0.5*(3.0*cnum + nu);
-    double          term;
+    //double          term;
+    const int       vlen = cdsA->vlen;//, cnum = cdsA->cnum;
+    double          ndf = algo->ndf;
+    double         *var = cdsA->samplevar;
+    double          fact, term, ri;
     int             i;
 
+    stats->hierarch_p1 = 0.5*phi;
+
     if (phi <= 0.0)
         return(DBL_MAX);
 
+    //term = CalcMgLogLNu(cdsA);
+
     term = 0.0;
     for (i = 0; i < vlen; ++i)
-        term += log(phi + var3N[i]);
+    {
+        ri = cdsA->df[i];
+        fact = 0.5*(3.0*ri + ndf);
+        term += fact*log(phi + 3.0*ri*var[i]);
+    }
 
-    //term = 0.5*vlen*log(phi) - fact*term;
-    term = 0.5*vlen*nu*log(phi) - fact*term;
-    //term += log(phi) - phi/1; // prior on phi
+    term = 0.5*vlen*ndf*log(phi) - term;
+    term -= phi; // exponential prior on phi, mean = 1 // must change marg lik too!
+
+//     term += log(phi) - phi/1; // prior on phi
 
     if (!isfinite(term))
         return(DBL_MAX);
@@ -314,9 +347,9 @@ LogPrPhiCov(double phi, void *params)
 {
     CdsArray       *cdsA = (CdsArray *) params;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
-    double          nu = algo->covnu;
-    double         *evals = cdsA->samplevar3N;
-    double          fact = 0.5*(3.0*cnum + nu);
+    double          ndf = algo->ndf;
+    double         *evals = cdsA->samplevar;
+    double          fact = 0.5*(3.0*cnum + ndf);
     double          term;
     int             i;
 
@@ -327,9 +360,10 @@ LogPrPhiCov(double phi, void *params)
     for (i = 0; i < vlen; ++i)
         term += log(phi + 3.0 * cnum * evals[i]);
 
-    //term = 0.5*vlen*log(phi) - fact*term;
-    term = 0.5*vlen*nu*log(phi) - fact*term;
-    //term += log(phi); // - phi/1; // prior on phi
+    term = 0.5*vlen*ndf*log(phi) - fact*term;
+    //term += log(phi) - phi/1; // prior on phi
+
+    term -= phi; // exponential prior on phi, mean = 1
 
     if (!isfinite(term))
         return(DBL_MAX);
@@ -343,11 +377,10 @@ LogPrPhi_f(double phi, void *params)
 {
     CdsArray       *cdsA = (CdsArray *) params;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
-    double          nu = 1.0;
-    double         *var3N = cdsA->samplevar3N;
-    double          fact = (3.0*cnum + nu);
-    double          term;
-    int             i;
+    double          ndf = algo->ndf;
+    double         *var = cdsA->samplevar;
+    double          fact, term, ri;
+    int             i, j;
 
     if (phi <= 0.0)
         phi = DBL_EPSILON;
@@ -355,21 +388,27 @@ LogPrPhi_f(double phi, void *params)
     term = 0.0;
     for (i = 0; i < vlen; ++i)
     {
-        term += 1.0 / (phi + var3N[i]);
+        ri = 0.0;
+        for (j = 0; j < cnum; ++j)
+            ri += cdsA->cds[j]->nu[i];
+
+        fact = 0.5*(3.0*ri + ndf);
+        term += fact / (phi + 3.0*ri*var[i]);
     }
 
-    return(vlen/phi - fact*term);
+    return(0.5*vlen*ndf/phi - term);
 }
 
 
+// DLT FIX --- this has to incorporate incomplete info nu
 static double
 LogPrPhi_df(double phi, void *params)
 {
     CdsArray       *cdsA = (CdsArray *) params;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
-    double          nu = 1.0;
-    double         *var3N = cdsA->samplevar3N;
-    double          fact = (3.0*cnum + nu);
+    double          ndf = algo->ndf;
+    double         *var = cdsA->samplevar;
+    double          fact = (3.0*cnum + ndf);
     double          term2, tmp;
     int             i;
 
@@ -379,7 +418,7 @@ LogPrPhi_df(double phi, void *params)
     term2 = 0.0;
     for (i = 0; i < vlen; ++i)
     {
-        tmp = 1.0 / (phi + var3N[i]);
+        tmp = 1.0 / (phi + 3.0*cnum*var[i]);
         term2 += tmp*tmp;
     }
 
@@ -387,14 +426,15 @@ LogPrPhi_df(double phi, void *params)
 }
 
 
+// DLT FIX --- this has to incorporate incomplete info nu
 static void
 LogPrPhi_fdf(double phi, void *params, double *y, double *dy)
 {
     CdsArray       *cdsA = (CdsArray *) params;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
-    double          nu = 1.0;
-    double         *var3N = cdsA->samplevar3N;
-    double          fact = (3.0*cnum + nu);
+    double          ndf = algo->ndf;
+    double         *var = cdsA->samplevar;
+    double          fact = (3.0*cnum + ndf);
     double          term, term2, tmp;
     int             i;
 
@@ -404,7 +444,7 @@ LogPrPhi_fdf(double phi, void *params, double *y, double *dy)
     term = term2 = 0.0;
     for (i = 0; i < vlen; ++i)
     {
-        tmp = 1.0 / (phi + var3N[i]);
+        tmp = 1.0 / (phi + 3.0*cnum*var[i]);
         term += tmp;
         term2 += tmp*tmp;
     }
@@ -426,16 +466,16 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
 {
     const int           vlen = cdsA->vlen, cnum = cdsA->cnum;
     int                 status, i;
-    int                 iter, max_iter = 50;
+    int                 iter, max_iter = 100;
     const               gsl_min_fminimizer_type *T = NULL;
     gsl_min_fminimizer *s = NULL;
     double              oldphi = 2.0*stats->hierarch_p1;
     double              a, b, phi;
     double              chi2 = 0.0, logL;
-    double              nu = 1.0;
+    double              ndf = algo->ndf;
     gsl_function        F;
-    double              absprec = algo->precision;
-    double              relprec = FLT_EPSILON/10;
+    double              absprec = FLT_EPSILON*0.1;
+    double              relprec = 0.0;
 
     T = gsl_min_fminimizer_brent;
 //    T = gsl_min_fminimizer_goldensection;
@@ -451,29 +491,31 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
 
     if (algo->varweight > 0)
     {
-        double     *var3N = cdsA->samplevar3N;
-
-        nu = 1.0;
-
-        CalcVar(cdsA);
+        double     *svar = cdsA->samplevar;
+        double     *var = cdsA->var;
 
 //         phi = 0.0;
 //         for (i = 0; i < vlen; ++i)
-//             phi += 1.0 / (var3N[i]/(3.0*cnum) + oldphi);
-//         phi *= vlen;
+//             phi += 1.0 / (svar[i] + oldphi);
+//         phi = vlen/phi;
+
+// double marg1, marg2, marg3; // DLT TEST
+// if (algo->alignment)
+//  marg1 = CalcMgLogLNu(cdsA);
+// else
+//  marg1 = CalcMgLogL(cdsA);
 
         phi = 0.0;
         for (i = 0; i < vlen; ++i)
-            phi += var3N[i];
-        phi /= 3.0*cnum*vlen;
-
-        //phi = oldphi;
+            phi += 1.0/svar[i];
+        //phi /= vlen;
+        phi = ndf*vlen/(phi+2);
 
         a = DBL_MIN;
         //b = DBL_MAX;
         // phi should never, ever be larger than the largest variance
         // in fact, it probably shouldn't be larger than the average variance (its approx the harm ave)
-        b = var3N[findlargest(var3N, vlen)]/(3.0*cnum) + 1.0;
+        b = svar[findlargest(svar, vlen)] + 1.0;
 
 //         printf("\na: %g phi: %g b: %g \n\n",
 //                LogPrPhi(a, (void *) cdsA),
@@ -499,14 +541,50 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
 
 //             if (status == GSL_SUCCESS)
 //                 printf ("Converged:\n");
-// //
+//
 //             printf ("%5d [%.7f, %.7f] %.7f %.7f\n", iter, a, b, phi, b - a);
+
         }
         while(status == GSL_CONTINUE && iter < max_iter);
 
-        InvGammaAdjustVar3N(cdsA->var, vlen, cnum, var3N, phi, nu);
+//         double tmpsum = 0.0;
+//         for (i=0; i < vlen; ++i)
+//             tmpsum += 1.0 / var[i];
+//
+//         printf ("%5d [%.7f, %.7f] %.7f %.7f %.7f\n", iter, a, b, phi, b - a, ndf*vlen/(tmpsum+2));
+        //phi = ndf*vlen/(tmpsum+2);
+        //phi = (ndf*vlen - 2.0)/tmpsum;
 
-        chi2 = chi_sqr_adapt(cdsA->var, vlen, 0, &logL, 0.5*phi, 0.5*nu,
+// if (algo->alignment) // DLT TEST
+//  marg2 = CalcMgLogLNu(cdsA);
+// else
+//  marg2 = CalcMgLogL(cdsA);
+
+// printf("PhiMargLogL ///////////////////////////////////////////// % e % e % e\n",
+//     marg1, marg2, marg2 - marg1);
+// fflush(NULL); // DLT TEST
+
+        if (algo->alignment)
+            InvGammaAdjustVarPhiNu(cdsA->cds, var, vlen, cnum, svar, phi, ndf);
+        else
+            InvGammaAdjustVarPhi(var, vlen, cnum, svar, phi, ndf);
+
+//         tmpsum = 0.0;
+//         for (i=0; i < vlen; ++i)
+//             tmpsum += 1.0 / var[i];
+//
+//         printf ("      [%.7f, %.7f] %.7f %.7f %.7f\n", a, b, phi, b - a, ndf*vlen/(tmpsum+2));
+
+// if (algo->alignment) // DLT TEST
+//  marg3 = CalcMgLogLNu(cdsA);
+// else
+//  marg3 = CalcMgLogL(cdsA);
+
+// printf("Var2MargLogL ///////////////////////////////////////////// % e % e % e\n",
+//     marg2, marg3, marg3 - marg2);
+// fflush(NULL); // DLT TEST
+
+        chi2 = chi_sqr_adapt(var, vlen, 0, &logL, 0.5*phi, 0.5*ndf,
                              invgamma_pdf, invgamma_lnpdf, invgamma_int);
     }
     else if (algo->covweight > 0)
@@ -514,14 +592,24 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
         double     *evals = cdsA->evals;
         double     *invevals = cdsA->tmpvecK;
         double    **evecs = cdsA->tmpmatKK2;
-        double     *tmpevals = cdsA->samplevar3N;
+        double     *tmpevals = cdsA->samplevar;
         int         i, j;
 
-        nu = algo->covnu;
+
+        ndf = algo->ndf;
         /* must calc evals before minimizer is set */
         /* EigenGSL 0 evals are small to large */
         EigenGSL((const double **) cdsA->CovMat, vlen, tmpevals, evecs, 0);
         //eigensym((const double **) cdsA->CovMat, tmpevals, evecs, vlen);
+//        VecPrint(tmpevals, vlen);
+
+// double marg1, marg2, marg3; // DLT TEST
+// memcpy(evals, tmpevals, vlen*sizeof(double));
+// marg1 = CalcMgLogL(cdsA);
+
+        //tmpevals[0] = phi/2;//ExpXn(phi/2, ndf, vlen);
+
+//printf("phi:%f %f\n", oldphi, tmpevals[0]); fflush(NULL);
 
         phi = 0.0;
         for (i = 0; i < vlen; ++i)
@@ -575,9 +663,27 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
 
         //phi += FLT_MIN;
 
-        InvGammaAdjustEvals(evals, vlen, cnum, tmpevals, phi, nu);
+// DLT TEST
+// stats->hierarch_p1 = 0.5*phi;
+// marg2 = CalcMgLogL(cdsA);
+
+// printf("PhiMargLogL ///////////////////////////////////////////// % e % e % e\n",
+//     marg1, marg2, marg2 - marg1);
+// fflush(NULL); // DLT TEST
+
+        InvGammaAdjustEvals(evals, vlen, cnum, tmpevals, phi, ndf);
         EigenReconSym(cdsA->CovMat, (const double **) evecs, evals, vlen);
 
+// marg3 = CalcMgLogL(cdsA);
+
+// printf("VarMargLogL ///////////////////////////////////////////// % e % e % e\n",
+//        marg2, marg3, marg3 - marg2);
+// fflush(NULL); // DLT TEST
+
+// stats->hierarch_p1 = 0.5*phi;
+// printf("MargLogL ///////////////////////////////////////////// % e\n",
+//         CalcMgLogL(cdsA));
+
         for (i = 0; i < vlen; ++i)
             invevals[i] = 1.0 / evals[i];
 
@@ -593,19 +699,183 @@ InvGammaFitMarginalGSLBrent(CdsArray *cdsA)
         for (i = 0; i < vlen; ++i)
             cdsA->var[i] = cdsA->CovMat[i][i];
 
-        chi2 = chi_sqr_adapt(evals, vlen, 0, &logL, 0.5*phi, 0.5*nu,
+        chi2 = chi_sqr_adapt(evals, vlen, 0, &logL, 0.5*phi, 0.5*ndf,
                              invgamma_pdf, invgamma_lnpdf, invgamma_int);
     }
 
     gsl_min_fminimizer_free(s);
 
+    stats->phi = phi;
     stats->hierarch_p1 = 0.5*phi;
-    stats->hierarch_p2 = 0.5*nu;
+    stats->hierarch_p2 = 0.5*ndf;
     stats->hierarch_chi2 = chi2;
 }
 
 
-/* Finging the root doesn't seem to work */
+/* marginal solution, currently the default (2015-04-06) */
+/* DEFAULT */
+/* Fit of the variances/eigenvalues to an inverse gamma distribution with
+   a fixed c shape parameter.
+   c > 1 -> guarantees that the distribution has a finite mean
+   c > 2 -> finite variance
+   c > 3 -> finite skewness
+   c > 4 -> finite kurtosis
+*/
+void
+InvGammaFitMarginalMLPhi(CdsArray *cdsA) // DLT FIX: not yet modified for missing data (does it need to be?)
+{
+    const int           vlen = cdsA->vlen, cnum = cdsA->cnum;
+    int                 i, iter;
+    double              oldphi = 2.0*stats->hierarch_p1;
+    double              phi, sum;
+    double              chi2 = 0.0, logL;
+    double              ndf = algo->ndf;
+
+    phi = DBL_MAX;
+
+    if (algo->varweight > 0)
+    {
+        double     *svar = cdsA->samplevar;
+        double     *var = cdsA->var;
+//VecPrint(var, vlen);
+        iter = 0;
+        while(fabs(phi - oldphi) > 1e-8)
+        {
+            oldphi = phi;
+            sum = 0.0;
+            for (i = 0; i < vlen; ++i)
+                sum += 1.0 / var[i];
+
+            phi = ndf*vlen/(sum+2.0);
+            //phi = (ndf*vlen - 2.0)/tmpsum;
+
+//             if (iter)
+//                 printf ("%5d %.7f [%.7f, %.7f] %.7f\n", iter, sum, oldphi, phi, oldphi-phi);
+
+            ++iter;
+
+// if (algo->alignment) // DLT TEST
+//  marg2 = CalcMgLogLNu(cdsA);
+// else
+//  marg2 = CalcMgLogL(cdsA);
+
+// printf("PhiMargLogL ///////////////////////////////////////////// % e % e % e\n",
+//     marg1, marg2, marg2 - marg1);
+// fflush(NULL); // DLT TEST
+
+            if (algo->alignment)
+                InvGammaAdjustVarPhiNu(cdsA->cds, var, vlen, cnum, svar, phi, ndf);
+            else
+                InvGammaAdjustVarPhi(var, vlen, cnum, svar, phi, ndf);
+        }
+        //printf("\n");
+// if (algo->alignment) // DLT TEST
+//  marg3 = CalcMgLogLNu(cdsA);
+// else
+//  marg3 = CalcMgLogL(cdsA);
+
+// printf("Var2MargLogL ///////////////////////////////////////////// % e % e % e\n",
+//     marg2, marg3, marg3 - marg2);
+// fflush(NULL); // DLT TEST
+
+        chi2 = chi_sqr_adapt(var, vlen, 0, &logL, 0.5*phi, 0.5*ndf,
+                             invgamma_pdf, invgamma_lnpdf, invgamma_int);
+    }
+    else if (algo->covweight > 0)
+    {
+        double     *evals = cdsA->evals;
+        double     *invevals = cdsA->tmpvecK;
+        double    **evecs = cdsA->tmpmatKK2;
+        double     *tmpevals = cdsA->samplevar;
+        int         i, j;
+
+        /* EigenGSL 0 evals are small to large */
+        EigenGSL((const double **) cdsA->CovMat, vlen, tmpevals, evecs, 0);
+//        VecPrint(tmpevals, vlen);
+
+// double marg1, marg2, marg3; // DLT TEST
+// memcpy(evals, tmpevals, vlen*sizeof(double));
+// marg1 = CalcMgLogL(cdsA);
+
+        //tmpevals[0] = phi/2;//ExpXn(phi/2, ndf, vlen);
+
+//printf("phi:%f %f\n", oldphi, tmpevals[0]); fflush(NULL);
+
+        iter = 0;
+        while(fabs(phi - oldphi) > 1e-8)
+        {
+            oldphi = phi;
+            sum = 0.0;
+            for (i = 0; i < vlen; ++i)
+                sum += 1.0 / evals[i];
+
+            phi = ndf*vlen/(sum+2.0);
+            //phi = (ndf*vlen - 2.0)/tmpsum;
+
+            //if (iter)
+            //printf ("%5d [%.7f, %.7f] %.7f\n", iter, oldphi, phi, oldphi-phi);
+            ++iter;
+
+    // DLT TEST
+    // stats->hierarch_p1 = 0.5*phi;
+    // marg2 = CalcMgLogL(cdsA);
+
+    // printf("PhiMargLogL ///////////////////////////////////////////// % e % e % e\n",
+    //     marg1, marg2, marg2 - marg1);
+    // fflush(NULL); // DLT TEST
+
+            InvGammaAdjustEvals(evals, vlen, cnum, tmpevals, phi, ndf);
+            EigenReconSym(cdsA->CovMat, (const double **) evecs, evals, vlen);
+        }
+// marg3 = CalcMgLogL(cdsA);
+
+// printf("VarMargLogL ///////////////////////////////////////////// % e % e % e\n",
+//        marg2, marg3, marg3 - marg2);
+// fflush(NULL); // DLT TEST
+
+// stats->hierarch_p1 = 0.5*phi;
+// printf("MargLogL ///////////////////////////////////////////// % e\n",
+//         CalcMgLogL(cdsA));
+
+        for (i = 0; i < vlen; ++i)
+            invevals[i] = 1.0 / evals[i];
+
+        EigenReconSym(cdsA->WtMat, (const double **) evecs, invevals, vlen);
+
+        // Normalize the inverse cov mat (for translations); also shouldn't matter for rotations
+//         double wtsum = 0.0;
+//         for (i = 0; i < vlen; ++i)
+//             for (j = 0; j < vlen; ++j)
+//                 wtsum += cdsA->WtMat[i][j];
+//
+//         for (i = 0; i < vlen; ++i)
+//             for (j = 0; j < vlen; ++j)
+//                 cdsA->WtMat[i][j] /= wtsum;
+
+        if (algo->rounds < 3)
+        {
+            for (i = 0; i < vlen; ++i)
+                for (j = 0; j < i; ++j)
+                    cdsA->WtMat[i][j] = cdsA->WtMat[j][i] = 0.0;
+        }
+
+        for (i = 0; i < vlen; ++i)
+            cdsA->var[i] = cdsA->CovMat[i][i];
+
+//VecPrint(evals, vlen);
+
+        //chi2 = chi_sqr_adapt(cdsA->var, vlen, 0, &logL, 0.5*phi, 0.5*ndf,
+        //                     invgamma_pdf, invgamma_lnpdf, invgamma_int);
+    }
+
+    stats->phi = phi;
+    stats->hierarch_p1 = 0.5*phi;
+    stats->hierarch_p2 = 0.5*ndf;
+    stats->hierarch_chi2 = chi2;
+}
+
+
+/* Finding the root doesn't seem to work */
 #include <gsl/gsl_roots.h>
 void
 InvGammaFitMarginalGSLRoot(CdsArray *cdsA)
@@ -626,17 +896,17 @@ InvGammaFitMarginalGSLRoot(CdsArray *cdsA)
 
     if (algo->varweight > 0)
     {
-        double     *var3N = cdsA->samplevar3N;
+        double     *var = cdsA->samplevar;
 
         nu = 1.0;
 
-        CalcVar(cdsA);
+        CalcSampleVar(cdsA);
 
         phi = 0.0;
         for (i = 0; i < vlen; ++i)
-            phi +=  var3N[i];
+            phi += var[i];
 
-        phi /= 3.0*cnum*vlen;
+        phi /= vlen;
         phi0 = phi;
 
         FDF.f = &LogPrPhi_f;
@@ -668,7 +938,7 @@ InvGammaFitMarginalGSLRoot(CdsArray *cdsA)
         }
         while(status == GSL_CONTINUE && iter < max_iter);
 
-        InvGammaAdjustVar3N(cdsA->var, vlen, cnum, var3N, phi, nu);
+        InvGammaAdjustVarPhi(cdsA->var, vlen, cnum, var, phi, nu);
 
         chi2 = chi_sqr_adapt(cdsA->var, vlen, 0, &logL, 0.5*phi, 0.5*nu,
                              invgamma_pdf, invgamma_lnpdf, invgamma_int);
@@ -780,7 +1050,7 @@ InvGammaFitEvalsEMFixedC(CdsArray *cdsA, const double c, int iterate)
         for (j = 0; j < missing; ++j)
             variance[missi[j]] = expinvx;
 
-        free(missi);
+        MyFree(missi);
         chi2 = chi_sqr_adapt(variance, vlen, 0, &logL, b, c, invgamma_pdf, invgamma_lnpdf, invgamma_int);
     }
     else if (algo->covweight > 0)
@@ -904,7 +1174,7 @@ InvGammaFitEvalsEMFixedC(CdsArray *cdsA, const double c, int iterate)
     stats->hierarch_p2 = c;
     stats->hierarch_chi2 = chi2;
 
-    free(newvar);
+    MyFree(newvar);
 }
 
 
@@ -1107,7 +1377,7 @@ InvGammaFitEvalsML(CdsArray *cdsA, int iterate)
     stats->hierarch_p2 = c;
     stats->hierarch_chi2 = chi2;
 
-    free(newvar);
+    MyFree(newvar);
 }
 
 
@@ -1274,6 +1544,6 @@ InvGammaFitEvals(CdsArray *cdsA, int iterate)
     stats->hierarch_p2 = c;
     stats->hierarch_chi2 = chi2;
 
-    free(newvar);
+    MyFree(newvar);
 }
 
