@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -41,36 +40,60 @@
 
 
 static int
-range_selxn(int index, int *lower, int *upper, int range_num);
+RangeSelxn(int index, int *lower, int *upper, int range_num);
 
 static int
-atom_selxn(char *name, int mode);
+AtomSelxn(char *name, int mode);
 
 static const char atoms0[] = ":CA :C1*:C1'";
 static const char atoms1[] = ":N  :C  :O  :CA :";
 
 
-static int
-atom_selxn(char *name, int mode)
-{
-    switch(mode)
-    {
-        case 0: /* "CA :P  " */
-            if (strstr(atoms0, name) != NULL)
-                return (1);
-            break;
-        case 1: /* "N  :C  :O  :CA :" */
-            if (strstr(atoms1, name) != NULL)
-                return (1);
-            break;
-    }
 
-    return (0);
+static int
+AltLocSelxn(int altLoc)
+{
+    if (altLoc == ' ' ||
+        altLoc == 'A' ||
+        altLoc == '1')
+        return(1);
+    else
+        return(0);
 }
 
 
 static int
-range_selxn(int index, int *lower, int *upper, int range_num)
+RecordSelxn(char *record)
+{
+    if (strncmp(record, "ATOM  ", 6) == 0 ||
+        strncmp(record, "HETATM", 6) == 0)
+        return(1);
+    else
+        return(0);
+}
+
+
+static int
+AtomSelxn(char *name, int mode)
+{
+    switch(mode)
+    {
+        case 0: /* "CA :P  " */
+            if (strstr(atoms0, name))
+                return(1);
+            break;
+        case 1: /* "N  :C  :O  :CA :" */
+            if (strstr(atoms1, name))
+                return(1);
+            break;
+    }
+
+    return(0);
+}
+
+
+static int
+RangeSelxn(int index, int *lower, int *upper, int range_num)
 {
     int             i;
 
@@ -84,33 +107,6 @@ range_selxn(int index, int *lower, int *upper, int range_num)
 }
 
 
-/*
-char
-aa3toaa1(char *aa3_s)
-{
-    int            index_i;
-
-    index_i = (strstr(&aa3[0], aa3_s) - &aa3[0]) / 3;
-
-    return (aa1[index_i]);
-}
-
-
-void
-aa1toaa3(char aa1_c, char *aa3_s)
-{
-    int            i, index_i;
-
-    index_i = (strchr(&aa1[0], aa1_c) - &aa1[0]) * 3;
-
-    for (i = 0; i < 3; ++i)
-        aa3_s[i] = aa3[index_i + i];
-
-    aa3_s[3] = '\0';
-}
-*/
-
-
 static void
 VerifyAlignmentVsPDBs(PDBCdsArray *pdbA, CdsArray *baseA, MSA *msa,
                       const int atomsel, int *map)
@@ -130,8 +126,8 @@ VerifyAlignmentVsPDBs(PDBCdsArray *pdbA, CdsArray *baseA, MSA *msa,
 
         aalen = 0;
         for (j = 0; j < pdbA->cds[i]->vlen; ++j)
-            if (atom_selxn(pdbA->cds[i]->name[j], atomsel) &&
-                (pdbA->cds[i]->altLoc[j] == ' ' || pdbA->cds[i]->altLoc[j] == 'A'))
+            if (AtomSelxn(pdbA->cds[i]->name[j], atomsel) &&
+                AltLocSelxn(pdbA->cds[i]->altLoc[j]))
                 ++aalen;
 
         baseA->cds[i]->aalen = aalen;
@@ -141,7 +137,7 @@ VerifyAlignmentVsPDBs(PDBCdsArray *pdbA, CdsArray *baseA, MSA *msa,
             fprintf(stderr, "\n\n  ERROR1122: PDB file '%s' and sequence '%s' in '%s'",
                     pdbA->cds[i]->filename, msa->name[map[i]], msa->filename);
             fprintf(stderr, "\n            have different lengths (%d vs %d)\n\n",
-                   aalen, len);
+                    aalen, len);
             PrintTheseusTag();
             exit(EXIT_FAILURE);
         }
@@ -358,8 +354,8 @@ static Seq2PDB
 }
 
 
-/* Constructs an integer vector that maps a Cds structure to a sequence name in the alignment */
-/* First matches PDB filename to the PDB filename given in the mapfile.
+/* Constructs an integer vector that maps a Cds structure to a sequence name in the alignment.
+   First matches PDB filename to the PDB filename given in the mapfile.
    Every filename in the mapfile is associated with a sequence name of the same index.
    Then the fxn matches the sequence name in the mapfile with a sequence name in the alignment.
    Given these two matchings and the mapfile, the map[] vector can be constructed.
@@ -369,12 +365,34 @@ static Seq2PDB
 static void
 GetSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 {
-    int             i, j, k, str1len, str2len, str3len, str4len;
+    int             i, j, k;
     const int       cnum = pdbA->cnum;
-    char           *filename_root = NULL;
-    char           *msaname_root = NULL;
-    char           *mapseqname_root = NULL;
-    char           *mappdbname_root = NULL;
+    char          **filename_root = NULL;
+    char          **msaname_root = NULL;
+    char          **mapseqname_root = NULL;
+    char          **mappdbname_root = NULL;
+
+
+    filename_root   = malloc(cnum * sizeof(char *));
+    msaname_root    = malloc(msa->seqnum * sizeof(char *));
+    mapseqname_root = malloc(seq2pdb->seqnum * sizeof(char *));
+    mappdbname_root = malloc(seq2pdb->seqnum * sizeof(char *));
+
+    for (i = 0; i < cnum; ++i)
+    {
+        filename_root[i] = getroot(pdbA->cds[i]->filename);
+    }
+
+    for (j = 0; j < seq2pdb->seqnum; ++j)
+    {
+        mappdbname_root[j] = getroot(seq2pdb->pdbfile_name[j]);
+        mapseqname_root[j] = getroot(seq2pdb->seqname[j]);
+    }
+
+    for (k = 0; k < msa->seqnum; ++k)
+    {
+        msaname_root[k] = getroot(msa->name[k]);
+    }
 
     /* First set all flags in the MSA to 1, and later reset to 0 if the sequences have a
        corresponding PDB file.
@@ -386,41 +404,24 @@ GetSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
     {
         for (j = 0; j < seq2pdb->seqnum; ++j)
         {
-            filename_root = getroot(pdbA->cds[i]->filename);
-            mappdbname_root = getroot(seq2pdb->pdbfile_name[j]);
-            mapseqname_root = getroot(seq2pdb->seqname[j]);
-
-            str1len = strlen(filename_root);
-            str2len = strlen(mappdbname_root);
-            str3len = strlen(mapseqname_root);
-
             //printf("\ni:%3d j:%3d -- %s %s", i, j, filename_root, mappdbname_root);
 
-            if ((strncmp(filename_root, mappdbname_root, str1len) == 0) &&
-                (str1len == str2len))
+            if (strncmp(filename_root[i], mappdbname_root[j], FILENAME_MAX) == 0)
             {
                 for (k = 0; k < msa->seqnum; ++k)
                 {
-                    msaname_root = getroot(msa->name[k]);
-
-                    str4len = strlen(msaname_root);
-
                     //printf("\ni:%3d j:%3d k:%3d -- %s %s %s", i, j, k, filename_root, mapseqname_root, msaname_root);
 
-                    if ((strncmp(mapseqname_root, msaname_root, str3len) == 0) &&
-                        (str3len == str4len)) /* now k (alignment) and i (pdb cds) match */
+                    if (strncmp(mapseqname_root[j], msaname_root[k], FILENAME_MAX) == 0) /* now k (alignment) and i (pdb cds) match */
                     {
                         //printf("\nMATCH");
                         pdbA->seq2pdb->map[i] = k;
                         msa->flag[k] = 0; /* reset msa flag to 0 since this structure exists */
 
-                        free(msaname_root);
-
                         break;
                     }
-                    fflush(NULL);
 
-                    free(msaname_root);
+                    fflush(NULL);
                 }
 
                 if (k == msa->seqnum)
@@ -432,16 +433,8 @@ GetSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
                     exit(EXIT_FAILURE);
                 }
 
-                free(filename_root);
-                free(mappdbname_root);
-                free(mapseqname_root);
-
                 break;
             }
-
-            free(filename_root);
-            free(mappdbname_root);
-            free(mapseqname_root);
         }
 
         if (j == seq2pdb->seqnum)
@@ -453,6 +446,27 @@ GetSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
             exit(EXIT_FAILURE);
         }
     }
+
+    for (i = 0; i < cnum; ++i)
+    {
+        MyFree(filename_root[i]);
+    }
+
+    for (j = 0; j < seq2pdb->seqnum; ++j)
+    {
+        MyFree(mappdbname_root[j]);
+        MyFree(mapseqname_root[j]);
+    }
+
+    for (k = 0; k < msa->seqnum; ++k)
+    {
+        MyFree(msaname_root[k]);
+    }
+
+    MyFree(filename_root);
+    MyFree(msaname_root);
+    MyFree(mapseqname_root);
+    MyFree(mappdbname_root);
 }
 
 
@@ -461,32 +475,42 @@ GetSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 static void
 DefaultSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 {
-    int             i, j;
+    int             i, j, same;
     const int       cnum = pdbA->cnum;
-    char            filename_root[FILENAME_MAX], msaname_root[FILENAME_MAX];
+    const int       seqnum = msa->seqnum;
+    char          **filename_root = NULL, **msaname_root = NULL;
+
+    filename_root = malloc(cnum   * sizeof(char *));
+    msaname_root  = malloc(seqnum * sizeof(char *));
 
     /* First set all flags in the MSA to 1, and later reset to 0 if the sequences have a
        corresponding PDB file.
        This allows for sequences in the MSA that aren't being superpositioned. */
-    for (i = 0; i < msa->seqnum; ++i)
+    for (i = 0; i < seqnum; ++i)
         msa->flag[i] = 1;
 
     for (i = 0; i < cnum; ++i)
-    {
-        for (j = 0; j < msa->seqnum; ++j)
-        {
-            strncpy(filename_root, getroot(pdbA->cds[i]->filename), strlen(pdbA->cds[i]->filename)+1);
-            strncpy(msaname_root, getroot(msa->name[j]), strlen(msa->name[j])+1);
+        filename_root[i] = getroot(pdbA->cds[i]->filename);
 
-            if (strncmp(filename_root, msaname_root, FILENAME_MAX) == 0)
+    for (j = 0; j < seqnum; ++j)
+        msaname_root[j] = getroot(msa->name[j]);
+
+    for (i = 0; i < cnum; ++i)
+    {
+        for (j = 0; j < seqnum; ++j)
+        {
+            same = strncmp(filename_root[i], msaname_root[j], FILENAME_MAX);
+
+            if (same == 0)
             {
                 seq2pdb->map[i] = j; /* now j (alignment) and i (pdb cds) match */
                 msa->flag[j] = 0; /* reset msa flag to 0 since this structure exists */
+
                 break;
             }
         }
 
-        if (j == msa->seqnum)
+        if (j == seqnum)
         {
             fprintf(stderr,
                     "\n  ERROR690: PDB file #%d (%s) has no corresponding sequence in the alignment\n",
@@ -495,11 +519,20 @@ DefaultSeq2PDBMap(PDBCdsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
             exit(EXIT_FAILURE);
         }
     }
+
+    for (j = 0; j < seqnum; ++j)
+        MyFree(msaname_root[j]);
+
+    for (i = 0; i < cnum; ++i)
+        MyFree(filename_root[i]);
+
+    MyFree(filename_root);
+    MyFree(msaname_root);
 }
 
 
 /* check for singletons, i.e. columns with only one non-gap residue */
-static void
+void
 GetSingletons(int *singletons, MSA *msa)
 {
     int             i, j, count;
@@ -553,9 +586,9 @@ GetUbiqs(int *ubiqs, MSA *msa)
 
 static void
 GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
-                     Seq2PDB *seq2pdb, MSA *msa,
-                     const int vlen, const int alignlen,
-                     int *singletons)
+                  Seq2PDB *seq2pdb, MSA *msa,
+                  const int vlen, const int alignlen,
+                  int *singletons)
 {
     int             j, k, m, n, p, q;
     const int       cnum = pdbA->cnum;
@@ -563,12 +596,19 @@ GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
     int            *upper = pdbA->upper; /* arrays of ints holding the upper and lower range bounds */
     int            *lower = pdbA->lower;
     int             range_num = pdbA->range_num;
+    char            altLoc_m;
+    char           *name_m = NULL;
+    const PDBCds   *pdbj = NULL;
+    Cds            *cdsj = NULL;
 
     for (j = 0; j < cnum; ++j)
     {
+        pdbj = (const PDBCds *) pdbA->cds[j];
+        cdsj = baseA->cds[j];
+
         k = map[j];
         m = n = p = 0;
-        while(m < pdbA->cds[j]->vlen && n < vlen  && p < alignlen)
+        while(m < pdbj->vlen && n < vlen  && p < alignlen)
         {
             /* m = PDB length */
             /* n = baseA cds length */
@@ -577,47 +617,48 @@ GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
             /* j = pdb and baseA cds index */
 
 /*                             printf("\n1 n:%d(%d) atomname:pdbA->cds[%d]->name[%d] %s", */
-/*                                    n, alignlen, j, m, pdbA->cds[j]->name[m]); */
+/*                                    n, alignlen, j, m, pdbj->name[m]); */
 /*                             fflush(NULL); */
 /* if (j == cnum - 1) */
-/* printf("\naltLoc:%c", pdbA->cds[j]->altLoc[m]); */
-            if (atom_selxn(pdbA->cds[j]->name[m], algo->atoms) &&
-                (pdbA->cds[j]->altLoc[m] == ' ' || pdbA->cds[j]->altLoc[m] == 'A'))
+/* printf("\naltLoc:%c", pdbj->altLoc[m]); */
+
+            name_m = pdbj->name[m];
+            altLoc_m = pdbj->altLoc[m];
+
+            if (AtomSelxn(name_m, algo->atoms) && AltLocSelxn(altLoc_m))
             {
 /*                                 printf("\n1 m:%d(%d) msa->seq[%d(%d)][%d(%d)] = %c", */
-/*                                        m, pdbA->cds[j]->vlen, k, cnum, n, alignlen, msa->seq[k][n]); */
+/*                                        m, pdbj->vlen, k, cnum, n, alignlen, msa->seq[k][n]); */
 /*                                 fflush(NULL); */
-                if (range_selxn(p, lower, upper, range_num) - algo->revsel && /* in-range, or out-of-range if revsel == 1 */
+                if (RangeSelxn(p, lower, upper, range_num) - algo->revsel && /* in-range, or out-of-range if revsel == 1 */
                     singletons[p] == 0) /* not a singleton */
                 {
                     if (msa->seq[k][p] != '-') /* not a gap */
                     {
-                        strncpy(baseA->cds[j]->resName[n], pdbA->cds[j]->resName[m], 3);
-                        baseA->cds[j]->chainID[n] = pdbA->cds[j]->chainID[m];
-                        baseA->cds[j]->resSeq[n]  = pdbA->cds[j]->resSeq[m];
-                        baseA->cds[j]->x[n]       = pdbA->cds[j]->x[m];
-                        baseA->cds[j]->y[n]       = pdbA->cds[j]->y[m];
-                        baseA->cds[j]->z[n]       = pdbA->cds[j]->z[m];
-                        baseA->cds[j]->o[n]       = pdbA->cds[j]->occupancy[m];
-                        baseA->cds[j]->b[n]       = pdbA->cds[j]->tempFactor[m];
-                        baseA->cds[j]->nu[n]       = 1;
-                        baseA->cds[j]->mu[n]       = 0;
+                        strncpy(cdsj->resName[n], pdbj->resName[m], 3);
+                        cdsj->chainID[n] = pdbj->chainID[m];
+                        cdsj->resSeq[n]  = pdbj->resSeq[m];
+                        cdsj->x[n]       = pdbj->x[m];
+                        cdsj->y[n]       = pdbj->y[m];
+                        cdsj->z[n]       = pdbj->z[m];
+                        cdsj->o[n]       = pdbj->occupancy[m];
+                        cdsj->b[n]       = pdbj->tempFactor[m];
+                        cdsj->nu[n]      = 1;
 
                         ++m;
                         ++n;
                     }
                     else /* is a gap */
                     {
-                        strncpy(baseA->cds[j]->resName[n], "GAP", 3);
-                        baseA->cds[j]->chainID[n] = pdbA->cds[j]->chainID[m];
-                        baseA->cds[j]->resSeq[n]  = 0;
-                        baseA->cds[j]->x[n]       = 0.0;
-                        baseA->cds[j]->y[n]       = 0.0;
-                        baseA->cds[j]->z[n]       = 0.0;
-                        baseA->cds[j]->o[n]       = 0.0;
-                        baseA->cds[j]->b[n]       = 99.99;
-                        baseA->cds[j]->nu[n]       = 0;
-                        baseA->cds[j]->mu[n]       = 1;
+                        strncpy(cdsj->resName[n], "GAP", 3);
+                        cdsj->chainID[n] = pdbj->chainID[m];
+                        cdsj->resSeq[n]  = 0;
+                        cdsj->x[n]       = 0.0;
+                        cdsj->y[n]       = 0.0;
+                        cdsj->z[n]       = 0.0;
+                        cdsj->o[n]       = 0.0;
+                        cdsj->b[n]       = 99.99;
+                        cdsj->nu[n]      = 0;
 
                         ++n;
                     }
@@ -634,7 +675,7 @@ GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
                 ++p;
 /*                          if (j == cnum-1) */
 /*                                 printf("\n2 m:%d(%d) msa->seq[%d(%d)][%d(%d)] = %c", */
-/*                                        m, pdbA->cds[j]->vlen, k, cnum, n, alignlen, msa->seq[k][n]); */
+/*                                        m, pdbj->vlen, k, cnum, n, alignlen, msa->seq[k][n]); */
 /*                                 fflush(NULL); */
             }
             else /* not the proper atom slxn */
@@ -643,7 +684,7 @@ GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
             }
 /*                      if (j == cnum-1) */
 /*                             printf("\n2 n:%d(%d) atomname:pdbA->cds[%d]->name[%d] %s", */
-/*                                    n, alignlen, j, m, pdbA->cds[j]->name[m]); */
+/*                                    n, alignlen, j, m, pdbj->name[m]); */
 /*                             fflush(NULL); */
         }
         /*******************************************************************************************/
@@ -651,20 +692,19 @@ GetCdsFrAlignment(PDBCdsArray *pdbA, CdsArray *baseA,
         if (n < vlen && m != 0 && n != 0)
         {
 /*          printf("\nHere: m:%4d n:%4d p:%4d j:%4d -- %4d %4d %4d %4d", */
-/*                  m, n, p, j, pdbA->cds[j]->vlen, vlen, alignlen, cnum); */
+/*                  m, n, p, j, pdbj->vlen, vlen, alignlen, cnum); */
 
             for (q = n; q < vlen; ++q)
             {
-                strncpy(baseA->cds[j]->resName[q], "GAP", 3);
-                baseA->cds[j]->chainID[q] = pdbA->cds[j]->chainID[m-1];
-                baseA->cds[j]->resSeq[q]  = 0;
-                baseA->cds[j]->x[q]       = 0.0;
-                baseA->cds[j]->y[q]       = 0.0;
-                baseA->cds[j]->z[q]       = 0.0;
-                baseA->cds[j]->o[q]       = 0.0;
-                baseA->cds[j]->b[q]       = 66.66;
-                baseA->cds[j]->nu[n]       = 0;
-                baseA->cds[j]->mu[n]       = 1;
+                strncpy(cdsj->resName[q], "GAP", 3);
+                cdsj->chainID[q] = pdbj->chainID[m-1];
+                cdsj->resSeq[q]  = 0;
+                cdsj->x[q]       = 0.0;
+                cdsj->y[q]       = 0.0;
+                cdsj->z[q]       = 0.0;
+                cdsj->o[q]       = 0.0;
+                cdsj->b[q]       = 66.66;
+                cdsj->nu[n]      = 0;
             }
         }
     }
@@ -679,8 +719,9 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
     char          **endptr = NULL;
     char          **selections = NULL; /* an array of pdbA->range_num strings to hold each range selection */
     char            delims[] = ":";
+    int            *upper = NULL, *lower = NULL; /* an array of ints holding the upper and lower range bounds */
 
-    if (selection != NULL)
+    if (selection)
     {
         selxn_len = strlen(selection);
 
@@ -692,9 +733,9 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
         }
 
         selections = (char **) calloc(pdbA->range_num, sizeof(char *));
-        pdbA->lower = (int *)  calloc(pdbA->range_num, sizeof(int));
-        pdbA->upper = (int *)  calloc(pdbA->range_num, sizeof(int));
-        if (selections == NULL || pdbA->lower == NULL || pdbA->upper == NULL)
+        pdbA->lower = lower = (int *)  calloc(pdbA->range_num, sizeof(int));
+        pdbA->upper = upper = (int *)  calloc(pdbA->range_num, sizeof(int));
+        if (selections == NULL || lower == NULL || upper == NULL)
         {
             perror("\n  ERROR");
             fprintf(stderr, " ERROR: could not allocate memory for selections in GetCdsSelection(). \n\n");
@@ -733,7 +774,7 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
 
             if (isdigit(selections[j][i]))
             {
-                pdbA->lower[j] = (int) strtol(&selections[j][i], endptr, 10) - 1;
+                lower[j] = (int) strtol(&selections[j][i], endptr, 10) - 1;
 
                 while(selections[j][i] != '-' && i < selxn_len)
                     ++i;
@@ -743,7 +784,7 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
                     ++i;
 
                 if (isdigit(selections[j][i]))
-                    pdbA->upper[j] = (int) strtol(&selections[j][i], endptr, 10) - 1;
+                    upper[j] = (int) strtol(&selections[j][i], endptr, 10) - 1;
                 else
                 {
                     fprintf(stderr, "\n\n  ERROR987: one of the column selections has no upper bound. \n\n");
@@ -752,18 +793,18 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
                     exit(EXIT_FAILURE);
                 }
 
-                if (pdbA->upper[j] >= alignlen)
+                if (upper[j] >= alignlen)
                 {
                     fprintf(stderr, "\n\n  ERROR988: one of the column selections is out of bounds for the alignment. \n");
-                    fprintf(stderr, "  ERROR988: upper limit %d = %d; alignment length = %d \n\n", j, pdbA->upper[j], alignlen);
+                    fprintf(stderr, "  ERROR988: upper limit %d = %d; alignment length = %d \n\n", j, upper[j], alignlen);
                     PrintTheseusTag();
                     exit(EXIT_FAILURE);
                 }
             }
             else
             {
-                pdbA->lower[j] = 0;
-                pdbA->upper[j] = alignlen - 1;
+                lower[j] = 0;
+                upper[j] = alignlen - 1;
             }
         }
     }
@@ -771,10 +812,10 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
     {
         pdbA->range_num = 1;
         selections = (char **) calloc(1, sizeof(char *));
-        pdbA->lower = (int *) calloc(1, sizeof(int));
-        pdbA->upper = (int *) calloc(1, sizeof(int));
+        pdbA->lower = lower = (int *) calloc(1, sizeof(int));
+        pdbA->upper = upper = (int *) calloc(1, sizeof(int));
         selections[0] = (char *) calloc(128, sizeof(char));
-        if (selections == NULL || pdbA->lower == NULL || pdbA->upper == NULL || selections[0] == NULL)
+        if (selections == NULL || lower == NULL || upper == NULL || selections[0] == NULL)
         {
             perror("\n  ERROR");
             fprintf(stderr, " ERROR989: could not allocate memory for selections in GetCdsSelection(). \n\n");
@@ -782,27 +823,27 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
             exit(EXIT_FAILURE);
         }
 
-        pdbA->lower[0] = 0;
-        pdbA->upper[0] = alignlen - 1;
+        lower[0] = 0;
+        upper[0] = alignlen - 1;
     }
 
     if (revsel == 0)
     {
         vlen = 0;
         for (j = 0; j < pdbA->range_num; ++j)
-            vlen += (pdbA->upper[j] - pdbA->lower[j] + 1);
+            vlen += (upper[j] - lower[j] + 1);
 
         /* don't count singletons that are in selected ranges */
         for (j = 0; j < pdbA->range_num; ++j)
             for (i = 0; i < alignlen; ++i)
-                if (i >= pdbA->lower[j] && i <= pdbA->upper[j] && singletons[i])
+                if (i >= lower[j] && i <= upper[j] && singletons[i])
                     vlen--;
     }
     else
     {
         vlen = alignlen;
         for (j = 0; j < pdbA->range_num; ++j)
-            vlen -= (pdbA->upper[j] - pdbA->lower[j] + 1);
+            vlen -= (upper[j] - lower[j] + 1);
 
         singleton_cnt = 0;
         for (i = 0; i < alignlen; ++i)
@@ -812,7 +853,7 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
         /* don't count singletons that are in (un)selected ranges */
         for (j = 0; j < pdbA->range_num; ++j)
             for (i = 0; i < alignlen; ++i)
-                if (i >= pdbA->lower[j] && i <= pdbA->upper[j] && singletons[i])
+                if (i >= lower[j] && i <= upper[j] && singletons[i])
                     singleton_cnt--;
 
         vlen -= singleton_cnt;
@@ -820,84 +861,42 @@ ParseSelxns(PDBCdsArray *pdbA, char *selection, const int alignlen, int *singlet
 
     if (vlen > alignlen)
     {
-        fprintf(stderr, "\n  ERROR663: the number of selected alignment columns (%d) exceeds the maximum length of the alignment (%d) \n\n", vlen, alignlen);
+        fprintf(stderr,
+                "\n  ERROR663: selected columns (%d) exceed length of alignment (%d) \n\n",
+                vlen, alignlen);
         PrintTheseusTag();
         exit(EXIT_FAILURE);
     }
 
     for (i = 0; i < pdbA->range_num; ++i)
-        free(selections[i]);
+        MyFree(selections[i]);
 
-    free(selections);
+    MyFree(selections);
 
     return(vlen);
 }
 
 
+/* extract all coords, ignoring singleton status
+  (usually we don't extract singletons, as they are irrelevant to superposition) */
 void
-Align2MSA(PDBCdsArray *pdbA, CdsArray *baseA, char *msafile_name, char *mapfile_name)
+Align2MSA_singletons(PDBCdsArray *pdbA, CdsArray *baseA, MSA *msa)
 {
     int             i, alnlen, vlen;
     const int       cnum = pdbA->cnum;
-    MSA            *msa = NULL;
-    //int            *singletons = NULL;
 
-    if (mapfile_name == NULL)
-    {
-        pdbA->seq2pdb = Seq2pdbInit();
-        Seq2pdbAlloc(pdbA->seq2pdb, cnum);
-    }
-    else
-    {
-        pdbA->seq2pdb = GetMapFile(mapfile_name);
-    }
-
-    pdbA->seq2pdb->msa = msa = getmsa(msafile_name);
-
-//    MSAprint(msa);
-/*     printf("\n%d", msa->seqlen); */
-/*     fflush(NULL); */
-
-    if (msa->seqnum < pdbA->cnum)
-    {
-        fprintf(stderr,
-                "\n  ERROR682: # alignment sequences (%d) < # cds (%d)\n",
-                msa->seqnum, pdbA->cnum);
-        PrintTheseusTag();
-        exit(EXIT_FAILURE);
-    }
+    pdbA->seq2pdb = Seq2pdbInit();
+    Seq2pdbAlloc(pdbA->seq2pdb, cnum);
 
     alnlen = msa->seqlen;
-    pdbA->seq2pdb->singletons = malloc(alnlen * sizeof(int));
+    pdbA->seq2pdb->singletons = calloc(alnlen, sizeof(int));
 
-    if (mapfile_name != NULL)
-        GetSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
-    else
-        DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+    DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
 
-    GetSingletons(pdbA->seq2pdb->singletons, msa);
-    if (algo->missing)
-        GetUbiqs(pdbA->seq2pdb->singletons, msa);
-//    for (i=0;i<alnlen;++i)
-//        printf("singleton[%3d]:%d\n", i, singletons[i]);
-    vlen = ParseSelxns(pdbA, algo->selection, alnlen, pdbA->seq2pdb->singletons, algo->revsel);
+//    GetSingletons(pdbA->seq2pdb->singletons, msa);
+//    vlen = ParseSelxns(pdbA, algo->selection, alnlen, pdbA->seq2pdb->singletons, algo->revsel);
 
-/*     printf("\n%d %d %d", msa->seqlen, alnlen, vlen); */
-
-    /* check for singletons, i.e. columns with only one non-gap residue */
-/*     for (i = 0; i < alnlen; ++i) */
-/*     { */
-/*         count = 0; */
-/*         for (j = 0; j < cnum; ++j) */
-/*         { */
-/*             if (msa->seq[j][i] != '-') */
-/*                 ++count; */
-/*         } */
-/* //printf("\ncol:%d count:%d", i, count); */
-/*         if (count == 1) */
-/*             --vlen; */
-/*     } */
-//    printf("\n%d %d %d\n", msa->seqlen, alnlen, vlen);
+    vlen = alnlen;
 
     /* allocate a CdsArray based on this alignment */
     if (algo->atoms == 0)
@@ -917,114 +916,184 @@ Align2MSA(PDBCdsArray *pdbA, CdsArray *baseA, char *msafile_name, char *mapfile_
 
     VerifyAlignmentVsPDBs(pdbA, baseA, msa, algo->atoms, pdbA->seq2pdb->map);
     GetCdsFrAlignment(pdbA, baseA, pdbA->seq2pdb, msa, vlen, alnlen, pdbA->seq2pdb->singletons);
-
-//    free(singletons);
+    GetSingletons(pdbA->seq2pdb->singletons, msa);
 }
 
 
-double
-CalcRotationsNu(CdsArray *cdsA)
+/* Only difference from regular Align2MSA, here we are handed an explicit msa structure
+   rather than a msa filename to read */
+void
+Align2MSA_crush(PDBCdsArray *pdbA, CdsArray *baseA, MSA *msa)
 {
-    Cds           **cds = cdsA->cds;
-    Cds            *avecds = cdsA->avecds;
-    const double   *wts = (const double *) cdsA->w;
-    double          deviation = 0.0, deviation_sum = 0.0;
-    int             i;
+    int             i, alnlen, vlen;
+    const int       cnum = pdbA->cnum;
 
-    for (i = 0; i < cdsA->cnum; ++i)
+    pdbA->seq2pdb = Seq2pdbInit();
+    Seq2pdbAlloc(pdbA->seq2pdb, cnum);
+
+    pdbA->seq2pdb->msa = msa;
+    MSAdelgap(msa);
+
+//     MSAprint(msa);
+//     printf("\nseqlen: %d\n", msa->seqlen);
+//     printf("\nseqnum: %d\n", msa->seqnum);
+//     fflush(NULL);
+
+    if (msa->seqnum < pdbA->cnum)
     {
-        if(algo->covweight)
-        {
-                deviation = ProcGSLSVDCovNu(cds[i], avecds, cds[i]->matrix,
-                                                (const double **) cdsA->WtMat,
-                                                cdsA->tmpmat3a,
-                                                cdsA->tmpmat3b,
-                                                cdsA->tmpmat3c,
-                                                cdsA->tmpvec3a);
-        }
-        else if(algo->varweight || algo->leastsquares)
-        {
-                deviation = ProcGSLSVDNu(cds[i], avecds,
-                                             cds[i]->matrix,
-                                             wts,
-                                             cdsA->tmpmat3a,
-                                             cdsA->tmpmat3b,
-                                             cdsA->tmpmat3c,
-                                             cdsA->tmpvec3a);
-        }
-
-        cds[i]->wRMSD_from_mean = sqrt(deviation / cdsA->vlen);
-        deviation_sum += deviation;
-
-        if (algo->verbose)
-        {
-            /* rmsd from mean would usually need a 2 in denom,\
-               but this is already deviation from mean,
-               since structure #2 is the average structure */
-            printf("CalcRotationsNu:%5d %8.3f %13.3f \n",
-                   i+1,
-                   cds[i]->wRMSD_from_mean,
-                   deviation);
-        }
+        fprintf(stderr,
+                "\n  ERROR682: # alignment sequences (%d) < # cds (%d)\n",
+                msa->seqnum, pdbA->cnum);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
     }
 
-    return(deviation_sum);
+    alnlen = msa->seqlen;
+    pdbA->seq2pdb->singletons = calloc(alnlen, sizeof(int));
+
+    DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+
+    GetSingletons(pdbA->seq2pdb->singletons, msa);
+
+    vlen = ParseSelxns(pdbA, algo->selection, alnlen, pdbA->seq2pdb->singletons, algo->revsel);
+
+    /* allocate a CdsArray based on this alignment */
+    if (algo->atoms == 0)
+    {
+        CdsArrayAlloc(baseA, cnum, vlen);
+    }
+    else
+    {
+        fprintf(stderr,
+                "\n  ERROR683: atom selection must be CAs for superpositioning with a sequence alignment\n");
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    for (i = 0; i < cnum; ++i)
+        strncpy(baseA->cds[i]->filename, pdbA->cds[i]->filename, FILENAME_MAX - 1);
+
+    VerifyAlignmentVsPDBs(pdbA, baseA, msa, algo->atoms, pdbA->seq2pdb->map);
+    GetCdsFrAlignment(pdbA, baseA, pdbA->seq2pdb, msa, vlen, alnlen, pdbA->seq2pdb->singletons);
 }
 
 
-// DLT OP
-double
-CalcRotationsNu2(CdsArray *cdsA)
+void
+Align2MSA(PDBCdsArray *pdbA, CdsArray *baseA, char *msafile_name, char *mapfile_name)
 {
-    Cds           **cds = cdsA->cds;
-    const Cds      *avecds = cdsA->avecds;
-    const double   *wts = (const double *) cdsA->w;
-    const double  **wtmat = (const double **) cdsA->WtMat;
-    Cds            *tcds = NULL;
-    double          deviation, deviation_sum;
-    double          norm1, norm2, innprod;
-    const int       vlen = cdsA->vlen;
-    int             i;
+    int             i, alnlen, vlen;
+    const int       cnum = pdbA->cnum;
+    MSA            *msa = NULL;
 
-
-    if (algo->covweight)
+    if (mapfile_name)
     {
-        tcds = cdsA->tcds;
-        MatMultCdsMultMatDiag(tcds, wtmat, avecds);
+        pdbA->seq2pdb = GetMapFile(mapfile_name);
     }
-    else if (algo->varweight)
+    else
     {
-        tcds = cdsA->tcds;
-        MatDiagMultCdsMultMatDiag(tcds, wts, avecds);
-    }
-    else if (algo->leastsquares)
-    {
-        tcds = cdsA->avecds;
+        pdbA->seq2pdb = Seq2pdbInit();
+        Seq2pdbAlloc(pdbA->seq2pdb, cnum);
     }
 
-    deviation = deviation_sum = 0.0;
-    for (i = 0; i < cdsA->cnum; ++i)
-    {
-        deviation = ProcGSLSVDvanNu2((const double **) cds[i]->sc,
-                                     (const double **) tcds->wc,
-                                     (const int *) cds[i]->nu,
-                                     vlen, cds[i]->matrix,
-                                     cdsA->tmpmat3a,
-                                     cdsA->tmpmat3b,
-                                     cdsA->tmpmat3c,
-                                     cdsA->tmpvec3a,
-                                     &norm1, &norm2, &innprod);
+    pdbA->seq2pdb->msa = msa = getmsa(msafile_name);
+    MSAdelgap(msa);
 
-        cds[i]->wRMSD_from_mean = sqrt(deviation / (3 * cdsA->vlen));
-        deviation_sum += deviation;
+//     MSAprint(msa);
+//     printf("\nseqlen: %d\n", msa->seqlen);
+//     printf("\nseqnum: %d\n", msa->seqnum);
+//     fflush(NULL);
+
+    if (msa->seqnum < pdbA->cnum)
+    {
+        fprintf(stderr,
+                "\n  ERROR682: # alignment sequences (%d) < # cds (%d)\n",
+                msa->seqnum, pdbA->cnum);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
     }
 
-    return(deviation_sum);
+    alnlen = msa->seqlen;
+    pdbA->seq2pdb->singletons = calloc(alnlen, sizeof(int));
+
+    if (mapfile_name)
+        GetSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+    else
+        DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+
+    GetSingletons(pdbA->seq2pdb->singletons, msa);
+
+    if (algo->missing)
+        GetUbiqs(pdbA->seq2pdb->singletons, msa);
+
+    vlen = ParseSelxns(pdbA, algo->selection, alnlen, pdbA->seq2pdb->singletons, algo->revsel);
+
+    /* allocate a CdsArray based on this alignment */
+    if (algo->atoms == 0)
+    {
+        CdsArrayAlloc(baseA, cnum, vlen);
+    }
+    else
+    {
+        fprintf(stderr,
+                "\n  ERROR683: atom selection must be CAs for superpositioning with a sequence alignment\n");
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    for (i = 0; i < cnum; ++i)
+        strncpy(baseA->cds[i]->filename, pdbA->cds[i]->filename, FILENAME_MAX - 1);
+
+    VerifyAlignmentVsPDBs(pdbA, baseA, msa, algo->atoms, pdbA->seq2pdb->map);
+    GetCdsFrAlignment(pdbA, baseA, pdbA->seq2pdb, msa, vlen, alnlen, pdbA->seq2pdb->singletons);
 }
 
 
-/* char aa1[] = "ARNDCQEGHILKMFPSTWYV"; */
-/* char aa3[] = "ALAARGASNASPCYSGLNGLUGLYHISILELEULYSMETPHEPROSERTHRTRPTYRVAL"; */
+void
+PDB2MapFile(PDBCdsArray *myPDBs, char *mapfile_name)
+{
+    FILE           *mapfile = NULL;
+    int             i, numscanned, seqnum, maxseqnum;
+    char            line[FILENAME_MAX + 256];
+
+    mapfile = fopen(mapfile_name, "w");
+    if (mapfile == NULL)
+    {
+        fprintf(stderr,
+                "\n  ERROR_691: cannot write alignment -> PDB mapping file \"%s\" \n",
+                mapfile_name);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    maxseqnum = 0;
+
+    rewind(mapfile);
+
+    for(i = 0, seqnum = 0; i < maxseqnum; ++i)
+    {
+        if (fgets(line, FILENAME_MAX + 256, mapfile) == NULL)
+            break;
+
+        if (line[0] == '#')
+            continue;
+
+        if (strlen(line) > 1)
+        {
+            Seq2PDB        *seq2pdb = NULL;
+
+            numscanned = sscanf(line, "%s %s", seq2pdb->pdbfile_name[i], seq2pdb->seqname[i]);
+
+            if (numscanned == 2)
+            {
+                ++seqnum;
+                /* printf("\nline %d (len %d): %s", i, (int) strlen(line), line); */
+            }
+        }
+    }
+
+    fclose(mapfile);
+}
+
 
 void
 pdb2fst(PDBCdsArray *pdbA)
@@ -1033,33 +1102,32 @@ pdb2fst(PDBCdsArray *pdbA)
     char           *pindex = NULL;
     FILE           *fp = NULL;
     char            outfile[FILENAME_MAX];
+    const PDBCds   *pdbi = NULL;
 
     for (i = 0; i < pdbA->cnum; ++i)
     {
-        strncpy(outfile, pdbA->cds[i]->filename, strlen(pdbA->cds[i]->filename)+1);
+        pdbi = (const PDBCds *) pdbA->cds[i];
+        strncpy(outfile, pdbi->filename, strlen(pdbi->filename)+1);
         strncat(outfile, ".fst", 4);
 
         fp = fopen(outfile, "w");
 
-        //fprintf(fp, ">%-72s", pdbA->cds[i]->filename);
-        fprintf(fp, ">%-s", pdbA->cds[i]->filename);
+        //fprintf(fp, ">%-72s", pdbi->filename);
+        fprintf(fp, ">%-s", pdbi->filename);
 
         count = 0;
-        for (j = 0; j < pdbA->cds[i]->vlen; ++j)
+        for (j = 0; j < pdbi->vlen; ++j)
         {
-            //printf("ATOM %4d:%4d \'%s\'\n", i, j, pdbA->cds[i]->name[j]);
+            //printf("ATOM %4d:%4d \'%s\'\n", i, j, pdbi->name[j]);
 
-            if ( (strncmp(pdbA->cds[i]->record[j], "ATOM  ", 6) == 0 || strncmp(pdbA->cds[i]->record[j], "HETATM", 6) == 0)
-                &&
-                (strncmp(pdbA->cds[i]->name[j], "CA ", 3) == 0 ||
-                 strncmp(pdbA->cds[i]->name[j], "C1*", 3) == 0 ||
-                 strncmp(pdbA->cds[i]->name[j], "C1'", 3) == 0) &&
-                (pdbA->cds[i]->altLoc[j] == ' ' || pdbA->cds[i]->altLoc[j] == 'A'))
+            if (RecordSelxn(pdbi->record[j]) &&
+                AtomSelxn(pdbi->name[j], 0) &&
+                AltLocSelxn(pdbi->altLoc[j]))
             {
                 if (count % 72 == 0)
                     fputc('\n', fp);
 
-                pindex = strstr(aan3, pdbA->cds[i]->resName[j]);
+                pindex = strstr(aan3, pdbi->resName[j]);
                 if (pindex == NULL)
                     fputc('X', fp);
                 else
