@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -87,18 +86,13 @@ CalcPCA(CdsArray *cdsA)
     cdsA->pcamat = MatAlloc(vlen, vlen);
     cdsA->pcavals = malloc(vlen * sizeof(double));
 
-    EigenGSLDest(CovMat, vlen, cdsA->pcavals, cdsA->pcamat, 1);
+    EigenGSL((const double **) CovMat, vlen, cdsA->pcavals, cdsA->pcamat, 1);
+    // EigenGSL order=1 gives evals large-to-small
     MatTransIp(cdsA->pcamat, vlen);
     //MatPrint(cdsA->pcamat, vlen);
 
-    /* LAPACK DSYEVR() computes selected eigenvalues, and optionally, eigenvectors of a
-       real symmetric matrix.  Find all eigenvalues (w[]) and eigenvectors (mat[][]).
-       The pcanum eigenvalues are in the first pcanum elements of the w[] vector,
-       ordered smallest to biggest.  Weird and horrible, but true (and in fact
-       makes sense if you think about it). */
-    //dsyevr_dest(CovMat, vlen, lower, upper, cdsA->pcavals, cdsA->pcamat, 1e-8);
-
     //MatPrint(cdsA->pcamat, vlen);
+    //VecPrint(cdsA->pcavals, vlen);
 
     pcvecs_name = mystrcat(algo->rootname, "_pcvecs.txt");
     pcstats_name = mystrcat(algo->rootname, "_pcstats.txt");
@@ -191,16 +185,16 @@ CalcPCA(CdsArray *cdsA)
     fclose(pcastats_fp);
     fclose(pcavecs_fp);
 
-    if (cov_name != NULL)
+    if (cov_name)
         free(cov_name);
 
-    if (cor_name != NULL)
+    if (cor_name)
         free(cor_name);
 
-    if (pcvecs_name != NULL)
+    if (pcvecs_name)
         free(pcvecs_name);
 
-    if (pcstats_name != NULL)
+    if (pcstats_name)
         free(pcstats_name);
 }
 
@@ -357,27 +351,26 @@ WritePCAFile(PDBCdsArray *parray, CdsArray *cdsA, const char *outfile_root)
         biggest = -DBL_MAX;
         for (j = 0; j < cvlen; ++j)
         {
-            /* printf("\n%3d %3d % f", i, j, mat[i][j]); */
-            if (biggest < fabs(mat[algo->pca - 1 - i][j]))
-                biggest = fabs(mat[algo->pca - 1 - i][j]);
+            //printf("\n%3d %3d % f", i, j, mat[i][j]);
+
+            if (biggest < fabs(mat[i][j]))
+                biggest = fabs(mat[i][j]);
         }
-/*     } */
+
         biggest = 99.99 / biggest;
 
-/*     for (i = 0; i < algo->pca; ++i) */
-/*     { */
         /* rescale (for rasmol really) so that the largest eigenvector component
            is = 99.99, i.e. the largest value allowable in the b-value column
            of a PDB file */
-       if (algo->atoms == 0)
-       {
+        if (algo->atoms == 0)
+        {
             m = 0;
             for (j = 0; j < cvlen; ++j)
             {
                 /* skip inital PDBCds that may have been selected out */ /* DLT debug fix */
                 while (/* strncmp(cdsA->cds[0]->resName[j], parray->cds[0]->resName[m], 3) != 0 || */
                        cdsA->cds[0]->chainID[j] != parray->cds[0]->chainID[m] ||
-                       cdsA->cds[0]->resSeq[j] != parray->cds[0]->resSeq[m])
+                       cdsA->cds[0]->resSeq[j]  != parray->cds[0]->resSeq[m])
                 {
                     ++m;
 
@@ -388,10 +381,10 @@ WritePCAFile(PDBCdsArray *parray, CdsArray *cdsA, const char *outfile_root)
                 /* while they match, set the B-factor according to the given PC */
                 while (/* strncmp(cdsA->cds[0]->resName[j], parray->cds[0]->resName[m], 3) == 0 && */
                        cdsA->cds[0]->chainID[j] == parray->cds[0]->chainID[m] &&
-                       cdsA->cds[0]->resSeq[j] == parray->cds[0]->resSeq[m])
+                       cdsA->cds[0]->resSeq[j]  == parray->cds[0]->resSeq[m])
                 {
-                    tempFactor = mat[algo->pca - 1 - i][j] * biggest;
-                    /* printf("\n%4d %4d % f", j, m, tempFactor); */
+                    tempFactor = mat[i][j] * biggest;
+                    //printf("\n%4d %4d % f % f", j, m, tempFactor, mat[i][j];
                     for (k = 0; k < parray->cnum; ++k)
                         parray->cds[k]->tempFactor[m] = tempFactor;
 
@@ -409,7 +402,7 @@ WritePCAFile(PDBCdsArray *parray, CdsArray *cdsA, const char *outfile_root)
         {
             for (j = 0; j < cvlen; ++j)
             {
-                tempFactor = mat[algo->pca - 1 - i][j] * biggest;
+                tempFactor = mat[i][j] * biggest;
 
                 for (k = 0; k < parray->cnum; ++k)
                     parray->cds[k]->tempFactor[j] = tempFactor;
@@ -455,8 +448,6 @@ WritePCAFile(PDBCdsArray *parray, CdsArray *cdsA, const char *outfile_root)
 
         fclose(pdbfile);
     }
-
-    PCADestroy(cdsA);
 }
 
 
