@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -395,7 +394,7 @@ CalcJacobiSVD(double **A, double **U, double *z, double **Vt, double tol)
     for (i = 0; i < 3; i++)
         U[i][i] = 1.0;
 
-    MatCpySym(Vt, (const double **) A, 3);
+    MatCpySqr(Vt, (const double **) A, 3);
 
     eps = tol * tol * fabs(Vt[0][0] + Vt[1][1] + Vt[2][2]);
 
@@ -533,7 +532,7 @@ CalcJacobiSVDpp(double **A, double **U, double *z, double **Vt, double tol)
     for (i = 0; i < 3; i++)
         U[i][i] = 1.0;
 
-    MatCpySym(Vt, (const double **) A, 3);
+    MatCpySqr(Vt, (const double **) A, 3);
 
     eps = tol * tol * fabs(Vt[0][0] + Vt[1][1] + Vt[2][2]);
 
