@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -107,7 +106,8 @@ QCProot(double *coeff, double guess, const double delta)
     }
 
     fprintf(stderr,
-            "\n\n WARNING: Newton-Raphson root-finding in \'QCProot()\' did not converge \n");
+            "\n WARNING: Newton-Raphson root-finding in \'QCProot()\' did not converge: %g %g %g\n",
+            guess, oldg, guess-oldg);
 
     return(guess);
 //    exit(EXIT_FAILURE);
@@ -374,7 +374,7 @@ CalcQuarticCoeffsPu(const FragCds *frag1, const FragCds *frag2, const int len, d
                    *fx2 = frag2->x, *fy2 = frag2->y, *fz2 = frag2->z;
     int             i;
     double          lambdamax;
-    double          precision = 1e-14;
+    double          precision = 1e-8; // 1e-14 is too stringent, can prevent convergence of QCProot.
 //    double          precision2 = 1e-7;
 
     Sxx = Sxy = Sxz = Syx = Syy = Syz = Szx = Szy = Szz = 0.0;
@@ -534,7 +534,7 @@ CalcQuarticCoeffsPu(const FragCds *frag1, const FragCds *frag2, const int len, d
 }
 
 
-static void
+void
 RotMatToQuaternion(const double **rot, double *quat)
 {
     double              trace, s, w, x, y, z;
@@ -587,6 +587,7 @@ RotMatToQuaternion(const double **rot, double *quat)
 
 /* returns the sum of the deviations
    rmsd = sqrt(sum_dev/atom_num) */
+   /* rotmat checks out vs GSLSVD method */
 double
 QuarticHornFrag(const FragCds *frag1, const FragCds *frag2, double *coeff)
 {
@@ -599,35 +600,35 @@ QuarticHornFrag(const FragCds *frag1, const FragCds *frag2, double *coeff)
     frag2_radgyr = RadGyrSqrFrag(frag2);
 
     double         *quat = malloc(4 * sizeof(double));
-    double        **rotmat = MatAlloc(3, 3);
+    //double        **rotmat = MatAlloc(3, 3);
     lambdamax = CalcQuarticCoeffsPu(frag1, frag2, frag1->fraglen, coeff, frag1_radgyr + frag2_radgyr, quat);
     rmsd2 = (frag1_radgyr + frag2_radgyr) - (2.0 * lambdamax);
-    printf("\n% -14.6e % -14.6e % -14.6e % -14.6e", quat[0], quat[1], quat[2], quat[3]);
-    QuatToRotmat(rotmat, quat);
-    Mat3Print(rotmat);
+    //printf("\n% -14.6e % -14.6e % -14.6e % -14.6e", quat[0], quat[1], quat[2], quat[3]);
+    //QuatToRotmat(rotmat, quat);
+    //Mat3Print(rotmat);
 
-    double        **Rmat = MatAlloc(3, 3);
-    double        **Umat = MatAlloc(3, 3);
-    double        **VTmat = MatAlloc(3, 3);
-    double        *sigma = malloc(3 * sizeof(double));
-    double        **rotmat2 = MatAlloc(3, 3);
+//     double        **Rmat = MatAlloc(3, 3);
+//     double        **Umat = MatAlloc(3, 3);
+//     double        **VTmat = MatAlloc(3, 3);
+//     double        *sigma = malloc(3 * sizeof(double));
+//     double        **rotmat2 = MatAlloc(3, 3);
 
-    ProcGSLSVDFrag(frag1, frag2, rotmat2, Rmat, Umat, VTmat, sigma);
+    //ProcGSLSVDFrag(frag1, frag2, rotmat2, Rmat, Umat, VTmat, sigma);
 
-    RotMatToQuaternion((const double **) rotmat2, quat);
-    printf("\n% -14.6e % -14.6e % -14.6e % -14.6e", quat[0], quat[1], quat[2], quat[3]);
-    Mat3Print(rotmat2);
+    //RotMatToQuaternion((const double **) rotmat2, quat);
+    //printf("\n% -14.6e % -14.6e % -14.6e % -14.6e", quat[0], quat[1], quat[2], quat[3]);
+    //Mat3Print(rotmat2);
 
-    if (Mat3Eq((const double **) rotmat, (const double **) rotmat2, 1e-8) == 0)
-        printf("\n Not equal\n");
+    //if (Mat3Eq((const double **) rotmat, (const double **) rotmat2, 1e-8) == 0)
+    //    printf("\n Not equal\n");
 
-    MatDestroy(&rotmat);
-    MatDestroy(&rotmat2);
-    MatDestroy(&Umat);
-    MatDestroy(&Rmat);
-    MatDestroy(&VTmat);
+//     MatDestroy(&rotmat);
+//     MatDestroy(&rotmat2);
+//     MatDestroy(&Umat);
+//     MatDestroy(&Rmat);
+//     MatDestroy(&VTmat);
 
-    free(sigma);
+//    free(sigma);
     free(quat);
 
     //ComputeQuartHornFrag(frag1, frag2, coeff);
@@ -638,8 +639,6 @@ QuarticHornFrag(const FragCds *frag1, const FragCds *frag2, double *coeff)
 }
 
 
-#include "FragCds.h"
-
 void
 FragDistPu(CdsArray *cdsA, int fraglen, int center_ca, int pu)
 {
