@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -58,7 +57,7 @@ dist_name[DISTNUM][32] =
 {
     "Normal",      "Laplace",   "EVD",        "Logistic",
     "Cauchy",      "Uniform",   "Weibull",
-    "Gamma",       "ChisqrGen",  "Lognormal",
+    "Gamma",       "ChisqrGen", "Lognormal",
     "Invgamma",    "Invgauss",  "Recinvgauss", "Pareto",
     "Exponential", "Invchisqr", "Chisqr",      "Chi",
     "Rayleigh", "Rice*",
@@ -322,7 +321,7 @@ CalcEdgeworthVanHulleEntropy(double **vec, int dim, int len)
 //     printf ("\n\nEdgeworth covariance matrix:");
 //     MatPrintLowerDiag(cov, dim);
 
-    EigenvalsGSL(cor, dim, eval);
+    EigenvalsGSL((const double **) cor, dim, eval);
 
     //VecPrint(eval, dim);
 
@@ -491,9 +490,9 @@ CalcEdgeworthVanHulleEntropy(double **vec, int dim, int len)
     MatDestroy(&dif);
     MatDestroy(&cor);
     MatDestroy(&cov);
-    free(eval);
-    free(std);
-    free(ave);
+    MyFree(eval);
+    MyFree(std);
+    MyFree(ave);
 
     return(entropy);
 }
@@ -573,7 +572,7 @@ CalcDistStats(const double *array, const int len)
     printf("\n% 10.2f (+/- %8.2f) % 7.2f % 7.2f % 10.2f (% 10.2f - % 10.2f) (% 10.2f - % 10.2f)\n\n",
            pstats[0], pstats[1], pstats[2], pstats[3], pstats[4], pstats[5], pstats[6], pstats[7], pstats[8]);
 
-    free(x);
+    MyFree(x);
 }
 
 
@@ -583,7 +582,7 @@ ParseCmdLine(int *argc, char **argv[])
     int             option;
 
     /* get the options */
-    while ((option = getopt (*argc, *argv, "1b:c:C:d:D:g:G:hI:m:n:op:P:s:S:t:T:vz")) != -1)
+    while ((option = getopt (*argc, *argv, "1b:B:c:C:d:D:g:G:hI:m:n:op:P:s:S:t:T:vz")) != -1)
     {
         switch (option)
         {
@@ -595,6 +594,10 @@ ParseCmdLine(int *argc, char **argv[])
                 dfparams->bootstrap = (int) strtol(optarg, NULL, 10);
                 break;
 
+            case 'B':
+                dfparams->burnin = (double) strtod(optarg, NULL);
+                break;
+
             case 'c':
                 dfparams->col = (int) strtol(optarg, NULL, 10) - 1;
 
@@ -796,6 +799,7 @@ main(int argc, char *argv[])
     dfparams->trigamma = 0.0;
     dfparams->invdigamma = 0.0;
     dfparams->neg = 0;
+    dfparams->burnin = 0.0;
 
     //printf("\n###### %e\n", normal_sdf(5.0, 0.0, 1.0));
 
@@ -823,7 +827,7 @@ main(int argc, char *argv[])
         double x = dfparams->invdigamma;
         double lambda;
 
-        if (dfparams->neg)
+        if (dfparams->neg == 1)
             x = -x;
 
         // Inverse digamma (psi) function.  The digamma function is the
@@ -858,7 +862,7 @@ main(int argc, char *argv[])
         exit(EXIT_SUCCESS);
     }
 
-    if (pdfparams != NULL)
+    if (pdfparams)
     {
         if (dfparams->distindex == -1)
             dfparams->distindex = 0;
@@ -868,10 +872,10 @@ main(int argc, char *argv[])
                pdfparams[0], pdfparams[1], pdfparams[2],
                distpdf_func[dfparams->distindex](pdfparams[0], pdfparams[1], pdfparams[2]));
 
-        free(pdfparams);
+        MyFree(pdfparams);
         exit(EXIT_SUCCESS);
     }
-    else if (cdfparams != NULL)
+    else if (cdfparams)
     {
         if (dfparams->distindex == -1)
             dfparams->distindex = 0;
@@ -881,10 +885,10 @@ main(int argc, char *argv[])
                cdfparams[0], cdfparams[1], cdfparams[2],
                distcdf_func[dfparams->distindex](cdfparams[0], cdfparams[1], cdfparams[2]));
 
-        free(cdfparams);
+        MyFree(cdfparams);
         exit(EXIT_SUCCESS);
     }
-    else if (sdfparams != NULL)
+    else if (sdfparams)
     {
         if (dfparams->distindex == -1)
             dfparams->distindex = 0;
@@ -894,10 +898,10 @@ main(int argc, char *argv[])
                sdfparams[0], sdfparams[1], sdfparams[2],
                distsdf_func[dfparams->distindex](sdfparams[0], sdfparams[1], sdfparams[2]));
 
-        free(sdfparams);
+        MyFree(sdfparams);
         exit(EXIT_SUCCESS);
     }
-    else if (intparams != NULL)
+    else if (intparams)
     {
         if (dfparams->distindex == -1)
             dfparams->distindex = 0;
@@ -907,7 +911,7 @@ main(int argc, char *argv[])
                intparams[0], intparams[1], intparams[2], intparams[3],
                distint_func[dfparams->distindex](intparams[0], intparams[1], intparams[2], intparams[3]));
 
-        free(intparams);
+        MyFree(intparams);
         exit(EXIT_SUCCESS);
     }
     else if (dfparams->distdev > -1)
@@ -1007,9 +1011,9 @@ main(int argc, char *argv[])
     else
         dfparams->distnum = DISTNUM;
 
-    diststats = malloc((dfparams->distnum) * sizeof(DistStats *));
+    diststats = calloc(dfparams->distnum, sizeof(DistStats *));
     for(i = 0; i < dfparams->distnum; ++i)
-        diststats[i] = malloc(sizeof(DistStats));
+        diststats[i] = calloc(1, sizeof(DistStats));
 
     if (dfparams->distindex == -1)
     {
@@ -1063,17 +1067,17 @@ main(int argc, char *argv[])
         printf("\nlogL:% 11.3f n:%5d AIC:% 11.3f chi^2:% 9.6f\n\n",
                omni_logL, length, omni_logL - 3 * dfparams->mix + 1, mix_chi2);
 
-        free(mean);
-        free(var);
-        free(mixp);
+        MyFree(mean);
+        MyFree(var);
+        MyFree(mixp);
     }
 
     /* do the bootstrap if so desired */
-    if (dfparams->distnum && dfparams->bootstrap > 1)
+    if (dfparams->distnum == 1 && dfparams->bootstrap > 1)
     {
         BootFit(diststats, dfparams->distindex, array, length, dfparams);
     }
-    else if(dfparams->distnum)
+    else if(dfparams->distnum == 1)
     {
         SingleDistFit(distfit_func, dfparams->distindex, dist_name[dfparams->distindex], diststats[0], array, length, dfparams);
     }
@@ -1088,7 +1092,7 @@ main(int argc, char *argv[])
 
             TestData(array, length);
 
-            if (dfparams->histo)
+            if (dfparams->histo == 1)
             {
                 MultiHistFit(array, freq, length);
             }
@@ -1111,11 +1115,11 @@ main(int argc, char *argv[])
 
     free(array);
     free(warray);
-    if (dfparams->histo)
+    if (dfparams->histo == 1)
         free(freq);
     for(i = 0; i < dfparams->distnum; ++i)
-        free(diststats[i]);
-    free(diststats);
+        MyFree(diststats[i]);
+    MyFree(diststats);
 
     return (EXIT_SUCCESS);
 }
@@ -1163,15 +1167,15 @@ TestData(const double *data, const int length)
 int
 TestDistVsData(const int ndist)
 {
-    if (dist_beta[ndist] && dfparams->isbeta == 0)
+    if (dist_beta[ndist] == 1 && dfparams->isbeta == 0)
         return(0);
-    else if (dist_posdef[ndist] && dfparams->isneg)
+    else if (dist_posdef[ndist] == 1 && dfparams->isneg == 1)
         return(0);
-    else if (dist_nonneg[ndist] && dfparams->isneg)
+    else if (dist_nonneg[ndist] == 1 && dfparams->isneg == 1)
         return(0);
-    else if (dist_posdef[ndist] && dfparams->iszero)
+    else if (dist_posdef[ndist] == 1 && dfparams->iszero == 1)
         return(0);
-    else if (dist_nonneg[ndist] && dfparams->iszero) /* should return 1, but there are problems fitting data with zeros */
+    else if (dist_nonneg[ndist] == 1 && dfparams->iszero == 1) /* should return 1, but there are problems fitting data with zeros */
         return(0);
     else
         return(1);
@@ -1210,7 +1214,7 @@ PrintRandDist(char *fname, double (*dist_dev)(double param1, double param2, cons
     fflush(NULL);
 
     fclose(tmpfile);
-    free(array);
+    MyFree(array);
     gsl_rng_free(r2);
 }
 
@@ -1396,9 +1400,9 @@ BootFit(DistStats **diststats, int distindex, double *array, int length, DFParam
 
     PrintDistSD(diststats);
 
-    free(weights);
-    free(jackarray);
-    free(p);
+    MyFree(weights);
+    MyFree(jackarray);
+    MyFree(p);
     gsl_rng_free(r2);
 }
 
@@ -1470,7 +1474,16 @@ MultiDistFit(DistFit *distfit_func, char dist_name[][32],
         strcpy(diststats[j]->dist_name, dist_name[j]);
 
         if (TestDistVsData(j) == 0)
+        {
+            diststats[j]->param1 = param1;
+            diststats[j]->param2 = param2;
+            diststats[j]->chisq = chisq;
+            diststats[j]->logL = logL;
+            diststats[j]->logLper = logLper;
+            diststats[j]->AIC = AIC;
+            diststats[j]->BIC = BIC;
             continue;
+        }
 
 /*         printf("\n Fitting %s ...\n", dist_name[j]); */
 /*         fflush(NULL); */
@@ -1709,10 +1722,10 @@ InsortDiststats(DistStats **diststats, int distnum)
         distsort[j] = tmpj;
     }
 
-/*     for (i = 0; i < distnum; ++i) */
-/*         printf("\n %3d %3d %e", i, distsort[i], diststats[distsort[i]]->AIC); */
-/*  */
-/*     exit(0); */
+//     for (i = 0; i < distnum; ++i)
+//         printf("\n %3d %3d %e", i, distsort[i], diststats[distsort[i]]->AIC);
+//
+//     exit(0);
 }
 
 
@@ -1785,7 +1798,7 @@ gethistvals(FILE *listfile, double *data, double *freq, int length)
 
     rewind(listfile);
     i = 0;
-    while(fgets(line, 512, listfile) != NULL && i < length)
+    while(fgets(line, 512, listfile) && i < length)
     {
         sscanf(line, "%le %le", &data[i], &freq[i]);
         ++i;
@@ -1836,7 +1849,7 @@ GetFileColLen(FILE *fp)
     rewind(fp);
 
     /* keep getting lines as long as the first data element is not a number */
-    while(fgets(line, linelen, fp) != NULL)
+    while(fgets(line, linelen, fp))
     {
         nscan = sscanf(line, "%le", &tmp);
         if (nscan > 0)
@@ -1892,7 +1905,7 @@ Actually, it ignores any lines beginning with a word (i.e., a non-number).
 Longest line allowed is 4096 chars.
 In principle, the number of records and columns is unlimited.
 Returns a (col x recordlen) array of numbers.
-Each column in the file ends up being a contiguous vector of floats.
+Each column in the file ends up being a contiguous vector of doubles.
 */
 static double
 **ReadValsMulti(const char *listfile_name, int *col, int *len)
@@ -2039,7 +2052,7 @@ double
     rewind(listfile);
 
     i = 0;
-    while(fgets(line, 512, listfile) != NULL && i < *length)
+    while(fgets(line, 512, listfile) && i < *length)
     {
         if (line[0] == '#')
         {
@@ -2049,7 +2062,7 @@ double
 
         comment = strchr(line, '#');
 
-        if (comment != NULL)
+        if (comment)
             *comment = '\0';
 
 /*         numscanned = sscanf(line, "%le %le %le %le %le %le %le %le %le %le %le %le %le %le %le %le", */
@@ -2223,9 +2236,9 @@ DistFitMix(DistPDF distpdf, DistFitW distfitw, InitMixParams distinit,
     omni_logL = CalcOmniLogL(distpdf, x, n, mixn, param1, param2, mixp);
 
     for (i = 0; i < mixn; ++i)
-        free(probs[i]);
-    free(probs);
-    free(oldprobs);
+        MyFree(probs[i]);
+    MyFree(probs);
+    MyFree(oldprobs);
 
     return(omni_logL);
 }
