@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2012 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -183,7 +182,7 @@ betasym_fit(const double *x, const int n, double *ra, double *rb, double *prob);
 #ifndef BIMULTINOM_SEEN
 #define BIMULTINOM_SEEN
 
-double 
+double
 binomialdev(double pp, int n);
 
 void
@@ -795,48 +794,48 @@ shufflef(double *a, int n);
  * Prototypes for portable math library (ported from C Toolkit)
  */
 
-/*#include <algo/blast/core/ncbi_std.h> 
+/*#include <algo/blast/core/ncbi_std.h>
 #include <algo/blast/core/blast_export.h>*/
 
-double 
+double
 s_PolyGamma(double x, int order);
 
 /** Natural logarithm with shifted input
  *  @param x input operand (x > -1)
  *  @return log(x+1)
  */
- 
+
 double BLAST_Log1p (double x);
 
-/** Exponentional with base e 
+/** Exponentional with base e
  *  @param x input operand
  *  @return exp(x) - 1
  */
- 
+
 double BLAST_Expm1 (double x);
 
 /** Factorial function
  *  @param n input operand
  *  @return (double)(1 * 2 * 3 * ... * n)
  */
- 
+
 double BLAST_Factorial(int n);
 
-/** Logarithm of the factorial 
+/** Logarithm of the factorial
  *  @param x input operand
  *  @return log(1 * 2 * 3 * ... * x)
  */
- 
+
 double BLAST_LnFactorial (double x);
 
-/** log(gamma(n)), integral n 
+/** log(gamma(n)), integral n
  *  @param n input operand
  *  @return log(1 * 2 * 3 * ... (n-1))
  */
- 
+
 double BLAST_LnGammaInt (int n);
 
-/** Romberg numerical integrator 
+/** Romberg numerical integrator
  *  @param f Pointer to the function to integrate; the first argument
  *               is the variable to integrate over, the second is a pointer
  *               to a list of additional arguments that f may need
@@ -848,25 +847,25 @@ double BLAST_LnGammaInt (int n);
  *  @param q Right-hand endpoint of the integration interval
  *           (q is assumed > p)
  *  @param eps The relative error tolerance that indicates convergence
- *  @param epsit The number of consecutive diagonal entries in the 
+ *  @param epsit The number of consecutive diagonal entries in the
  *               Romberg array whose relative difference must be less than
- *               eps before convergence is assumed. This is presently 
+ *               eps before convergence is assumed. This is presently
  *               limited to 1, 2, or 3
  *  @param itmin The minimum number of diagnonal Romberg entries that
  *               will be computed
  *  @return The computed integral of f() between p and q
  */
- 
-double BLAST_RombergIntegrate (double (*f) (double, void*), 
-                               void* fargs, double p, double q, 
+
+double BLAST_RombergIntegrate (double (*f) (double, void*),
+                               void* fargs, double p, double q,
                                double eps, int epsit, int itmin);
 
-/** Greatest common divisor 
+/** Greatest common divisor
  *  @param a First operand (any integer)
  *  @param b Second operand (any integer)
  *  @return The largest integer that evenly divides a and b
  */
- 
+
 int BLAST_Gcd (int a, int b);
 
 /** Divide 3 numbers by their greatest common divisor
@@ -875,29 +874,29 @@ int BLAST_Gcd (int a, int b);
  * @param c Third integer [in] [out]
  * @return The greatest common divisor
  */
- 
+
 int BLAST_Gdb3(int* a, int* b, int* c);
 
-/** Nearest integer 
+/** Nearest integer
  *  @param x Input to round (rounded value must be representable
  *           as a 32-bit signed integer)
  *  @return floor(x + 0.5);
  */
- 
+
 long BLAST_Nint (double x);
 
-/** Integral power of x 
+/** Integral power of x
  * @param x floating-point base of the exponential
  * @param n (integer) exponent
  * @return x multiplied by itself n times
  */
- 
+
 double BLAST_Powi (double x, int n);
 
-/** Number of derivatives of log(x) to carry in gamma-related 
+/** Number of derivatives of log(x) to carry in gamma-related
     computations */
-#define LOGDERIV_ORDER_MAX    4  
-/** Number of derivatives of polygamma(x) to carry in gamma-related 
+#define LOGDERIV_ORDER_MAX    4
+/** Number of derivatives of polygamma(x) to carry in gamma-related
     computations for non-integral values of x */
 #define POLYGAMMA_ORDER_MAX    LOGDERIV_ORDER_MAX
 
@@ -1064,11 +1063,11 @@ pareto_fit(const double *data, const int num, double *a, double *c, double *prob
 /*
  * The key TYPE.
  * COARRAY_T is the type of the companion array
- * The keys are the array items moved with the SWAP macro
- * around using the SWAP macro.
+ * The keys are the array items moved with the NWAP macro
+ * around using the NWAP macro.
  * the comparison macros can compare either the key or things
  * referenced by the key (if its a pointer)
- */ 
+ */
 typedef double      KEY_T;
 typedef char       *COARRAY_T;
 /*
@@ -1091,12 +1090,12 @@ typedef char       *COARRAY_T;
 /*
  * Swap macro:
  */
- 
+
 /* double              tempd; */
 /* char               *tempc; */
 /*  */
-/* #define SWAPD(x, y) tempd = (x); (x) = (y); (y) = tempd */
-/* #define SWAPC(x, y) tempc = (x); (x) = (y); (y) = tempc */
+/* #define NWAPD(x, y) tempd = (x); (x) = (y); (y) = tempd */
+/* #define NWAPC(x, y) tempc = (x); (x) = (y); (y) = tempc */
 
 extern void
 swapd(double *x, double *y);
@@ -1221,6 +1220,7 @@ beta(double z, double w);
 #ifndef STATISTICS_SEEN
 #define STATISTICS_SEEN
 
+
 double
 average(const double *data, const int dim);
 
@@ -1273,10 +1273,10 @@ dblcmp(const void *dbl1, const void *dbl2);
 int
 dblcmp_rev(const void *dbl1, const void *dbl2);
 
-double 
+double
 normal_dist(double x);
 
-double 
+double
 probks(double alam);
 
 double
@@ -1285,7 +1285,7 @@ kstwo(double *data1, int n1, double *data2, int n2);
 double
 ksone(double *data, int n, double (*func)(double));
 
-double 
+double
 probks_dw(double ksd, int em);
 
 void
@@ -1294,7 +1294,7 @@ mMultiply(double *A, double *B, double *C, int m);
 void
 mPower(double *A, int eA, double *V, int *eV, int m, int n);
 
-double 
+double
 KS(double d, int n);
 
 double
