@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -35,6 +34,24 @@
 #include "DLTutils.h"
 
 
+void
+MyFree(void *p)
+{
+    if (p)
+    {
+        free(p);
+        p = NULL;
+    }
+//     else
+//     {
+//         fprintf(stderr, "\nERROR623: trying to free NULL ptr -- %s:%d:%p\n\n",
+//                 __FILE__, __LINE__, p);
+//         fflush(NULL);
+//         //exit(EXIT_FAILURE);
+//     }
+}
+
+
 void
 myfloath(void)
 {
@@ -67,7 +84,7 @@ myfloath(void)
     printf(" LDBL_MAX_EXP:    %d\n",  LDBL_MAX_EXP);
     printf(" LDBL_MIN:        %Le\n", LDBL_MIN);
     printf(" LDBL_MIN_10_EXP: %d\n",  LDBL_MIN_10_EXP);
-    printf(" LDBL_MIN_EXP:    %d\n",  LDBL_MIN_EXP); 
+    printf(" LDBL_MIN_EXP:    %d\n",  LDBL_MIN_EXP);
     printf("\n");
 }
 
@@ -327,7 +344,7 @@ FILE
 
     fp = fopen(newfname, mode);
 
-    free(newfname);
+    MyFree(newfname);
 
     return(fp);
 }
@@ -406,7 +423,7 @@ FILE
 
     fclose(infile);
     fclose(outfile);
-    
+
     return(outfile);
 }
 
@@ -534,14 +551,14 @@ char
 char
 *getroot(char *filename)
 {
-    char           *p;
+    char           *p = NULL;
     static char    *rootname = NULL;
     int             length = strlen(filename);
 
     rootname = (char *) malloc((length + 1) * sizeof(char));
 
     mystrncpy(rootname, filename, length);
-    p = strrchr(rootname, '.'); /* find where file extension is 
+    p = strrchr(rootname, '.'); /* find where file extension is
                                    strrchr finds _last_ '.' */
     if (p == NULL)
     {
