@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -204,10 +204,10 @@ double
 MatDiff(const double **mat1, const double **mat2, const int row, const int col);
 
 void
-MatCpySym(double **matrix2, const double **matrix1, const int dim);
+MatCpySqr(double **matrix2, const double **matrix1, const int dim);
 
 void
-MatCpySymgen(double **matrix2, const double **matrix1, const int rows, const int cols);
+MatCpySqrgen(double **matrix2, const double **matrix1, const int rows, const int cols);
 
 void
 MatMultGenUSVOp(double **c, const double **u, double *s, const double **v,
@@ -1057,10 +1057,10 @@ double
 MatDiff(const double **mat1, const double **mat2, const int row, const int col);
 
 void
-MatCpySym(double **matrix2, const double **matrix1, const int dim);
+MatCpySqr(double **matrix2, const double **matrix1, const int dim);
 
 void
-MatCpySymgen(double **matrix2, const double **matrix1, const int rows, const int cols);
+MatCpySqrgen(double **matrix2, const double **matrix1, const int rows, const int cols);
 
 void
 MatMultGenUSVOp(double **c, const double **u, double *s, const double **v,
