@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ RotMat2AxisAngleQuat(double **rot, double *v);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -193,7 +193,7 @@ Mat4TransposeOp(double **matrix2, const double **matrix1);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -237,9 +237,17 @@ MatPrintRec(double **matrix, const int n, const int m);
 void
 MatDestroy(double ***matrix_ptr);
 
+
+void
+MatCharDestroy(char ***matrix_ptr);
+
 double
 **MatAlloc(const int rows, const int cols);
 
+char
+**MatCharAlloc(const int rows, const int cols);
+
+
 void
 MatIntDestroy(int ***matrix);
 
@@ -259,7 +267,7 @@ double
 MatDiff(const double **mat1, const double **mat2, const int row, const int col);
 
 void
-MatCpySym(double **matrix2, const double **matrix1, const int dim);
+MatCpySqr(double **matrix2, const double **matrix1, const int dim);
 
 void
 MatCpyGen(double **matrix2, const double **matrix1, const int rows, const int cols);
@@ -320,7 +328,7 @@ FrobDiffNormIdentMat(const double **mat, const int dim);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -352,7 +360,7 @@ MultivarLnGamma(const int k, const double a);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -412,7 +420,7 @@ InGammaP(double a, double x);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -465,84 +473,11 @@ double
 VecBiggest(double *vec, const int len);
 
 #endif /* !MATRIXUTILS_SEEN */
-/*
-    Theseus - maximum likelihood superpositioning of macromolecular structures
-
-    Copyright (C) 2004-2014 Douglas L. Theobald
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
-
-    -/_|:|_|_\-
-*/
-
-#ifndef EIGEN_SEEN
-#define EIGEN_SEEN
-
-void
-eigen3(double **z, double *eigenval);
-
-void
-eigen4(double **Q, double *eigenval);
-
-void
-eigenval4(double **Q, double *eigenval);
-
-double
-pythag(double a, double b);
-
-void
-tred24(double **a, double *d, double *e);
-
-void
-tred24vals(double **a, double *d, double *e);
-
-void
-tqli4(double *d, double *e, double **z);
-
-void
-tqli4vals(double *d, double *e, double **z);
-
-int
-jacobi3(double **a, double *d, double **v, double tol);
-
-int
-jacobi3_cyc(double **a, double *d, double **v, double tol);
-
-void
-jacobi4(double **a, double *d, double **v);
-
-void
-rotate(double **a, double s, double tau,
-       int i, int j, int k, int l);
-
-double
-InvSymEigenOp(double **invmat, const double **mat, int n,
-              double *evals, double **evecs, const double tol);
-
-void
-EigenReconSym(double **mat, const double **evecs, const double *evals, const int n);
-
-#endif
 
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2010 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -593,11 +528,18 @@ CholeskyGSLDest(double **A, const int dim);
 void
 PseudoinvSymGSL(const double **inmat, double **outmat, int n, double tol);
 
+double
+InvSymEigenOp(double **invmat, const double **mat, int n,
+              double *evals, double **evecs, const double tol);
+
+void
+EigenReconSym(double **mat, const double **evecs, const double *evals, const int n);
+
 #endif
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -638,7 +580,7 @@ integrate_romberg(double (*f)(double a, double p1, double p2),
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -671,7 +613,7 @@ ludcmp(double **a, int n, int *indx, double *d);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -739,7 +681,7 @@ shufflef(double *a, int n);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -974,7 +916,7 @@ double BLAST_Powi (double x, int n);
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
