@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2010 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -256,3 +256,72 @@ PseudoinvSymGSL(const double **inmat, double **outmat, int n, double tol)
     MatDestroy(&u);
     MatDestroy(&v);
 }
+
+
+
+
+/* Calculates the Moore-Penrose pseudoinverse of a symmetric matrix
+   (if necessary). Returns the condition number of the matrix.
+   */
+double
+InvSymEigenOp(double **invmat, const double **mat, int n,
+              double *evals, double **evecs, const double tol)
+{
+    double                   cond;
+    int                      i, j, k;
+    double                 **tmpmat = MatAlloc(n, n);
+
+    memcpy(tmpmat[0], mat[0], n * n * sizeof(double));
+
+    EigenGSLDest(tmpmat, n, evals, evecs, 0);
+
+    cond = evals[n-1] / evals[0];
+
+    for (i = 0; i < n; ++i)
+    {
+        if (evals[i] > tol)
+            evals[i] = 1.0 / evals[i];
+        else
+            evals[i] = 0.0;
+    }
+
+    memset(invmat[0], 0, n * n * sizeof(double));
+
+    for (i = 0; i < n; ++i)
+        for (j = 0; j <= i; ++j)
+            for (k = 0; k < n; ++k)
+                invmat[i][j] += (evecs[i][k] * evals[k] * evecs[j][k]);
+
+    for (i = 0; i < n; ++i)
+        for (j = i + 1; j < n; ++j)
+            invmat[i][j] = invmat[j][i];
+
+    MatDestroy(&tmpmat);
+
+    return(cond);
+}
+
+
+/* Calculates A = LDL^t, where L is a matrix of right eigenvectors and D is
+   a diagonal matrix of eigenvalues, in one fell swoop. Except here the
+   eigenvalues are delivered as a 1 x n vector. */
+/* This function is consistent with eigensym() above - 2006-05-10 */
+void
+EigenReconSym(double **mat, const double **evecs, const double *evals, const int n)
+{
+    int             i, j, k;
+
+    /* (i x k)(k x j) = (i x j) */
+    for (i = 0; i < n; ++i)
+    {
+        for (j = 0; j < n; ++j)
+        {
+            mat[i][j] = 0.0;
+            for (k = 0; k < n; ++k)
+                mat[i][j] += (evecs[i][k] * evals[k] * evecs[j][k]);
+        }
+    }
+}
+
+
+
