@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 
 
 /* Calculates the "probabilists" Hermite polynomial */
-double 
+double
 Hermite(const int n, const double x)
 {
     double a, b, result = 0.0;
@@ -88,14 +88,14 @@ BesselI(const double nu, const double z)
 {
     int         k, iters = 200;
     double      term, sum;
-    
+
     sum = 0.0;
     for (k = 0; k < iters; ++k)
     {
         term = pow(0.25*z*z, k) / (Factorial(k) * tgamma(nu + k + 1));
         sum += term;
 
-        if (fabs(term < FLT_EPSILON))
+        if (fabs(term) < FLT_EPSILON)
             break;
     }
 
@@ -108,14 +108,14 @@ BesselI0(const double z)
 {
     int         k, iters = 200;
     double      term, sum;
-    
+
     sum = 0.0;
     for (k = 0; k < iters; ++k)
     {
         term = pow(0.25*z*z, k) / (Factorial(k) * tgamma(k + 1));
         sum += term;
 
-        if (fabs(term < FLT_EPSILON))
+        if (fabs(term) < FLT_EPSILON)
             break;
     }
 
@@ -128,14 +128,14 @@ BesselI1(const double z)
 {
     int         k, iters = 200;
     double      term, sum;
-    
+
     sum = 0.0;
     for (k = 0; k < iters; ++k)
     {
         term = pow(0.25*z*z, k) / (Factorial(k) * tgamma(k + 2));
         sum += term;
 
-        if (fabs(term < FLT_EPSILON))
+        if (fabs(term) < FLT_EPSILON)
             break;
     }
 
@@ -242,7 +242,7 @@ bessi0(const double x)
     else
     {
         y = 3.75 / ax;
-        ans = (exp(ax)/sqrt(ax))*(0.39894228 + y*(0.1328592e-1 
+        ans = (exp(ax)/sqrt(ax))*(0.39894228 + y*(0.1328592e-1
               + y*(0.225319e-2 + y*(-0.157565e-2 + y*(0.916281e-2
               + y*(-0.2057706e-1 + y*(0.2635537e-1 + y*(-0.1647633e-1
               + y*0.392377e-2))))))));
