@@ -1,9 +1,9 @@
 /***********************************************************************
-*  -/_|:|_|_\- 
+*  -/_|:|_|_\-
 *                                                                      *
 *  dssplite v.0.8                                                      *
 *                                                                      *
-*  Copyright (C) 2003-2014  Douglas L. Theobald                        *
+*  Copyright (C) 2003-2015  Douglas L. Theobald                        *
 *                                                                      *
 *  dssplite is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published   *
@@ -65,7 +65,7 @@ vCross(Vector x, Vector y, Vector z);
 /* static double */
 /* vLength(Vector u); */
 
-static double 
+static double
 Atan2(double y, double x);
 
 static double
@@ -75,14 +75,32 @@ Cosangle(double **Ad, int An,
          double **Dd, int Dn);
 
 
+static void
+MyFree(void *p)
+{
+    if (p)
+    {
+        free(p);
+        p = NULL;
+    }
+//     else
+//     {
+//         fprintf(stderr, "\nERROR623: trying to free NULL ptr -- %s:%d:%p\n\n",
+//                 __FILE__, __LINE__, p);
+//         fflush(NULL);
+//         //exit(EXIT_FAILURE);
+//     }
+}
+
+
 char
 *dssplite_interleave(double *coords,
                      char **resName, int *resSeq, char **name,
                      int len)
 {
     int             i;
-    double         *x, *y, *z;
-    char           *summary;
+    double         *x = NULL, *y = NULL, *z = NULL;
+    char           *summary = NULL;
 
     x = (double *) malloc(len * sizeof(double));
     y = (double *) malloc(len * sizeof(double));
@@ -117,8 +135,8 @@ char
               int len)
 {
     int             i;
-    double         *x, *y, *z;
-    char           *summary;
+    double         *x = NULL, *y = NULL, *z = NULL;
+    char           *summary = NULL;
 
     x = (double *) malloc(len * sizeof(double));
     y = (double *) malloc(len * sizeof(double));
@@ -154,8 +172,11 @@ GetResidueLen(char **name, int vlen)
 
     len = 0;
     for (i = 0; i < vlen; ++i)
-        if (strstr(name[i], "CA ") != NULL)
+    {
+        //printf("\'%s\'\n", name[i]);
+        if (strstr(name[i], "CA "))
             ++len;
+    }
 
     return(len);
 }
@@ -164,11 +185,12 @@ GetResidueLen(char **name, int vlen)
 char
 *dssplite(double *x, double *y, double *z,
           char **resName, int *resSeq, char **name,
-          int vlen) /* used to be that len = # of CA atoms, not full x vector length */
+          int vlen) /* vlen = # of atoms, full x vector length, len = # CA atoms */
 {
     DSSP           *dssp = NULL;
+    int             i, j;
     int             len;
-    char           *summary;
+    char           *summary = NULL;
 
     /* Setup DSSP structure */
     dssp = DSSPinit();
@@ -182,6 +204,15 @@ char
     len = GetResidueLen(name, vlen);
     DSSPalloc(dssp, len);
 
+    for(i = 0, j = 0; i < vlen; ++i)
+    {
+        if (strncmp(name[i], "CA ", 3) == 0)
+        {
+            strncpy(dssp->residue[j], resName[i], 4);
+            ++j;
+        }
+    }
+
     /* do the DSSP algorithm, Doug-style */
     GetCONHCA(dssp);
     FlagHBonds(dssp);
@@ -195,7 +226,13 @@ char
     Find4Helices(dssp);
 
 /* PrintCONHCA(dssp); */
-PrintSummary(dssp);
+//PrintSummary(dssp);
+
+    for (i = 0; i < len; ++i)
+        putchar(dssp->summary[i]);
+        /* printf("\n DSSP Summary [%3d] = %2c ", i, dssp->summary[i]); */
+    putchar('\n');
+    putchar('\n');
 
     summary = (char *) malloc(len * sizeof(char));
     strncpy(summary, dssp->summary, len);
@@ -375,7 +412,7 @@ FlagBridges(DSSP *dssp)
                 dssp->bridgeP[i][dssp->bridgePn[i]] = j;
                 ++dssp->bridgePn[i];
                 dssp->summary[i] = 'B';
-                
+
             }
 
             if (   (TestHBond(dssp, i, j)     && TestHBond(dssp, j, i))
@@ -407,7 +444,7 @@ FindStrands(DSSP *dssp)
             {
                 for (k = 0; k < dssp->bridgePn[i+1]; ++k) /* for each bridge 'k' made by residue 'i+1' */
                 {
-                    if (fabs(dssp->bridgeP[i+1][j] - dssp->bridgeP[i][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
+                    if (abs(dssp->bridgeP[i+1][j] - dssp->bridgeP[i][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
                     {
                         for (m = dssp->bridgeP[i][k]; m <= dssp->bridgeP[i+1][j]; ++m)
                             ss[m] = 'E';
@@ -424,7 +461,7 @@ FindStrands(DSSP *dssp)
             {
                 for (k = 0; k < dssp->bridgePn[i+2]; ++k) /* for each H-bond 'k' of bridge residue 'i+1' */
                 {
-                    if (fabs(dssp->bridgeP[i+2][j] - dssp->bridgeP[i][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
+                    if (abs(dssp->bridgeP[i+2][j] - dssp->bridgeP[i][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
                     {
                         for (m = dssp->bridgeP[i][k]; m <= dssp->bridgeP[i+2][j]; ++m)
                             ss[m] = 'E';
@@ -442,7 +479,7 @@ FindStrands(DSSP *dssp)
             {
                 for (k = 0; k < dssp->bridgeAn[i+1]; ++k) /* for each H-bond 'k' of bridge residue 'i+1' */
                 {
-                    if (fabs(dssp->bridgeA[i][j] - dssp->bridgeA[i+1][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
+                    if (abs(dssp->bridgeA[i][j] - dssp->bridgeA[i+1][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
                     {
                         for (m = dssp->bridgeA[i+1][k]; m <= dssp->bridgeA[i][j]; ++m)
                             ss[m] = 'E';
@@ -459,7 +496,7 @@ FindStrands(DSSP *dssp)
             {
                 for (k = 0; k < dssp->bridgeAn[i+2]; ++k) /* for each bridge 'k' made by residue 'i+2' */
                 {
-                    if (fabs(dssp->bridgeA[i][j] - dssp->bridgeA[i+2][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
+                    if (abs(dssp->bridgeA[i][j] - dssp->bridgeA[i+2][k]) < 6.0) /* connect gaps of 4 or less on strand 2 */
                     {
                         for (m = dssp->bridgeA[i+2][k]; m <= dssp->bridgeA[i][j]; ++m)
                             ss[m] = 'E';
@@ -581,7 +618,7 @@ GetCONHCA(DSSP *dssp)
     /* This is Kabsch & Sanders' quick method (buried in the DsspCMBI.c source).
        It assumes trans, planar peptides and it makes the NH bond
        exactly 1.0 angstroms long (which is correct, BTW).
-       It takes advantage of the fact that the OC (from the residue before) 
+       It takes advantage of the fact that the OC (from the residue before)
        and the NH bond vectors have the same orientation and direction,
        given the above assumptions. Dividing by the OC distance normalizes
        the OC vector to 1.0 angstroms,
@@ -722,7 +759,7 @@ vLength(Vector u)
 /*////////////////////////////////////////////////////////////*/
 /* Math utils                                                 */
 /*////////////////////////////////////////////////////////////*/
-static double 
+static double
 Atan2(double y, double x)
 {
     double          z;
@@ -773,13 +810,13 @@ Cosangle(double **Ad, int An,
         return (vDot(u, v) / sqrt(x));
     else
         return (0.0);
-}   
+}
 
 
 /* Calculates torsion angle of a set of 4 atoms a1-a2-a3-a4. Dihedralangle
    is the angle between the projection of a1-a2 and the projection of a4-a3
    onto a plane normal to bond a2-a3. */
-double 
+double
 Dihedralangle(double **Ad, int An,
               double **Bd, int Bn,
               double **Cd, int Cn,
@@ -830,7 +867,7 @@ DSSP
 {
     DSSP        *dssp = NULL;
 
-    dssp = (DSSP *) malloc(sizeof(DSSP));
+    dssp = (DSSP *) calloc(1, sizeof(DSSP));
 
     dssp->C = dssp->O = dssp->N = dssp->H = NULL;
     dssp->Hbond = NULL;
@@ -850,9 +887,10 @@ void
 DSSPalloc(DSSP *dssp, int len)
 {
     int             i;
-    int            *bondatmspace, *b1space, *b2space;
+    int            *bondatmspace = NULL, *b1space = NULL, *b2space = NULL;
     char           *residue_space = NULL;
-    double         *Cspace, *Ospace, *Nspace, *Hspace, *CAspace;
+    double         *Cspace = NULL, *Ospace = NULL, *Nspace = NULL, *Hspace = NULL,
+                   *CAspace = NULL;
 
     dssp->len = len;
 
@@ -973,41 +1011,47 @@ DSSPalloc(DSSP *dssp, int len)
 void
 DSSPfree(DSSP **dssp_ptr)
 {
-    DSSP *dssp = *dssp_ptr;
+    if (dssp_ptr)
+    {
+        DSSP *dssp = *dssp_ptr;
 
-    free(dssp->C[0]);
-    free(dssp->C);
-    free(dssp->O[0]);
-    free(dssp->O);
-    free(dssp->N[0]);
-    free(dssp->N);
-    free(dssp->H[0]);
-    free(dssp->H);
-    free(dssp->CA[0]);
-    free(dssp->CA);
+        if (dssp)
+        {
+            MyFree(dssp->C[0]);
+            MyFree(dssp->C);
+            MyFree(dssp->O[0]);
+            MyFree(dssp->O);
+            MyFree(dssp->N[0]);
+            MyFree(dssp->N);
+            MyFree(dssp->H[0]);
+            MyFree(dssp->H);
+            MyFree(dssp->CA[0]);
+            MyFree(dssp->CA);
 
-    free(dssp->bondatm[0]);
-    free(dssp->bondatm);
+            MyFree(dssp->bondatm[0]);
+            MyFree(dssp->bondatm);
 
-    free(dssp->residue[0]);
-    free(dssp->residue);
+            MyFree(dssp->residue[0]);
+            MyFree(dssp->residue);
 
-    free(dssp->Hbond);
-    free(dssp->bridgeP[0]);
-    free(dssp->bridgeP);
-    free(dssp->bridgeA[0]);
-    free(dssp->bridgeA);
-    free(dssp->bridgePn);
-    free(dssp->bridgeAn);
-    free(dssp->bend);
-    free(dssp->turn5);
-    free(dssp->turn4);
-    free(dssp->turn3);
-    free(dssp->cys);
-    free(dssp->disulph);
-    free(dssp->summary);
-    free(dssp);
-    *dssp_ptr = NULL;
+            MyFree(dssp->Hbond);
+            MyFree(dssp->bridgeP[0]);
+            MyFree(dssp->bridgeP);
+            MyFree(dssp->bridgeA[0]);
+            MyFree(dssp->bridgeA);
+            MyFree(dssp->bridgePn);
+            MyFree(dssp->bridgeAn);
+            MyFree(dssp->bend);
+            MyFree(dssp->turn5);
+            MyFree(dssp->turn4);
+            MyFree(dssp->turn3);
+            MyFree(dssp->cys);
+            MyFree(dssp->disulph);
+            MyFree(dssp->summary);
+            MyFree(dssp);
+            dssp = NULL;
+        }
+    }
 }
 
 
@@ -1064,7 +1108,7 @@ PrintSummary(DSSP *dssp)
 
 
 /*
-static void 
+static void
 h_vect(float *a, float *b, float cosa, float sina, float *c)
 {
     *(c) = *(a) * cosa + *(b) * sina;
@@ -1072,7 +1116,7 @@ h_vect(float *a, float *b, float cosa, float sina, float *c)
     *(c + 2) = *(a + 2) * cosa + *(b + 2) * sina;
 }
 
-static void 
+static void
 vect(float *a, float *b, float *c)
 {
     *(a) = *(b) - *(c);
@@ -1083,7 +1127,7 @@ vect(float *a, float *b, float *c)
 
 /* calculate position of main chain hydrogens */
 /*
-int 
+int
 H_calc(int nres, XB xb[], char seq[])
 {
     int             i, k;
