@@ -1,47 +1,24 @@
-/******************************************************************************
- *  -/_|:|_|_\- 
- *
- *  File:           msa.c
- *
- *  Function:       
- *
- *  Author(s):      Douglas L. Theobald
- *                  Biochemistry Department
- *                  Brandeis University
- *                  MS 009
- *                  415 South St
- *                  Waltham, MA  02454-9110
- *
- *                  dtheobald@gmail.com
- *                  dtheobald@brandeis.edu
- *
- *  Copyright:      Copyright (c) 2004-2014 Douglas L. Theobald
- *
- *  msa.c is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published
- *  by the Free Software Foundation; either version 2 of the License,
- *  or (at your option) any later version.
- *
- *  msa.c is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with msa.c in the file 'COPYING'; if not, write to the:
- *
- *  Free Software Foundation, Inc.,
- *  59 Temple Place, Suite 330,
- *  Boston, MA  02111-1307  USA
- *
- *  Source:         started anew.
- *
- *  Notes:          
- *
- *  Change History:
- *          9/19/04 9:33 PM   Started source
- *  
- *****************************************************************************/
+/*
+    MSA - multiple sequence alignment utils
+
+    Copyright (C) 2004-2015 Douglas L. Theobald
+
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
+
+    -/_|:|_|_\-
+*/
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -75,7 +52,7 @@ MSA
         perror("\n ERROR");
         fprintf(stderr, "\n ERROR5: file \"%s\" not found. \n\n", msafile_name);
         exit(EXIT_FAILURE);
-    } 
+    }
 
     fgets(line, 255, msafile);
     fclose(msafile);
@@ -86,7 +63,7 @@ MSA
     }
     else if (strncmp(line, "CLUSTAL", 7) == 0)
     {
-        readaln(msafile_name, msa); 
+        readaln(msafile_name, msa);
     }
     else if (strncmp(line, "#NEXUS", 6) == 0)
     {
@@ -107,11 +84,11 @@ MSA
     else if (strncmp(line, "!!AA_MULTIPLE_ALIGNMENT", 23) == 0)
     {
         readmsf(msafile_name, msa);
-    } 
+    }
     else if (ismsf(msafile_name) != 0)
     {
         readmsf(msafile_name, msa);
-    } 
+    }
     else
     {
         fprintf(stderr,
@@ -137,7 +114,7 @@ isnexdist(char *msafile_name)
         perror("\n ERROR");
         fprintf(stderr, "\n ERROR5: file \"%s\" not found. \n\n", msafile_name);
         exit(EXIT_FAILURE);
-    } 
+    }
 
     line = (char *) malloc(256 * sizeof(char));
 
@@ -147,11 +124,11 @@ isnexdist(char *msafile_name)
         fgets(line, 255, msafile);
         strtolower(line);
         if (strstr(line, "begin") && strstr(line, "distances"))
-            return(linenum);   
+            return(linenum);
     }
 
     fclose(msafile);
-    free(line);
+    MyFree(line);
     line = NULL;
 
     return(0);
@@ -171,7 +148,7 @@ ismsf(char *msafile_name)
         perror("\n ERROR");
         fprintf(stderr, "\n ERROR5: file \"%s\" not found. \n\n", msafile_name);
         exit(EXIT_FAILURE);
-    } 
+    }
 
     line = (char *) malloc(256 * sizeof(char));
 
@@ -180,11 +157,11 @@ ismsf(char *msafile_name)
     {
         fgets(line, 255, msafile);
         if (strstr(line, "MSF:"))
-            return(linenum);     
+            return(linenum);
     }
 
     fclose(msafile);
-    free(line);
+    MyFree(line);
     line = NULL;
 
     return(0);
@@ -194,7 +171,10 @@ ismsf(char *msafile_name)
 MSA
 *MSAinit(void)
 {
-    MSA        *msa = malloc(sizeof(MSA));
+    MSA        *msa = calloc(1, sizeof(MSA));
+
+    msa->filename  = calloc(FILENAME_MAX, sizeof(char));
+    msa->filename[0] = '\0';
 
     msa->seq      = NULL;
     msa->name     = NULL;
@@ -238,11 +218,11 @@ MSAalloc(MSA *msa, int seqnum, int alignment_len, int name_size)
 
     for (i = 0; i < seqnum; ++i)
     {
-        msa->seq[i] = (char *) malloc((alignment_len+2) * sizeof(char));
+        msa->seq[i] = (char *) calloc((alignment_len+2), sizeof(char));
         msa->seq[i][0] = '\0';
-        msa->seq[i][alignment_len] = '\0';
-        msa->name[i] = (char *) malloc((name_size+2) * sizeof(char));
-        msa->name[i][name_size] = '\0';
+        //msa->seq[i][alignment_len] = '\0';
+        msa->name[i] = (char *) calloc((name_size+2), sizeof(char));
+        msa->name[i][0] = '\0';
     }
 
     /* these should never be modified */
@@ -275,18 +255,15 @@ DISTfree(MSA *msa)
 {
     int         i;
 
-    if (msa->dist != NULL)
+    if (msa->dist)
     {
         for (i = 0; i < msa->seqnum; ++i)
         {
-            if (msa->dist[i] != NULL)
-            {
-                free(msa->dist[i]);
-                msa->dist[i] = NULL;
-            }
+            MyFree(msa->dist[i]);
+            msa->dist[i] = NULL;
         }
 
-        free(msa->dist);
+        MyFree(msa->dist);
         msa->dist = NULL;
     }
 }
@@ -295,10 +272,10 @@ DISTfree(MSA *msa)
 MSAA
 *MSAAalloc(int arraynum)
 {
-    MSAA       *array = malloc(sizeof(MSAA));
+    MSAA       *array = calloc(1, sizeof(MSAA));
     int         i;
 
-    array->array = malloc(arraynum * sizeof(MSA *));
+    array->array = calloc(arraynum, sizeof(MSA *));
     array->flag  = calloc(arraynum,  sizeof(int));
 
     if (array->array == NULL || array->flag  == NULL)
@@ -319,101 +296,90 @@ MSAA
 
 
 void
-MSAfree(MSA **msa_ptr)
+MSAdestroy(MSA **msa_ptr)
 {
-    MSA *msa = *msa_ptr;
-    int         i;
-
-    if (msa != NULL)
+    if (msa_ptr)
     {
-        if(msa->distance == 1)
-            DISTfree(msa);
+        MSA *msa = *msa_ptr;
 
-        if (msa->seq != NULL)
+        if (msa)
         {
-            for (i = 0; i < msa->seqnum; ++i)
+            int         i;
+
+            MyFree(msa->filename);
+            msa->filename = NULL;
+
+            if(msa->distance == 1)
+                DISTfree(msa);
+
+            if (msa->seq)
             {
-                if (msa->seq[i] != NULL)
+                for (i = 0; i < msa->allocnum; ++i)
                 {
-                    free(msa->seq[i]);
-                    msa->seq[i] = NULL;
+                    if (msa->seq[i])
+                    {
+                        MyFree(msa->seq[i]);
+                        msa->seq[i] = NULL;
+                    }
                 }
+
+                MyFree(msa->seq);
+                msa->seq = NULL;
             }
 
-            free(msa->seq);
-            msa->seq = NULL;
-        }
-
-        if (msa->name != NULL)
-        {
-            for (i = 0; i < msa->seqnum; ++i)
+            if (msa->name)
             {
-                if (msa->name[i] != NULL)
+                for (i = 0; i < msa->seqnum; ++i)
                 {
-                    free(msa->name[i]);
+                    MyFree(msa->name[i]);
                     msa->name[i] = NULL;
                 }
+
+                MyFree(msa->name);
+                msa->name = NULL;
             }
 
-            free(msa->name);
-            msa->name = NULL;
-        }
-
-        if (msa->checksum != NULL)
-        {
-            free(msa->checksum);
+            MyFree(msa->checksum);
             msa->checksum = NULL;
-        }
 
-        if (msa->flag != NULL)
-        {
-            free(msa->flag);
+            MyFree(msa->flag);
             msa->flag = NULL;
-        }
 
-        if (msa->flag2 != NULL)
-        {
-            free(msa->flag2);
+            MyFree(msa->flag2);
             msa->flag2 = NULL;
-        }
 
-        free(msa);
-        *msa_ptr = NULL;
+            MyFree(msa);
+            msa = NULL;
+        }
     }
 }
 
 
 /* free an array of MSAs */
 void
-MSAAfree(MSAA **msaa_ptr)
+MSAAdestroy(MSAA **msaa_ptr)
 {
     MSAA *msaa = *msaa_ptr;
     int             i;
 
-    if (msaa != NULL)
+    if (msaa)
     {
-        if (msaa->array != NULL)
+        if (msaa->array)
         {
             for (i = 0; i < msaa->arraynum; ++i)
             {
-                if (msaa->array[i] != NULL)
-                {
-                    MSAfree(&(msaa->array[i]));
-                    msaa->array[i] = NULL;
-                }
+                MSAdestroy(&(msaa->array[i]));
+                msaa->array[i] = NULL;
             }
 
-            free(msaa->array);
+            MyFree(msaa->array);
             msaa->array = NULL;
         }
 
-        if (msaa->flag != NULL)
-        {
-            free(msaa->flag);
-            msaa->flag = NULL;
-        }
+        MyFree(msaa->flag);
+        msaa->flag = NULL;
 
-        free(msaa);
+        MyFree(msaa);
         *msaa_ptr = NULL;
     }
 }
@@ -428,13 +394,16 @@ MSAprint(MSA *msa)
     int             i;
 
     printf("\n msa->seqnum = %d \n", msa->seqnum);
+    printf(  " msa->seqlen = %d \n\n", msa->seqlen);
 
     for(i = 0; i < msa->seqnum; ++i)
     {
         if(msa->flag[i] == 0)
             printf("%3d: %-16.16s %s\n", i, msa->name[i], msa->seq[i]);
     }
+
     printf("\n");
+    fflush(NULL);
 }
 
 
@@ -504,7 +473,7 @@ MSA
             newmsa->seq[i][j] = msa->seq[i][(int)floor(j / multiply)];
     }
 
-    MSAfree(&msa);
+    MSAdestroy(&msa);
     msa = newmsa;
     return(newmsa);
 }
@@ -602,23 +571,27 @@ MSA
 *MSAdeldups(MSA *msa)
 {
     int             i, j, k;
+    const int       seqnum = msa->seqnum;
+    const int       seqlen = msa->seqlen;
+    int            *flag = msa->flag;
+    const char    **seq = (const char **) msa->seq;
 
-    for(i = 0; i < msa->seqnum; ++i)
+    for(i = 0; i < seqnum; ++i)
     {
-        if(msa->flag[i] == 0) /* if already deleted, don't need to consider it */
+        if(flag[i] == 0) /* if already deleted, don't need to consider it */
         {
             for(j = 0; j < i; ++j)
             {
-                if(msa->flag[j] == 0) /* if already deleted, don't need to consider it */
+                if(flag[j] == 0) /* if already deleted, don't need to consider it */
                 {
-                    for(k = 0; k < msa->seqlen; ++k)
+                    for(k = 0; k < seqlen; ++k)
                     {
-                        if(msa->seq[i][k] != msa->seq[j][k])
+                        if(seq[i][k] != seq[j][k])
                             break;
                     }
 
-                    if(k == msa->seqlen)
-                        msa->flag[j] = 1;
+                    if(k == seqlen)
+                        flag[j] = 1;
                 }
             }
         }
@@ -744,7 +717,7 @@ MSA
     tmpmsa->seqlen = m;
 
     MSAcpy(msa, tmpmsa);
-    MSAfree(&tmpmsa);
+    MSAdestroy(&tmpmsa);
 
     return(msa);
 }
@@ -764,7 +737,7 @@ MSA
     {
         for (j = 0; j < msa->seqnum; ++j)
         {
-            if (msa->seq[j][i] != '-')
+            if (msa->flag[j] == 0 && msa->seq[j][i] != '-')
                 break;
         }
 
@@ -781,7 +754,7 @@ MSA
     tmpmsa->seqlen = m;
 
     MSAcpy(msa, tmpmsa);
-    MSAfree(&tmpmsa);
+    MSAdestroy(&tmpmsa);
 
     return(msa);
 }
@@ -818,7 +791,7 @@ MSA
     tmpmsa->seqlen = m - 1;
 
     MSAcpy(msa, tmpmsa);
-    MSAfree(&tmpmsa);
+    MSAdestroy(&tmpmsa);
 
     return(msa);
 }
@@ -972,8 +945,8 @@ MSAcombine2(MSAA *msaa, int msa1_i, int msa2_i, int seq1_i, int seq2_i, int msao
     maxlen = len1 + len2;
     seqnum = num1 + num2;
 
-    if (msao != NULL)
-        MSAfree(&msao);
+    if (msao)
+        MSAdestroy(&msao);
 
     msao = MSAinit();
     MSAalloc(msao, seqnum, maxlen, SEQ_NAME_LENGTH);
@@ -1134,6 +1107,8 @@ reada2m(char *msafile_name, MSA *msa)
         exit(EXIT_FAILURE);
     }
 
+    strncpy(msa->filename, msafile_name, strlen(msafile_name));
+
     /* get the number of sequences in the a2m file  */
     msa->allocnum = 0;
     do
@@ -1182,7 +1157,7 @@ reada2m(char *msafile_name, MSA *msa)
         }
 
         msa->name[i][j] = '\0';
-        if (strlen(msa->name[i]) < 1) 
+        if (strlen(msa->name[i]) < 1)
         {
             fprintf(stderr, "\n ERROR7: sequence %d in a2m file has no name. \n\n", i+1);
             exit(EXIT_FAILURE);
@@ -1267,7 +1242,7 @@ reada2m2(char *msafile_name, MSA *msa)
         }
 
         msa->name[i][j] = '\0';
-        if (strlen(msa->name[i]) < 1) 
+        if (strlen(msa->name[i]) < 1)
         {
             fprintf(stderr, "\n ERROR7: sequence %d in a2m file has no name. \n\n", i+1);
             exit(EXIT_FAILURE);
@@ -1289,7 +1264,7 @@ reada2m2(char *msafile_name, MSA *msa)
         /* at this point last character should be just past the first ">" of the next sequence */
     }
 
-    free(filestr);
+    MyFree(filestr);
 }
 
 
@@ -1316,7 +1291,7 @@ readaln(char *msafile_name, MSA *msa)
         perror("\n ERROR");
         fprintf(stderr, "\n ERROR15: file \"%s\" not found. \n\n", msafile_name);
         exit(EXIT_FAILURE);
-    } 
+    }
     rewind(msafile);
 
     strncpy(msa->filename, msafile_name, strlen(msafile_name));
@@ -1368,7 +1343,7 @@ readaln(char *msafile_name, MSA *msa)
     fseek(msafile, seq_start, SEEK_SET); /* move to beginning of name0, block0 */
     line_len   = 0;                      /* count sequence length in first block */
     whitespace = 0;
-    do    
+    do
     {
         ch = getc(msafile);
         if (ch == EOF)
@@ -1396,7 +1371,7 @@ readaln(char *msafile_name, MSA *msa)
     MSAalloc(msa, msa->allocnum, alignment_len, 32);
 
     /* allocate space for a line from the alignment (if alignment is not interleaved, this could be very large) */
-    free(line);
+    MyFree(line);
     line = (char *) malloc((line_len + 3) * sizeof(char));
     buff = (char *) malloc((block_seq_len + 3) * sizeof(char));
     if (line == NULL || buff == NULL)
@@ -1420,8 +1395,6 @@ readaln(char *msafile_name, MSA *msa)
             strcat(msa->seq[j], buff);
         }
 
-        /* The underbar '_' is my own metacharacter, not clustal's -- I use it in the 
-           MarkovPolo mod_perl module as a sub for weird tainted metacharacters */
         do
         {
             ch = getc(msafile);
@@ -1438,14 +1411,14 @@ readaln(char *msafile_name, MSA *msa)
 //    exit(1);
 
     fclose(msafile);
-    free(line);
-    free(buff);
+    MyFree(line);
+    MyFree(buff);
 }
 
 
 /* readnex() -
    This monster parses NEXUS files! Woohoo!!
-   It seems to work, and somewhat generally. 
+   It seems to work, and somewhat generally.
    It currently cannot handle nested comments. */
 void
 readnex(char *msafile_name, MSA *msa)
@@ -1496,7 +1469,7 @@ readnex(char *msafile_name, MSA *msa)
             break;
         if(ch == '[') /* 91 = '[' */
         {
-            while(ch != ']') /* this bonks on nested comments */ 
+            while(ch != ']') /* this bonks on nested comments */
             {
                 ch = getc(msafile);/* 93 = ']' */
                 ++j;
@@ -1512,7 +1485,7 @@ readnex(char *msafile_name, MSA *msa)
     if (i == 0)
     {
         fprintf(stderr, "\n ERROR332: NEXUS file does not contain data \n\n");
-        exit(EXIT_FAILURE);  
+        exit(EXIT_FAILURE);
     }
 
     strtolower(filestr);
@@ -1524,7 +1497,7 @@ readnex(char *msafile_name, MSA *msa)
     if(begin_data == NULL)
     {
         begin_data = strstr(filestr, "characters");
-        if(begin_data != NULL)
+        if(begin_data)
             begin_data = strchr(begin_data, ';') + 1;
 
         if(begin_data == NULL)
@@ -1594,7 +1567,7 @@ readnex(char *msafile_name, MSA *msa)
     /* HERE'S THE REAL DEAL! -- slurp in the matrix data                          */
 
     /* read in multiple blocks of data, if they exist --
-       check if the taxa names held in msa->name are the same as 
+       check if the taxa names held in msa->name are the same as
        from first block */
 
     i = 0;     /* matrix index (raw data string we are reading) */
@@ -1617,7 +1590,7 @@ readnex(char *msafile_name, MSA *msa)
             /* if this is the first pass, read the name into the MSA -
                if this is a subsequent pass (other interleaved blocks),
                check to see if the names match as they should */
-            if(block == 0) 
+            if(block == 0)
             {
                 k = 0;
                 /* read in the name of taxa j */
@@ -1656,7 +1629,7 @@ readnex(char *msafile_name, MSA *msa)
 
             /* skip over space between name and characters */
             while(isspace(matrix[i]))
-                ++i;  
+                ++i;
 
             /* read in the character data up until the newline */
             while((matrix[i] != '\r') &&
@@ -1672,7 +1645,7 @@ readnex(char *msafile_name, MSA *msa)
                     continue;
                 }
                 else if(matrix[i] == gap || /* replace NEXUS specified gap characters with a dash '-' */
-                        matrix[i] == '_')   /* The underbar '_' is my own metacharacter, not clustals' -- I use it in the */ 
+                        matrix[i] == '_')   /* The underbar '_' is my own metacharacter, not clustals' -- I use it in the */
                     msa->seq[j][l] = '-';   /* MarkovPolo mod_perl module as a sub for weird tainted metacharacters       */
                 else
                     msa->seq[j][l] = matrix[i];
@@ -1683,13 +1656,13 @@ readnex(char *msafile_name, MSA *msa)
             ++j; /* increment taxa index */
         }
         ++block;
-    }   
+    }
 
     msa->seqnum = msa->allocnum;
     msa->seqlen = msa->alloclen;
 
     fclose(msafile);
-    free(filestr);
+    MyFree(filestr);
 
     /* I've always wanted to do this */
     Hell:
@@ -1747,7 +1720,7 @@ readnexd(char *msafile_name, MSA *msa)
             break;
         if(ch == '[') /* 91 = '[' */
         {
-            while(ch != ']') /* this bonks on nested comments */ 
+            while(ch != ']') /* this bonks on nested comments */
             {
                 ch = getc(msafile);/* 93 = ']' */
                 ++j;
@@ -1763,7 +1736,7 @@ readnexd(char *msafile_name, MSA *msa)
     if (i == 0)
     {
         fprintf(stderr, "\n ERROR332: NEXUS file does not contain data \n\n");
-        exit(EXIT_FAILURE);  
+        exit(EXIT_FAILURE);
     }
 
     strtoupper(filestr);
@@ -1841,8 +1814,8 @@ readnexd(char *msafile_name, MSA *msa)
     msa->seqlen = msa->alloclen;
 
     fclose(msafile);
-    free(filestr);
-    free(triangle);
+    MyFree(filestr);
+    MyFree(triangle);
 }
 
 
@@ -1876,7 +1849,7 @@ nexus_dist(MSA *msa, char *filestr, int filesize, int format)
 
         msa->name[i][j+1] = '\0';
 
-        if (strlen(msa->name[i]) < 1) 
+        if (strlen(msa->name[i]) < 1)
         {
             fprintf(stderr, "\n ERROR7: sequence %d in a2m file has no name. \n\n", i+1);
             exit(EXIT_FAILURE);
@@ -1954,13 +1927,13 @@ nexus_dist(MSA *msa, char *filestr, int filesize, int format)
 /* ********************************************************************** */
 /* These four functions get key:value pairs of the form
    'key=value' or 'key = value', etc. from a string --
-   whitespace between key and value is ignored. 
+   whitespace between key and value is ignored.
    These are used in 'readnex()' for NEXUS format stuff.
-   Note the return types. 
+   Note the return types.
 
    The 'endptr' argument can be NULL, but if the value
-   is not found endptr = &string, so you can check. 
-   getvalc() only returns isgraph() characters 
+   is not found endptr = &string, so you can check.
+   getvalc() only returns isgraph() characters
    (no whitespace or control characters) */
 
 int
@@ -2004,7 +1977,7 @@ getvald(char *string, char *key, char delimiter, char **endptr)
           i < (long int) key_len)
         ++i;
 
-    val = strtod(&key_p[i], endptr); 
+    val = strtod(&key_p[i], endptr);
     if(*endptr == &key_p[i])
         endptr = &string;
 
@@ -2064,7 +2037,7 @@ getvalc(char *string, char *key, char delimiter, char **endptr)
 
     while((isspace(key_p[i]) || (key_p[i] == delimiter)) &&
           i < (long int) key_len)
-        ++i; 
+        ++i;
 
     if(isgraph(key_p[i]))
     {
@@ -2167,7 +2140,7 @@ readphylip(char *msafile_name, MSA *msa)
                 start = 0;
 
             //printf("\n%d %s", seqpos, &line[start]);
-            
+
             k = seqpos;
             for (i = start; i < strlen(line); ++i)
             {
@@ -2184,8 +2157,8 @@ readphylip(char *msafile_name, MSA *msa)
     msa->seqnum = msa->allocnum;
 
     fclose(msafile);
-    free(line);
-    free(buff);
+    MyFree(line);
+    MyFree(buff);
 }
 
 
@@ -2238,10 +2211,10 @@ readphylipd(char *msafile_name, MSA *msa)
         exit(EXIT_FAILURE);
     }
 
-    phylip_dist(msa, filestr, filesize, format);     
+    phylip_dist(msa, filestr, filesize, format);
 
-    free(filestr);
-    free(line);
+    MyFree(filestr);
+    MyFree(line);
 }
 
 
@@ -2269,7 +2242,7 @@ phylip_dist_format(MSA *msa, char *filestr, int filesize)
             break;
         while((isdigit(filestr[i]) ||
                filestr[i] == '-' ||
-               filestr[i] == '.' || 
+               filestr[i] == '.' ||
                filestr[i] == 'e') &&
               i < filesize)
             ++i;
@@ -2292,7 +2265,7 @@ phylip_dist_format(MSA *msa, char *filestr, int filesize)
             break;
         while((isdigit(filestr[i]) ||
                filestr[i] == '-' ||
-               filestr[i] == '.' || 
+               filestr[i] == '.' ||
                filestr[i] == 'e') &&
               i < filesize)
             ++i;
@@ -2334,7 +2307,7 @@ phylip_dist(MSA *msa, char *filestr, int filesize, int format)
         ++k;
     }
     ++k;
-    
+
     //printf("\nlast %d %c bbb", k, filestr[k]);
     //printf("\n\n");
     //fflush(NULL);
@@ -2354,7 +2327,7 @@ phylip_dist(MSA *msa, char *filestr, int filesize, int format)
             if(isspace(msa->name[i][m]) || msa->name[i][m] == '-')
                 msa->name[i][m] = '_';
 
-        if (strlen(msa->name[i]) < 1) 
+        if (strlen(msa->name[i]) < 1)
         {
             fprintf(stderr, "\n ERROR7: sequence %d in a2m file has no name. \n\n", i+1);
             exit(EXIT_FAILURE);
@@ -2423,14 +2396,14 @@ phylip_dist(MSA *msa, char *filestr, int filesize, int format)
         while(isspace(filestr[k]) && k < filesize)
             ++k;
     }
-    
+
 /*     printf("\n\n"); */
 /*     for (i = 0; i < msa->seqnum; ++i) */
 /*     { */
 /*         printf("\n%d %s", i, msa->name[i]); */
 /*     } */
 /*     fflush(NULL); */
-    
+
 }
 
 
@@ -2456,7 +2429,7 @@ writea2m_seq(MSA *msa, int seqID, char *outfile_root)
 
     writea2m(msa, 0, msa->seqlen, outfile_root);
 
-    for (i=0; i < msa->seqnum; ++i) 
+    for (i=0; i < msa->seqnum; ++i)
         msa->flag[i] = 0;
 }
 
@@ -2470,7 +2443,7 @@ writealn_seq(MSA *msa, int seqID, char *outfile_root)
         msa->flag[i] = 1;
     msa->flag[seqID] = 0;
     writealn(msa, 0, msa->seqlen, outfile_root);
-    for (i=0; i < msa->seqnum; ++i) 
+    for (i=0; i < msa->seqnum; ++i)
         msa->flag[i] = 0;
 }
 
@@ -2484,7 +2457,7 @@ writenex_seq(MSA *msa, int seqID, char *outfile_root)
         msa->flag[i] = 1;
     msa->flag[seqID] = 0;
     writenex(msa, 0, msa->seqlen, outfile_root);
-    for (i=0; i < msa->seqnum; ++i) 
+    for (i=0; i < msa->seqnum; ++i)
         msa->flag[i] = 0;
 }
 
@@ -2506,14 +2479,14 @@ writea2m(MSA *msa, int begin, int window, char *outfile_root)
         perror("\n ERROR");
         fprintf(stderr, "\n ERROR19: file \"%s\" not found. \n\n", outfile_name);
         exit(EXIT_FAILURE);
-    } 
+    }
 
-    /* see if the current sequence window is empty (matches "-" or "?" or ".") 
+    /* see if the current sequence window is empty (matches "-" or "?" or ".")
        if so, flag it so it won't be printed out to the alignment file        */
     for (i = 0; i < msa->seqnum; ++i)
     {
         msa->flag2[i] = 0; /* gotta make sure to reset it to 0 */
-        j = begin; 
+        j = begin;
         while((msa->seq[i][j] == '-' ||
                msa->seq[i][j] == '?' ||
                msa->seq[i][j] == '.') &&
@@ -2537,7 +2510,8 @@ writea2m(MSA *msa, int begin, int window, char *outfile_root)
         {
             fputc(msa->seq[k][j], outfile);
             ++linelen;
-            if (linelen == MAXLINE) 
+
+            if (linelen == MAXLINE-1)
             {
                 linelen = 0;
                 fputc('\n', outfile);
@@ -2558,7 +2532,7 @@ writealn(MSA *msa, int begin, int window, char *outfile_root)
     int            i, j, k, blocks;
     char          *outfile_name = NULL;
 
-    outfile_name = (char *) malloc(256 * sizeof(char)); 
+    outfile_name = (char *) malloc(256 * sizeof(char));
     if (outfile_name == NULL)
     {
         perror("\n ERROR");
@@ -2585,7 +2559,7 @@ writealn(MSA *msa, int begin, int window, char *outfile_root)
     {
         msa->flag2[i] = 0;
 
-        j = begin; 
+        j = begin;
         while((msa->seq[i][j] == '-' ||
                msa->seq[i][j] == '?' ||
                msa->seq[i][j] == '.') &&
@@ -2609,15 +2583,27 @@ writealn(MSA *msa, int begin, int window, char *outfile_root)
             if ((msa->flag2[j] == 1) || (msa->flag[j] == 1))
                 continue;
 
+            if (msa->seq[j][begin + i*MAXLINE] == '\0')
+            {
+                fputc('\n', outfile);
+                break;
+            }
+
             /* fprintf(outfile, "%-30.30s      ", msa->name[j]); */
             fprintf(outfile, "%-16.16s      ", msa->name[j]);
+            //printf("\n%d\n", j);fflush(NULL);
 
             for (k = begin + i*MAXLINE;
                  k < begin + window &&
                  k < msa->seqlen &&
-                 k < (begin + i*MAXLINE + MAXLINE);  
+                 k < (begin + i*MAXLINE + MAXLINE);
                  ++k)
             {
+                if (msa->seq[j][k] == '\0')
+                {
+                    fputc('\n', outfile);
+                    break;
+                }
                 fputc(msa->seq[j][k], outfile);
             }
             fputc('\n', outfile);
@@ -2627,11 +2613,10 @@ writealn(MSA *msa, int begin, int window, char *outfile_root)
 
     fputc('\n', outfile);
     fclose(outfile);
-    free(outfile_name);
+    MyFree(outfile_name);
 }
 
 
-
 void
 writephylip(MSA *msa, int begin, int window, char *outfile_root)
 {
@@ -2639,7 +2624,7 @@ writephylip(MSA *msa, int begin, int window, char *outfile_root)
     int            i, j, k, blocks;
     char          *outfile_name = NULL;
 
-    outfile_name = (char *) malloc(256 * sizeof(char)); 
+    outfile_name = (char *) malloc(256 * sizeof(char));
     if (outfile_name == NULL)
     {
         perror("\n ERROR");
@@ -2666,7 +2651,7 @@ writephylip(MSA *msa, int begin, int window, char *outfile_root)
     {
         msa->flag2[i] = 0;
 
-        j = begin; 
+        j = begin;
         while((msa->seq[i][j] == '-' ||
                msa->seq[i][j] == '?' ||
                msa->seq[i][j] == '.') &&
@@ -2697,7 +2682,7 @@ writephylip(MSA *msa, int begin, int window, char *outfile_root)
             for (k = begin + i*MAXLINE;
                  k < begin + window &&
                  k < msa->seqlen &&
-                 k < (begin + i*MAXLINE + MAXLINE);  
+                 k < (begin + i*MAXLINE + MAXLINE);
                  ++k)
             {
                 fputc(msa->seq[j][k], outfile);
@@ -2709,7 +2694,7 @@ writephylip(MSA *msa, int begin, int window, char *outfile_root)
 
     fputc('\n', outfile);
     fclose(outfile);
-    free(outfile_name);
+    MyFree(outfile_name);
 }
 
 
@@ -2721,7 +2706,7 @@ writenex(MSA *msa, int begin, int window, char *outfile_root)
     char          *outfile_name = NULL;
     char           seqtype[16];
 
-    outfile_name = (char *) malloc(256 * sizeof(char)); 
+    outfile_name = (char *) malloc(256 * sizeof(char));
     if (outfile_name == NULL)
     {
         perror("\n ERROR");
@@ -2746,7 +2731,7 @@ writenex(MSA *msa, int begin, int window, char *outfile_root)
     for (i = 0; i < msa->seqnum; ++i)
     {
         msa->flag2[i] = 0;
-        j = begin; 
+        j = begin;
         while((msa->seq[i][j] == '-' ||
                msa->seq[i][j] == '?' ||
                msa->seq[i][j] == '.') &&
@@ -2831,7 +2816,7 @@ writenex(MSA *msa, int begin, int window, char *outfile_root)
             for (k = begin + i*MAXLINE;
                  k < begin + window &&
                  k < msa->seqlen &&
-                 k < (begin + i*MAXLINE + MAXLINE);  
+                 k < (begin + i*MAXLINE + MAXLINE);
                  ++k)
             {
                 fputc(msa->seq[j][k], outfile);
@@ -2846,7 +2831,7 @@ writenex(MSA *msa, int begin, int window, char *outfile_root)
     fprintf(outfile, "End;\n\n");
 
     fclose(outfile);
-    free(outfile_name);
+    MyFree(outfile_name);
 }
 
 
@@ -2962,7 +2947,7 @@ writepsib(MSA *msa, int begin, int window, char *outfile_root)
         fputc('\n', outfile);
     }
     fclose(outfile);
-    free(outfile_name);
+    MyFree(outfile_name);
 }
 
 
@@ -2986,7 +2971,7 @@ scan_fasta_name(const char *fstname, char **subname, int nfield)
         items_read = sscanf(ptr, "%63[^|]%n", field, &n);
 
         printf("field[%d] = \"%s\"\n", cnt, field);
-        
+
         if (cnt == nfield)
         {
             printf("** field[%d] = \"%s\"\n", cnt, field);
@@ -3024,17 +3009,17 @@ writefasta_explode(MSA *msa, int nfield)
     for (i = 0; i < (unsigned long) msa->seqnum; ++i)
     {
         /* see if the current sequence window is empty (matches "-" or "?" or ".") */
-        j = 0; 
+        j = 0;
         while(msa->seq[i][j] == '-' || msa->seq[i][j] == '?' || msa->seq[i][j] == '.')
         {
-            ++j; 
+            ++j;
         }
 
         /* if so, don't write this file */
         if (j > strlen(msa->seq[i]))
             return;
 
-        //sscanf(&msa->name[i][0], "%s", outfile_name); /* "1" to get rid of initial > */ 
+        //sscanf(&msa->name[i][0], "%s", outfile_name); /* "1" to get rid of initial > */
         scan_fasta_name(&msa->name[i][0], &outfile_name, nfield);
         strcat(outfile_name, ".fst");
         outfile = fopen(outfile_name, "w");
@@ -3048,7 +3033,7 @@ writefasta_explode(MSA *msa, int nfield)
             {
                 putc(ch, outfile);
                 ++linelen;
-                if (linelen == FSTMAXLINE) 
+                if (linelen == FSTMAXLINE)
                 {
                     linelen = 0;
                     putc('\n', outfile);
@@ -3059,7 +3044,7 @@ writefasta_explode(MSA *msa, int nfield)
         putc('\n', outfile);
         fclose(outfile);
     }
-    
-    free(outfile_name);
+
+    MyFree(outfile_name);
 }
 
