@@ -1,37 +1,36 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
 
-const char aa1[] = "ARNDCQEGHILKMFPSTWYV";
-const char aa3[] = "ALAARGASNASPCYSGLNGLUGLYHISILELEULYSMETPHEPROSERTHRTRPTYRVAL";
+const char aa1[]  = "ARNDCQEGHILKMFPSTWYV";
+const char aa3[]  = "ALAARGASNASPCYSGLNGLUGLYHISILELEULYSMETPHEPROSERTHRTRPTYRVAL";
 const char aan1[] = "ARNDCQEGHILKMFPSTWYVACGTTACGTT";
 const char aan3[] = "ALAARGASNASPCYSGLNGLUGLYHISILELEULYSMETPHEPROSERTHRTRPTYRVAL  A  C  G  T  UADECYTGUATHYURA";
-const char ss1[] = "GHTBESC ";
+const char ss1[]  = "GHTBESC ";
 
 /* These are all in nats, *not* bits */
 
-const double aafreq_blosum50[20] = 
+const double aafreq_blosum50[20] =
 {
     0.077728, 0.051421, 0.041057, 0.052374, 0.024305,
     0.034190, 0.058888, 0.083353, 0.024669, 0.061626,
@@ -42,10 +41,10 @@ const double aafreq_blosum50[20] =
 
 /* Andreas Prlic, Francisco S. Domingues, and Manfred J. Sippl
    Structure-derived substitution matrices for alignment of distantly related sequences
-   Protein Eng. 2000 13: 545-550 
+   Protein Eng. 2000 13: 545-550
    http://peds.oxfordjournals.org/cgi/content/abstract/13/8/545
 */
-const double s_dssp8[8][8] = 
+const double s_dssp8[8][8] =
 {
     /*     G         H         T         B         E         S         C         X */
     { 5.4620,  -0.5337,   1.4210,  -1.0120,  -5.8987,   0.1594,  -0.8041,  -1.8230},
@@ -59,7 +58,7 @@ const double s_dssp8[8][8] =
 };
 
 
-const double s_simple[20][20] = 
+const double s_simple[20][20] =
 {
     {  1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0},
     { -1.0,  1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0},
@@ -83,7 +82,7 @@ const double s_simple[20][20] =
     { -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0,  1.0},
 };
 
-const double s_simpler[20][20] = 
+const double s_simpler[20][20] =
 {
     {  2.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0},
     {  1.0,  2.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0},
@@ -107,7 +106,7 @@ const double s_simpler[20][20] =
     {  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  2.0},
 };
 
-const double s_hsdm[20][20] = 
+const double s_hsdm[20][20] =
 {
     { 0.7625, -0.3105, -0.2454, -0.3299,  0.0624, -0.2994, -0.0652,  0.0873, -0.4173, -0.2384, -0.1511, -0.1691,  0.0222, -0.3355, -0.1539,  0.1761,  0.0832, -0.3618, -0.5850,  0.0222},
     {-0.3105,  1.1908,  0.0333, -0.0457, -0.8720, -0.1026,  0.3923, -0.4700,  0.0970, -0.5448, -0.3923,  0.5393, -0.1982, -0.6044,  0.1816, -0.0693,  0.0471,  0.1414, -0.1400, -0.5268},
@@ -131,7 +130,7 @@ const double s_hsdm[20][20] =
     { 0.0222, -0.5268, -0.7833, -0.8456,  0.1830, -0.6890, -0.5864, -0.7375, -0.2260,  0.7250,  0.3161, -0.4949,  0.1636,  0.0721, -0.3202, -0.3729, -0.0319, -0.2953,  0.0915,  0.7320},
 };
 
-const double s_blosum30[20][20] = 
+const double s_blosum30[20][20] =
 {
     { 0.5433, -0.1314, -0.0386,  0.0665, -0.3513,  0.1260, -0.0648, -0.0597, -0.2770, -0.0568, -0.2036, -0.0308,  0.0746, -0.2087, -0.1370,  0.1345,  0.1145, -0.6291, -0.5876,  0.1343 },
     {-0.1314,  1.1549, -0.2940, -0.1896, -0.3037,  0.3897, -0.1658, -0.2929, -0.1807, -0.4233, -0.3324,  0.1919, -0.0399, -0.1163, -0.1840, -0.0740, -0.3685, -0.0681,  0.0659, -0.2027 },
@@ -155,7 +154,7 @@ const double s_blosum30[20][20] =
     { 0.1343, -0.2027, -0.2364, -0.2637, -0.3219, -0.4239, -0.4269, -0.3522, -0.4565,  0.5325,  0.2056, -0.2794,  0.0677,  0.0977, -0.4887, -0.1533,  0.1497, -0.4021,  0.0699,  0.6756 },
 };
 
-const double s_blosum35[20][20] = 
+const double s_blosum35[20][20] =
 {
     { 0.7833, -0.1606, -0.1137, -0.1445, -0.2784, -0.0346, -0.1445,  0.0675, -0.3109, -0.1532, -0.2885, -0.0733, -0.0111, -0.3555, -0.2820,  0.1645,  0.0425, -0.3155, -0.2229,  0.0539},
     {-0.1606,  1.3240, -0.1634, -0.1867, -0.4878,  0.3270, -0.1608, -0.3771, -0.1614, -0.4456, -0.3184,  0.3143, -0.0125, -0.2021, -0.3783, -0.1094, -0.2962, -0.0399, -0.0828, -0.1901},
@@ -179,7 +178,7 @@ const double s_blosum35[20][20] =
     { 0.0539, -0.1901, -0.4299, -0.3912, -0.4266, -0.4735, -0.3422, -0.5316, -0.6224,  0.6462,  0.2822, -0.3174,  0.1212,  0.1038, -0.4425, -0.1957,  0.1558, -0.4124, -0.0395,  0.8074},
 };
 
-const double s_blosum40[20][20] = 
+const double s_blosum40[20][20] =
 {
     { 0.9119, -0.2743, -0.1296, -0.2441, -0.2707, -0.0494, -0.1384,  0.0932, -0.3821, -0.2251, -0.3190, -0.1296, -0.0902, -0.4743, -0.2940,  0.2366,  0.0049, -0.4468, -0.3354,  0.0417},
     {-0.2743,  1.4840, -0.0848, -0.2115, -0.5700,  0.3116, -0.1169, -0.5246, -0.0817, -0.5653, -0.3921,  0.5393, -0.1280, -0.3942, -0.5004, -0.1993, -0.2881, -0.3091, -0.1090, -0.3643},
@@ -203,7 +202,7 @@ const double s_blosum40[20][20] =
     { 0.0417, -0.3643, -0.5228, -0.5015, -0.3571, -0.5285, -0.4764, -0.6181, -0.7444,  0.7273,  0.3125, -0.3959,  0.1217, -0.0049, -0.5599, -0.2499,  0.1581, -0.6012, -0.1600,  0.9123},
 };
 
-const double s_blosum50[20][20] = 
+const double s_blosum50[20][20] =
 {
     { 1.1859, -0.4029, -0.3292, -0.4160, -0.1192, -0.1986, -0.2265,  0.0965, -0.4444, -0.3353, -0.4201, -0.2908, -0.1585, -0.6046, -0.3358,  0.3101, -0.0332, -0.7688, -0.5168, -0.0186},
     {-0.4029,  1.6823, -0.1581, -0.3878, -0.8490,  0.3055, -0.0239, -0.6597, -0.0186, -0.8317, -0.5906,  0.7250, -0.3886, -0.7727, -0.6565, -0.2175, -0.3058, -0.6371, -0.3186, -0.6685},
@@ -227,7 +226,7 @@ const double s_blosum50[20][20] =
     {-0.0186, -0.6685, -0.7695, -0.8400, -0.2124, -0.5845, -0.7359, -0.9095, -0.8828,  0.8384,  0.2809, -0.6486,  0.1933, -0.1512, -0.6455, -0.3876,  0.0582, -0.7239, -0.3184,  1.1370},
 };
 
-const double s_blosum62[20][20] = 
+const double s_blosum62[20][20] =
 {
     { 1.3618, -0.4899, -0.5305, -0.6077, -0.1416, -0.2786, -0.2994,  0.0553, -0.5633, -0.4581, -0.5076, -0.2544, -0.3241, -0.7659, -0.2822,  0.3867, -0.0157, -0.8757, -0.6114, -0.0656 },
     {-0.4899,  1.8969, -0.1524, -0.5565, -1.1746,  0.3406, -0.0400, -0.7986, -0.0866, -1.0363, -0.7467,  0.7309, -0.4738, -0.9657, -0.7308, -0.2651, -0.3890, -0.9286, -0.5870, -0.8673 },
@@ -251,7 +250,7 @@ const double s_blosum62[20][20] =
     {-0.0656, -0.8673, -0.9969, -1.0891, -0.2799, -0.7619, -0.8464, -1.0878, -1.0804,  0.8827,  0.2732, -0.7841,  0.2382, -0.2942, -0.8140, -0.5705, -0.0193, -0.9823, -0.4185,  1.3062 },
 };
 
-const double s_blosum100[20][20] = 
+const double s_blosum100[20][20] =
 {
     { 1.8726, -0.7745, -0.8255, -1.0539, -0.4822, -0.4817, -0.6184, -0.3267, -0.9615, -0.9110, -0.9206, -0.5365, -0.7933, -1.2362, -0.4993,  0.3219, -0.1797, -1.4372, -1.2231, -0.3471},
     {-0.7745,  2.3921, -0.4537, -1.0968, -1.8282,  0.0321, -0.5220, -1.3028, -0.2352, -1.5465, -1.3033,  0.7261, -0.8189, -1.4928, -1.0776, -0.6138, -0.8019, -1.5301, -1.1691, -1.4058},
