@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -42,10 +41,10 @@
 
 
 static int
-atom_selxn(char *name, int mode, char *useratoms);
+AtomSelxn2(char *name, int mode, char *useratoms);
 
 static int
-range_selxn(int chainID, char *chains, int resSeq, int *lower, int *upper, int range_num);
+RangeSelxn2(int chainID, char *chains, int resSeq, int *lower, int *upper, int range_num);
 
 static void
 XPLORcorrections(PDBCds *pdbcds, int record);
@@ -88,11 +87,34 @@ static const char atoms3[] = "CA :CB :P  :C1*:C1'";
 
 
 static int
-atom_selxn(char *name, int mode, char *useratoms)
+AltLocSelxn(int altLoc)
+{
+    if (altLoc == ' ' ||
+        altLoc == 'A' ||
+        altLoc == '1')
+        return(1);
+    else
+        return(0);
+}
+
+
+static int
+RecordSelxn(char *record)
+{
+    if (strncmp(record, "ATOM  ", 6) == 0 ||
+        strncmp(record, "HETATM", 6) == 0)
+        return(1);
+    else
+        return(0);
+}
+
+
+static int
+AtomSelxn2(char *name, int mode, char *useratoms)
 {
     const char     *atoms = NULL;
 
-    if (useratoms != NULL)
+    if (useratoms)
         mode = 10;
 
     if (mode == 4)
@@ -122,7 +144,7 @@ atom_selxn(char *name, int mode, char *useratoms)
             break;
     }
 
-    if (strstr(atoms, name) != NULL)
+    if (strstr(atoms, name))
         return (1);
     else
         return (0);
@@ -130,7 +152,7 @@ atom_selxn(char *name, int mode, char *useratoms)
 
 
 static int
-range_selxn(int chainID, char *chains, int resSeq, int *lower, int *upper, int range_num)
+RangeSelxn2(int chainID, char *chains, int resSeq, int *lower, int *upper, int range_num)
 {
     int             i;
 
@@ -252,7 +274,7 @@ ScanPDBLine(char *buff, PDBCds *cp, int j, int amber)
     //char            amber_atom_name = buff[12];
 
     endline = strpbrk(&buff[54], "\r\n\f");
-    if (endline != NULL) /* kill the newline, if there */
+    if (endline) /* kill the newline, if there */
         *endline = '\0';
 
     //if (!isdigit(amber_atom_name) && !isspace(amber_atom_name))
@@ -285,7 +307,7 @@ ScanPDBLine(char *buff, PDBCds *cp, int j, int amber)
         sscanf(&buff[76], "%2c",   cp->element[j]);   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
         sscanf(&buff[78], "%2c",   cp->charge[j]);    /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
     }
-    else /* standard PDB format http://www.rcsb.org/pdb/file_formats/pdb/pdbguide2.2/guide2.2_frame.html */
+    else /* standard PDB format http://www.wwpdb.org/documentation/format33/sect9.html#ATOMl */
 //     {
 //         /*        1         2         3         4         5         6         7         8
 //         012345678901234567890123456789012345678901234567890123456789012345678901234567890
@@ -367,7 +389,7 @@ ReadPDBCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum, int a
     pdbfile = fopen(pdbfile_name, "r");
     if (pdbfile == NULL)
     {
-        if (getenv("PDBDIR") != NULL)
+        if (getenv("PDBDIR"))
         {
             strncpy(pdbdir, getenv("PDBDIR"), FILENAME_MAX);
             strncpy(dirpdbfile_name, pdbdir, FILENAME_MAX - 1);
@@ -393,7 +415,7 @@ ReadPDBCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum, int a
         /* move to next model */
         if (modelnum > 1)
         {
-            while (fgets(buff, bufflen, pdbfile) != NULL)
+            while (fgets(buff, bufflen, pdbfile))
             {
                 if (strncmp(buff, "MODEL", 5) == 0 ||
                     strncmp(buff, "REMARK FRAME:", 13) == 0 ||
@@ -419,8 +441,7 @@ ReadPDBCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum, int a
             }
 
             /* if the line is an ATOM or HETATM record */
-            if (strncmp(buff, "ATOM  ", 6) == 0 ||
-                strncmp(buff, "HETATM", 6) == 0)
+            if (RecordSelxn(buff))
             {
                 ScanPDBLine(buff, cp, j, amber);
                 if (fix_atom_names)
@@ -438,7 +459,7 @@ ReadPDBCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum, int a
     }
 
     //PrintPDBCds(stdout, pdbA->cds[0]);
-    free(buff);
+    MyFree(buff);
     fclose(pdbfile);
 
     return(EXIT_SUCCESS);
@@ -452,7 +473,7 @@ ScanTPSLine(char *buff, PDBCds *cp, int j)
     int             numret;
 
 //     endline = strpbrk(&buff[54], "\r\n\f");
-//     if (endline != NULL) /* kill the newline, if there */
+//     if (endline) /* kill the newline, if there */
 //         *endline = '\0';
 
     strncpy(cp->record[j], "ATOM  ", 6);
@@ -504,7 +525,7 @@ ReadTPSCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum)
     pdbfile = fopen(pdbfile_name, "r");
     if (pdbfile == NULL)
     {
-        if (getenv("PDBDIR") != NULL)
+        if (getenv("PDBDIR"))
         {
             strncpy(pdbdir, getenv("PDBDIR"), FILENAME_MAX);
             strncpy(dirpdbfile_name, pdbdir, FILENAME_MAX - 1);
@@ -530,7 +551,7 @@ ReadTPSCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum)
         /* move to next model */
         if (modelnum > 1)
         {
-            while (fgets(buff, bufflen, pdbfile) != NULL)
+            while (fgets(buff, bufflen, pdbfile))
             {
                 if (strncmp(buff, "LM3", 3) == 0)
                 {
@@ -577,7 +598,7 @@ ReadTPSCds(char *pdbfile_name, PDBCdsArray *pdbA, int cds_i, int modelnum)
 
     //PrintPDBCds(stdout, pdbA->cds[0]);
 
-    free(buff);
+    MyFree(buff);
     fclose(pdbfile);
 
     return(EXIT_SUCCESS);
@@ -597,7 +618,7 @@ PDBCdsArray
     FILE                *tpsfile = NULL;
     PDBCdsArray         *pdbA = NULL;
 
-    if (getenv("PDBDIR") != NULL)
+    if (getenv("PDBDIR"))
         strncpy(pdbdir, getenv("PDBDIR"), FILENAME_MAX);
 
     vlen = last_vlen = 0;
@@ -638,7 +659,7 @@ PDBCdsArray
 
         models = 0;
         last_vlen = 0;
-        while (fgets(buff, (int) sizeof(buff), tpsfile) != NULL)
+        while (fgets(buff, (int) sizeof(buff), tpsfile))
         {
             if (strncmp(buff, "LM3", 3) == 0) // DLT
             {
@@ -687,7 +708,7 @@ PDBCdsArray
             vlen = 0;
             rewind(tpsfile);
 
-            while (fgets(buff, sizeof(buff), tpsfile) != NULL)
+            while (fgets(buff, sizeof(buff), tpsfile))
             {
                 if (strncmp(buff, "LM3", 3) == 0) // DLT
                 {
@@ -740,36 +761,36 @@ PDBCdsArray
         j += models_per_tps[i];
     }
 
-    free(models_per_tps);
-    free(len_array);
+    MyFree(models_per_tps);
+    MyFree(len_array);
 
     return(pdbA);
 }
 
 
 static int
-GetSlxnLen(PDBCdsArray *pdbA, const int crds, Algorithm *algo, char *chains,
+GetSlxnLen(PDBCdsArray *pdbA, const int j, Algorithm *algo, char *chains,
            int *lower, int *upper, const int range_num, int *selection_index, int rev)
 {
-    int            i, vlen;
+    int             i, vlen;
+    const PDBCds   *pdbj = pdbA->cds[j];
 
     vlen = 0;
-    for (i = 0; i < pdbA->cds[crds]->vlen; ++i)
+    for (i = 0; i < pdbj->vlen; ++i)
     {
 /*         printf("\n%s, %s, %c, %c, %d", */
-/*                pdbA->cds[crds]->record[i], */
-/*                pdbA->cds[crds]->name[i], */
-/*                pdbA->cds[crds]->altLoc[i], */
-/*                pdbA->cds[crds]->chainID[i], */
-/*                pdbA->cds[crds]->resSeq[i]); */
+/*                pdbj->record[i], */
+/*                pdbj->name[i], */
+/*                pdbj->altLoc[i], */
+/*                pdbj->chainID[i], */
+/*                pdbj->resSeq[i]); */
 /*         fflush(NULL); */
-        if (   (   strncmp(pdbA->cds[crds]->record[i], "ATOM  ", 6) == 0
-                || strncmp(pdbA->cds[crds]->record[i], "HETATM", 6) == 0)
-            && atom_selxn(pdbA->cds[crds]->name[i], algo->atoms, algo->atomslxn)
-            && (pdbA->cds[crds]->altLoc[i] == ' ' || pdbA->cds[crds]->altLoc[i] == 'A')
-            && range_selxn(pdbA->cds[crds]->chainID[i], chains, pdbA->cds[crds]->resSeq[i], lower, upper, range_num) == rev) /* DLT debug FIX THIS!!!! */
+        if (RecordSelxn(pdbj->record[i]) &&
+            AtomSelxn2(pdbj->name[i], algo->atoms, algo->atomslxn) &&
+            AltLocSelxn(pdbj->altLoc[i]) &&
+            RangeSelxn2(pdbj->chainID[i], chains, pdbj->resSeq[i], lower, upper, range_num) == rev) /* DLT debug FIX THIS!!!! */
         {
-            if (selection_index != NULL)
+            if (selection_index)
                 selection_index[vlen] = i;
             ++vlen;
         }
@@ -793,7 +814,7 @@ GetCdsSelection(CdsArray *baseA, PDBCdsArray *pdbA)
     char          **selections = NULL; /* an array of range_num strings to hold each range selection */
     char            delims[] = ":";
 
-    if (algo->selection != NULL)
+    if (algo->selection)
     {
         selection_len = strlen(algo->selection);
 
@@ -828,7 +849,8 @@ GetCdsSelection(CdsArray *baseA, PDBCdsArray *pdbA)
         }
 
         /* if the user specified XPLOR/CNS-style atoms, translate them to standard PDB format (DNA primes should be "*", not "'") */
-        for (i = 0; i < strlen(algo->selection); ++i)
+        j = strlen(algo->selection);
+        for (i = 0; i < j; ++i)
             if (algo->selection[i] == '\'')
                 algo->selection[i] = '*';
 
@@ -912,7 +934,7 @@ GetCdsSelection(CdsArray *baseA, PDBCdsArray *pdbA)
 /*         exit(EXIT_FAILURE); */
 /*     } */
 
-    selection_index = (int *) calloc (pdbA->vlen, sizeof(int));
+    selection_index = (int *) calloc(pdbA->vlen, sizeof(int));
     if (selection_index == NULL)
     {
         perror("\n  ERROR");
@@ -982,7 +1004,6 @@ GetCdsSelection(CdsArray *baseA, PDBCdsArray *pdbA)
             baseA->cds[j]->o[i]      = pdbA->cds[j]->occupancy[selection_index[i]];
             baseA->cds[j]->b[i]      = pdbA->cds[j]->tempFactor[selection_index[i]];
             baseA->cds[j]->nu[i]     = 1;
-            baseA->cds[j]->mu[i]     = 0;
         }
     }
 
@@ -990,13 +1011,13 @@ GetCdsSelection(CdsArray *baseA, PDBCdsArray *pdbA)
 /* PrintCds(baseA->cds[1]); */
 
     for (i = 0; i < range_num; ++i)
-        free(selections[i]);
+        MyFree(selections[i]);
 
-    free(selections);
-    free(selection_index);
-    free(upper);
-    free(lower);
-    free(chains);
+    MyFree(selections);
+    MyFree(selection_index);
+    MyFree(upper);
+    MyFree(lower);
+    MyFree(chains);
 }
 
 
@@ -1013,7 +1034,7 @@ PDBCdsArray
     FILE                *pdbfile = NULL;
     PDBCdsArray      *pdbA = NULL;
 
-    if (getenv("PDBDIR") != NULL)
+    if (getenv("PDBDIR"))
         strncpy(pdbdir, getenv("PDBDIR"), FILENAME_MAX);
 
     vlen = last_vlen = 0;
@@ -1054,7 +1075,7 @@ PDBCdsArray
 
         models = 0;
         last_vlen = 0;
-        while (fgets(buff, (int) sizeof(buff), pdbfile) != NULL)
+        while (fgets(buff, (int) sizeof(buff), pdbfile))
         {
             if (strncmp(buff, "MODEL", 5) == 0 ||
                 strncmp(buff, "REMARK FRAME:", 13) == 0 ||
@@ -1065,7 +1086,7 @@ PDBCdsArray
                 {
                     if ((strncmp(buff, "END", 3) == 0) || (strncmp(buff, "ENDMDL", 6) == 0))
                         break;
-                    else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
+                    else if (RecordSelxn(buff))
                         ++vlen;
                 }
 
@@ -1112,11 +1133,11 @@ PDBCdsArray
             vlen = 0;
             rewind(pdbfile);
 
-            while (fgets(buff, sizeof(buff), pdbfile) != NULL)
+            while (fgets(buff, sizeof(buff), pdbfile))
             {
                 if (strncmp(buff, "END", 3) == 0 || (strncmp(buff, "ENDMDL", 6) == 0))
                     break;
-                else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
+                else if (RecordSelxn(buff))
                     ++vlen;
             }
         }
@@ -1157,8 +1178,8 @@ PDBCdsArray
         j += models_per_pdb[i];
     }
 
-    free(models_per_pdb);
-    free(len_array);
+    MyFree(models_per_pdb);
+    MyFree(len_array);
 
     return(pdbA);
 }
@@ -1222,13 +1243,10 @@ PrintPDBCds(FILE *pdbfile, PDBCds *pdbcds)
         /* first expression evaluates whether at end of internal ATOM chain */
         /* second expression evaluates if at end of cds and ATOM chain */
         if (
-            ((i + 1) <  pdbcds->vlen &&
-              ( strncmp(pdbcds->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcds->record[i], "HETATM", 6) == 0 )
-            &&
+            ((i + 1) <  pdbcds->vlen && RecordSelxn(pdbcds->record[i]) &&
              (pdbcds->chainID[i] != pdbcds->chainID[i+1] || pdbcds->xchainID[i] != pdbcds->xchainID[i+1]))
            ||
-            ((i + 1) == pdbcds->vlen &&
-             ( strncmp(pdbcds->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcds->record[i], "HETATM", 6) == 0 ))
+            ((i + 1) == pdbcds->vlen && RecordSelxn(pdbcds->record[i]))
            ||
             (i < pdbcds->vlen - 1 &&
             IsNameCAorP(pdbcds->name[i]) &&
@@ -1330,14 +1348,11 @@ PrintNuPDBCds(FILE *pdbfile, PDBCds *pdbcds)
             /* first expression evaluates whether at end of internal ATOM chain */
             /* second expression evaluates if at end of cds and ATOM chain */
             if (
-                ((i + 1) <  pdbcds->vlen &&
-                 ( strncmp(pdbcds->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcds->record[i], "HETATM", 6) == 0 )
+                ((i + 1) <  pdbcds->vlen && RecordSelxn(pdbcds->record[i])
                 &&
                  (pdbcds->chainID[i] != pdbcds->chainID[i+1] || pdbcds->xchainID[i] != pdbcds->xchainID[i+1]))
                ||
-                ((i + 1) == pdbcds->vlen
-                &&
-                ( strncmp(pdbcds->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcds->record[i], "HETATM", 6) == 0))
+                ((i + 1) == pdbcds->vlen && RecordSelxn(pdbcds->record[i]))
                ||
                 (i < pdbcds->vlen - 1 &&
                 IsNameCAorP(pdbcds->name[i]) &&
@@ -1371,6 +1386,59 @@ IsNameCAorP(char *name)
 }
 
 
+char
+*WriteFileMap(PDBCdsArray *myPDBs, const char *filemap_name)
+{
+    FILE           *filemap = NULL;
+    char            tmpStr[1024];
+    char           *fNameStr;
+    int             ii, tmpLen;
+
+    /* ////////////////////////////////////////////////////////////// */
+    tmpLen = strlen(filemap_name) + 9;
+    fNameStr = malloc(tmpLen * sizeof(char));
+    if (!fNameStr)
+    {
+        perror("\n\n  ERROR");
+        fprintf(stderr, "\n\n  ERROR_071: could not allocate a string of length %d in WriteFileMap. \n", tmpLen);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    sprintf(fNameStr, "%s.filemap", filemap_name);
+    filemap = fopen(fNameStr, "w");
+    if (filemap == NULL)
+    {
+        perror("\n  ERROR");
+        fprintf(stderr,
+                "\n  ERROR99: could not open file '%s.filemap' for writing. \n", filemap_name);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    for (ii = 0; ii < myPDBs->cnum; ++ii)
+    {
+        tmpLen = strlen(myPDBs->cds[ii]->filename);
+        strncpy(tmpStr, myPDBs->cds[ii]->filename, (tmpLen + 1) * sizeof(char));
+        tmpLen--;
+        while (tmpLen)
+        {
+            if (tmpStr[tmpLen] == '.')
+            {
+                tmpStr[tmpLen] = '\0';
+                break;
+            }
+            tmpLen--;
+        }
+        fprintf(filemap, "%s %s\n", myPDBs->cds[ii]->filename, tmpStr);
+    }
+
+    fclose(filemap);
+
+    return fNameStr;
+}
+
+
 void
 WriteModelFile(PDBCdsArray *pdbA, char *outfile_name)
 {
@@ -1639,7 +1707,7 @@ OverWriteTheseusCdsModelFile(CdsArray *cdsA, char *outfile_name)
 
 
 void
-WriteTheseusPDBFiles(PDBCdsArray *pdbA, Algorithm *algo, Statistics *stats)
+WriteTheseusPDBFiles(PDBCdsArray *pdbA, Algorithm *algo, Statistics *stats, char *fname)
 {
     FILE           *pdbfile = NULL;
     int             i;
@@ -1647,7 +1715,7 @@ WriteTheseusPDBFiles(PDBCdsArray *pdbA, Algorithm *algo, Statistics *stats)
 
     for (i = 0; i < pdbA->cnum; ++i)
     {
-        strncpy(outfile_name, algo->rootname, strlen(algo->rootname)+1);
+        strncpy(outfile_name, fname, strlen(fname)+1);
         strcat(outfile_name, "_");
         strncat(outfile_name, pdbA->cds[i]->filename, strlen(pdbA->cds[i]->filename)+1);
         //strcpy(outfile_name, getroot(pdbA->cds[i]->filename));
@@ -2097,7 +2165,7 @@ ConvertLele_freeform(char *fp_name, const int dim, const int forms, const int lm
     fprintf(fp1, "END\n\n");
     fclose(fp0);
     fclose(fp1);
-    free(vals);
+    MyFree(vals);
 
     return(1);
 }
@@ -2305,7 +2373,7 @@ WriteInstModelFile(char *fext, CdsArray *cdsA)
     fext_name = mystrcat(algo->rootname, fext);
     WriteTheseusModelFileNoStats(mpA, algo, fext_name);
 
-    free(fext_name);
+    MyFree(fext_name);
     PDBCdsArrayDestroy(&mpA);
 }
 
@@ -2352,7 +2420,7 @@ WriteOlveFiles(CdsArray *cdsA)
 
     olve_name = mystrcat(algo->rootname, "_olve.pdb");
     WriteOlveModelFile(olveA, algo, stats, olve_name);
-    free(olve_name);
+    MyFree(olve_name);
     PDBCdsArrayDestroy(&olveA);
 }
 
@@ -2380,7 +2448,7 @@ WriteDistMatTree(CdsArray *cdsA)
     {
         strcpy(distmat->taxa[i], cds[i]->filename);
         ptr = strrchr(distmat->taxa[i], '.');
-        if (ptr != NULL)
+        if (ptr)
             *ptr = '\0';
         sprintf(num, "_%d", i);
         strcat(distmat->taxa[i], num);
@@ -2423,7 +2491,7 @@ WriteDistMatTree(CdsArray *cdsA)
 
     tree_name = mystrcat(algo->rootname, "_ML_tree.nxs");
     print_NX_distmat(distmat, tree_name);
-    free(tree_name);
+    MyFree(tree_name);
 
 //         //double total = 0.0;
 //         for (i = 0; i < cnum; ++i)
@@ -2465,7 +2533,7 @@ WriteDistMatTree(CdsArray *cdsA)
     tree_name = mystrcat(algo->rootname, "_LS_tree.nxs");
     print_NX_distmat(distmat, tree_name);
 
-    free(tree_name);
+    MyFree(tree_name);
     DISTMATdestroy(&distmat);
 }
 
