@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -44,11 +43,12 @@ Seq2PDB
 {
     Seq2PDB        *seq2pdb = NULL;
 
-    seq2pdb = malloc(sizeof(Seq2PDB));
+    seq2pdb = calloc(1, sizeof(Seq2PDB));
 
     seq2pdb->pdbfile_name = NULL;
     seq2pdb->seqname = NULL;
     seq2pdb->map = NULL;
+    seq2pdb->msa = NULL;
     seq2pdb->singletons = NULL;
     seq2pdb->pdbfile_name_space = NULL;
     seq2pdb->seqname_space = NULL;
@@ -64,8 +64,8 @@ Seq2pdbAlloc(Seq2PDB *seq2pdb, const int seqnum)
 
     seq2pdb->seqnum = seqnum;
 
-    seq2pdb->pdbfile_name = (char **) malloc(seqnum * sizeof(char *));
-    seq2pdb->seqname = (char **) malloc(seqnum * sizeof(char *));
+    seq2pdb->pdbfile_name = (char **) calloc(seqnum, sizeof(char *));
+    seq2pdb->seqname = (char **) calloc(seqnum, sizeof(char *));
     seq2pdb->map = (int *) calloc(seqnum, sizeof(int));
 
     /* allocate space for the fields in total */
@@ -93,58 +93,85 @@ Seq2pdbAlloc(Seq2PDB *seq2pdb, const int seqnum)
 void
 Seq2pdbDestroy(Seq2PDB **seq2pdb_ptr)
 {
-    Seq2PDB *seq2pdb = *seq2pdb_ptr;
+    if (seq2pdb_ptr)
+    {
+        Seq2PDB *seq2pdb = *seq2pdb_ptr;
 
-    free(seq2pdb->pdbfile_name_space);
-    free(seq2pdb->seqname_space);
+        if (seq2pdb)
+        {
+            MyFree(seq2pdb->pdbfile_name_space);
+            MyFree(seq2pdb->seqname_space);
 
-    free(seq2pdb->pdbfile_name);
-    free(seq2pdb->seqname);
-    free(seq2pdb->map);
-    if (seq2pdb->singletons != NULL)
-        free(seq2pdb->singletons);
+            MyFree(seq2pdb->pdbfile_name);
+            MyFree(seq2pdb->seqname);
+            MyFree(seq2pdb->map);
+            MyFree(seq2pdb->singletons);
 
-    MSAfree(&(seq2pdb->msa));
+            MSAdestroy(&(seq2pdb->msa));
 
-    free(seq2pdb);
-    *seq2pdb_ptr = NULL;
+            MyFree(seq2pdb);
+            seq2pdb = NULL;
+        }
+    }
+}
+
+
+/* frees all but msa and seq2pdb */
+void
+Seq2pdbDealloc(Seq2PDB **seq2pdb_ptr)
+{
+    if (seq2pdb_ptr)
+    {
+        Seq2PDB *seq2pdb = *seq2pdb_ptr;
+
+        if (seq2pdb)
+        {
+            MyFree(seq2pdb->pdbfile_name_space);
+            MyFree(seq2pdb->seqname_space);
+
+            MyFree(seq2pdb->pdbfile_name);
+            MyFree(seq2pdb->seqname);
+            MyFree(seq2pdb->map);
+            MyFree(seq2pdb->singletons);
+        }
+    }
 }
 
 
 void
-AlgorithmDestroy(Algorithm *algo)
+AlgorithmDestroy(Algorithm **algo_ptr)
 {
-    int         i;
-    /* printf("\n AlgorithmDestroy(%p)", algo);fflush(NULL); */
-    if (algo->argv != NULL)
+    if (algo_ptr)
     {
-        for (i = 0; i < algo->argc; ++i)
-            if (algo->argv[i] != NULL)
-                free(algo->argv[i]);
+        Algorithm *algo = *algo_ptr;
 
-        free(algo->argv);
+        if (algo)
+        {
+            int         i;
+
+            if (algo->argv)
+            {
+                for (i = 0; i < algo->argc; ++i)
+                    MyFree(algo->argv[i]);
+
+                MyFree(algo->argv);
+            }
+
+            MyFree(algo->selection);
+            algo->selection = NULL;
+            MyFree(algo->atomslxn);
+            algo->atomslxn = NULL;
+            MyFree(algo);
+            algo = NULL;
+        }
     }
-
-    if (algo->selection != NULL)
-    {
-        free(algo->selection);
-        algo->selection = NULL;
-    }
-
-    if (algo->atomslxn != NULL)
-    {
-        free(algo->atomslxn);
-        algo->atomslxn = NULL;
-    }
-
-    free(algo);
 }
 
 
 Algorithm
 *AlgorithmInit(void)
 {
-    Algorithm      *algo = (Algorithm *) malloc (sizeof(Algorithm));
+    Algorithm      *algo = (Algorithm *) calloc (1, sizeof(Algorithm));
     if (algo == NULL)
     {
         perror("\n  ERROR");
@@ -156,11 +183,10 @@ Algorithm
     /* printf("\n AlgorithmInit(%p)", algo);fflush(NULL); */
     algo->argv         = NULL;
     mystrcpy(algo->rootname, "theseus");
-    algo->weight       = 0;
     algo->verbose      = 0; /* verbose behavior = 1 */
     algo->write_file   = 1;
     algo->precision    = 1e-7;
-    algo->iterations   = 400;
+    algo->iterations   = 10000;
     algo->selection    = NULL;
     algo->atomslxn     = NULL;
     algo->revsel       = 0;
@@ -175,7 +201,6 @@ Algorithm
     algo->tenberge     = 0;
     algo->morph        = 0;
     algo->stats        = 0;
-    algo->constant     = -1e-40; /* If negative, finds min var empirically */
     algo->info         = 0;
     algo->princaxes    = 1;
     algo->nullrun      = 0;
@@ -191,11 +216,11 @@ Algorithm
     algo->fmodel       = 0;
     algo->covweight    = 0;
     algo->varweight    = 1;
-    algo->hierarch     = 5;
+    algo->hierarch     = 6;
     algo->leastsquares = 0;
     algo->filenum      = 0;
     algo->infiles      = NULL;
-    algo->noinnerloop  = 0;
+    algo->noinnerloop  = 1;
     algo->rounds       = 0;
     algo->innerrounds  = 0;
     algo->fasta        = 0;
@@ -224,6 +249,10 @@ Algorithm
     algo->scaleanchor  = 0;
     algo->randgibbs    = 0;
     algo->covnu        = 1;
+    algo->crush        = 0;
+    algo->mg_crush     = 0;
+    algo->parallel     = 0;
+    algo->ndf          = 0; // default set in MultiPose, 3 for -v, vlen for -c
 
     return(algo);
 }
@@ -235,7 +264,7 @@ Statistics
     Statistics     *stats = NULL;
     int             i;
 
-    stats = (Statistics *) malloc(sizeof(Statistics));
+    stats = (Statistics *) calloc(1, sizeof(Statistics));
     if (stats == NULL)
     {
         perror("\n  ERROR");
@@ -260,7 +289,7 @@ CdsArray
 {
     CdsArray *cdsA = NULL;
 
-    cdsA = (CdsArray *) malloc(sizeof(CdsArray));
+    cdsA = (CdsArray *) calloc(1, sizeof(CdsArray));
 /* printf("\ncdsA malloc(%p)", (void *) cdsA); */
 /* fflush(NULL); */
     if (cdsA == NULL)
@@ -287,8 +316,9 @@ CdsArray
 
     cdsA->w = NULL;
     cdsA->var = NULL;
+    cdsA->invvar = NULL;
     cdsA->evals = NULL;
-    cdsA->samplevar3N = NULL;
+    cdsA->samplevar = NULL;
     cdsA->df = NULL;
     cdsA->S2 = NULL;
 
@@ -313,7 +343,7 @@ CdsArray
     cdsA->tmpmat3c = MatAlloc(3, 3);
     cdsA->tmpmat3d = MatAlloc(3, 3);
 
-    cdsA->tmpvec3a = malloc(3 * sizeof(double));
+    cdsA->tmpvec3a = calloc(3, sizeof(double));
 
     return(cdsA);
 }
@@ -324,7 +354,7 @@ CdsArrayAllocNum(CdsArray *cdsA, const int cnum)
 {
     cdsA->cnum = cnum;
 
-    cdsA->cds = (Cds **) malloc(cnum * sizeof(Cds *));
+    cdsA->cds = (Cds **) calloc(cnum, sizeof(Cds *));
     if (cdsA->cds == NULL)
     {
         perror("\n  ERROR");
@@ -351,14 +381,16 @@ CdsArrayAllocLen(CdsArray *cdsA, const int vlen)
 
     cdsA->avecds = CdsInit();
     CdsAlloc(cdsA->avecds, vlen);
-    cdsA->ac = MatAlloc(3, vlen);
     cdsA->tcds = CdsInit();
-    cdsA->tc = MatAlloc(3, vlen);
     CdsAlloc(cdsA->tcds, vlen);
 
+    cdsA->ac = MatAlloc(3, vlen);
+    cdsA->tc = MatAlloc(3, vlen);
+
     cdsA->var = (double *) calloc(vlen, sizeof(double));
+    cdsA->invvar = (double *) calloc(vlen, sizeof(double));
     cdsA->evals = (double *) calloc(vlen, sizeof(double));
-    cdsA->samplevar3N = (double *) calloc(vlen, sizeof(double));
+    cdsA->samplevar = (double *) calloc(vlen, sizeof(double));
     cdsA->w   = (double *) calloc(vlen, sizeof(double));
     cdsA->df  = (int *) calloc(vlen, sizeof(int));
 }
@@ -377,7 +409,7 @@ CdsArraySetup(CdsArray *cdsA)
 {
     memsetd(cdsA->var, 1.0, cdsA->vlen);
     memsetd(cdsA->evals, 1.0, cdsA->vlen);
-    memsetd(cdsA->samplevar3N, 1.0, cdsA->vlen);
+    memsetd(cdsA->samplevar, 1.0, cdsA->vlen);
     memsetd(cdsA->w, 1.0, cdsA->vlen);
 }
 
@@ -387,58 +419,58 @@ CdsArrayDestroy(CdsArray **cdsA_ptr)
 {
     int             i;
 
-    CdsArray *cdsA = *cdsA_ptr;
-
-    free(cdsA->msafile_name);
-    cdsA->msafile_name = NULL;
-    free(cdsA->mapfile_name);
-    cdsA->mapfile_name = NULL;
-
-    for (i = 0; i < cdsA->cnum; ++i)
+    if (cdsA_ptr)
     {
-        CdsDestroy(&(cdsA->cds[i]));
+        CdsArray *cdsA = *cdsA_ptr;
+
+        if (cdsA)
+        {
+            MyFree(cdsA->msafile_name);
+            cdsA->msafile_name = NULL;
+            MyFree(cdsA->mapfile_name);
+            cdsA->mapfile_name = NULL;
+
+            for (i = 0; i < cdsA->cnum; ++i)
+            {
+                CdsDestroy(&(cdsA->cds[i]));
+            }
+
+            MyFree(cdsA->cds);
+            cdsA->cds = NULL;
+
+            CdsDestroy(&(cdsA->avecds));
+            CdsDestroy(&(cdsA->tcds));
+
+            MatDestroy(&(cdsA->ac));
+            MatDestroy(&(cdsA->tc));
+
+            DistMatsDestroy(cdsA);
+            CovMatsDestroy(cdsA);
+            PCADestroy(cdsA);
+
+            MyFree(cdsA->tmpvec3a);
+
+            MyFree(cdsA->residuals);
+            cdsA->residuals = NULL;
+            MyFree(cdsA->w);
+            cdsA->w = NULL;
+            MyFree(cdsA->var);
+            cdsA->var = NULL;
+            MyFree(cdsA->invvar);
+            cdsA->var = NULL;
+            MyFree(cdsA->evals);
+            cdsA->evals = NULL;
+            MyFree(cdsA->samplevar);
+            cdsA->samplevar = NULL;
+            MyFree(cdsA->df);
+            cdsA->df = NULL;
+            MyFree(cdsA->S2);
+            cdsA->S2 = NULL;
+
+            MyFree(cdsA);
+            cdsA = NULL;
+        }
     }
-
-    free(cdsA->cds);
-    cdsA->cds = NULL;
-
-    CdsDestroy(&(cdsA->avecds));
-    CdsDestroy(&(cdsA->tcds));
-
-    MatDestroy(&(cdsA->ac));
-    MatDestroy(&(cdsA->tc));
-
-    DistMatsDestroy(cdsA);
-    CovMatsDestroy(cdsA);
-    PCADestroy(cdsA);
-
-    free(cdsA->tmpvec3a);
-
-    if (cdsA->residuals != NULL)
-    {
-        free(cdsA->residuals);
-        cdsA->residuals = NULL;
-    }
-
-    free(cdsA->w);
-    cdsA->w = NULL;
-    free(cdsA->var);
-    cdsA->var = NULL;
-    free(cdsA->evals);
-    cdsA->evals = NULL;
-    free(cdsA->samplevar3N);
-    cdsA->samplevar3N = NULL;
-    free(cdsA->df);
-    cdsA->df = NULL;
-
-    if (cdsA->S2 != NULL)
-    {
-        free(cdsA->S2);
-        cdsA->S2 = NULL;
-    }
-
-    free(cdsA);
-    *cdsA_ptr = NULL;
 }
 
 
@@ -454,11 +486,11 @@ DistMatsAlloc(CdsArray *cdsA)
 void
 DistMatsDestroy(CdsArray *cdsA)
 {
-/*     if (cdsA->Var_matrix != NULL) */
+/*     if (cdsA->Var_matrix) */
 /*         MatDestroy(&(cdsA->Var_matrix)); */
-    if (cdsA->Dij_matrix != NULL)
+    if (cdsA->Dij_matrix)
         MatDestroy(&(cdsA->Dij_matrix));
-/*     if (cdsA->distmat != NULL) */
+/*     if (cdsA->distmat) */
 /*         Mat3DDestroy(&(cdsA->distmat)); */
 }
 
@@ -466,26 +498,16 @@ DistMatsDestroy(CdsArray *cdsA)
 void
 CovMatsDestroy(CdsArray *cdsA)
 {
-    if (cdsA->CovMat != NULL)
-        MatDestroy(&(cdsA->CovMat));
-    if (cdsA->FullCovMat != NULL)
-        MatDestroy(&(cdsA->FullCovMat));
-    if (cdsA->WtMat != NULL)
-        MatDestroy(&(cdsA->WtMat));
-    if (cdsA->tmpmatKK1 != NULL)
-        MatDestroy(&(cdsA->tmpmatKK1));
-    if (cdsA->tmpmatKK2 != NULL)
-        MatDestroy(&(cdsA->tmpmatKK2));
-    if (cdsA->tmpvecK != NULL)
-        free(cdsA->tmpvecK);
-    if (cdsA->tmpmat3a != NULL)
-        MatDestroy(&(cdsA->tmpmat3a));
-    if (cdsA->tmpmat3b != NULL)
-        MatDestroy(&(cdsA->tmpmat3b));
-    if (cdsA->tmpmat3c != NULL)
-        MatDestroy(&(cdsA->tmpmat3c));
-    if (cdsA->tmpmat3d != NULL)
-        MatDestroy(&(cdsA->tmpmat3d));
+    MatDestroy(&(cdsA->CovMat));
+    MatDestroy(&(cdsA->FullCovMat));
+    MatDestroy(&(cdsA->WtMat));
+    MatDestroy(&(cdsA->tmpmatKK1));
+    MatDestroy(&(cdsA->tmpmatKK2));
+    MyFree(cdsA->tmpvecK);
+    MatDestroy(&(cdsA->tmpmat3a));
+    MatDestroy(&(cdsA->tmpmat3b));
+    MatDestroy(&(cdsA->tmpmat3c));
+    MatDestroy(&(cdsA->tmpmat3d));
 }
 
 
@@ -500,13 +522,8 @@ PCAAlloc(CdsArray *cdsA)
 void
 PCADestroy(CdsArray *cdsA)
 {
-    if (cdsA->pcamat != NULL)
-        MatDestroy(&(cdsA->pcamat));
-    if (cdsA->pcavals != NULL)
-    {
-        free(cdsA->pcavals);
-        cdsA->pcavals = NULL;
-    }
+    MatDestroy(&(cdsA->pcamat));
+    MyFree(cdsA->pcavals);
 }
 
 
@@ -516,7 +533,7 @@ Cds
     int             i;
     Cds         *cds = NULL;
 
-    cds = (Cds *) malloc(sizeof(Cds));
+    cds = (Cds *) calloc(1, sizeof(Cds));
 
     strncpy(cds->filename, "", 1);
 
@@ -531,7 +548,6 @@ Cds
     cds->o          = NULL;
     cds->b          = NULL;
     cds->nu         = NULL;
-    cds->mu         = NULL;
     cds->sc         = NULL;
     cds->sx         = NULL;
     cds->sy         = NULL;
@@ -542,10 +558,7 @@ Cds
     cds->covx       = NULL;
     cds->covy       = NULL;
     cds->covz       = NULL;
-
-    cds->residual_x = NULL;
-    cds->residual_y = NULL;
-    cds->residual_z = NULL;
+    cds->prvar      = NULL;
 
     cds->outerprod = NULL;
 
@@ -560,7 +573,10 @@ Cds
         cds->evecs[i][0] = 1.0;
 
     for (i = 0; i < 3; ++i)
+    {
         cds->matrix[i][i] = cds->last_matrix[i][i] = cds->last_outer_matrix[i][i] = 1.0;
+        cds->center[i] = cds->last_center[i] = 0.0;
+    }
 
     cds->bfact_c = 1.0;
     cds->scale = 1.0;
@@ -576,10 +592,10 @@ CdsAlloc(Cds *cds, const int vlen)
 
     cds->vlen = vlen;
 
-    cds->resName       = (char **)  calloc(vlen, sizeof(char *));
-    cds->resName_space = (char *)   calloc(4 * vlen, sizeof(char));
-    cds->chainID       = (char *)   calloc(vlen, sizeof(char));
-    cds->resSeq        = (int *)    calloc(vlen, sizeof(int));
+    cds->resName       = (char **) calloc(vlen, sizeof(char *));
+    cds->resName_space = (char *)  calloc(4 * vlen, sizeof(char));
+    cds->chainID       = (char *)  calloc(vlen, sizeof(char));
+    cds->resSeq        = (int *)   calloc(vlen, sizeof(int));
     cds->wc            = MatAlloc(5, vlen);
     cds->x             = cds->wc[0];
     cds->y             = cds->wc[1];
@@ -587,7 +603,6 @@ CdsAlloc(Cds *cds, const int vlen)
     cds->o             = cds->wc[3];
     cds->b             = cds->wc[4];
     cds->nu            = (int *) calloc(vlen, sizeof(int));
-    cds->mu            = (int *) calloc(vlen, sizeof(int));
     cds->sc            = MatAlloc(5, vlen);
     cds->sx            = cds->sc[0];
     cds->sy            = cds->sc[1];
@@ -599,9 +614,6 @@ CdsAlloc(Cds *cds, const int vlen)
     cds->covy          = cds->cc[1];
     cds->covz          = cds->cc[2];
     cds->prvar         = (double *) calloc(vlen, sizeof(double));
-    cds->residual_x    = (double *) calloc(vlen, sizeof(double));
-    cds->residual_y    = (double *) calloc(vlen, sizeof(double));
-    cds->residual_z    = (double *) calloc(vlen, sizeof(double));
 
     if (   cds->resName       == NULL
         || cds->resName_space == NULL
@@ -610,10 +622,7 @@ CdsAlloc(Cds *cds, const int vlen)
         || cds->wc            == NULL
         || cds->sc            == NULL
         || cds->cc            == NULL
-        || cds->prvar         == NULL
-        || cds->residual_x    == NULL
-        || cds->residual_y    == NULL
-        || cds->residual_z    == NULL)
+        || cds->prvar         == NULL)
     {
         perror("\n  ERROR");
         fprintf(stderr, "\n  ERROR5: could not allocate memory in function CdsAlloc(). \n");
@@ -622,7 +631,10 @@ CdsAlloc(Cds *cds, const int vlen)
     }
 
     for (i = 0; i < vlen; ++i)
+    {
         cds->resName[i] = cds->resName_space + (i * 4); /* 3 */
+        cds->resName[i][3] = '\0';
+    }
 
     CdsSetup(cds);
 }
@@ -646,33 +658,32 @@ CdsSetup(Cds *cds)
 void
 CdsDestroy(Cds **cds_ptr)
 {
-    Cds *cds = *cds_ptr;
+    if (cds_ptr)
+    {
+        Cds *cds = *cds_ptr;
 
-    free(cds->resSeq);
-    free(cds->resName_space);
-    free(cds->chainID);
-    free(cds->resName);
-    MatDestroy(&(cds->wc));
-    MatDestroy(&(cds->sc));
-    MatDestroy(&(cds->cc));
-    free(cds->nu);
-    free(cds->mu);
-    free(cds->prvar);
-    free(cds->residual_x);
-    free(cds->residual_y);
-    free(cds->residual_z);
-    MatDestroy(&(cds->matrix));
-    MatDestroy(&(cds->last_matrix));
-    MatDestroy(&(cds->last_outer_matrix));
-    MatDestroy(&(cds->innerprod));
+        if (cds)
+        {
+            MyFree(cds->resSeq);
+            MyFree(cds->resName_space);
+            MyFree(cds->chainID);
+            MyFree(cds->resName);
+            MatDestroy(&(cds->wc));
+            MatDestroy(&(cds->sc));
+            MatDestroy(&(cds->cc));
+            MyFree(cds->nu);
+            MyFree(cds->prvar);
+            MatDestroy(&(cds->matrix));
+            MatDestroy(&(cds->last_matrix));
+            MatDestroy(&(cds->last_outer_matrix));
+            MatDestroy(&(cds->innerprod));
+            MatDestroy(&(cds->outerprod));
+            MatDestroy(&(cds->evecs));
 
-    if (cds->outerprod != NULL)
-        MatDestroy(&(cds->outerprod));
-
-    MatDestroy(&(cds->evecs));
-
-    free(cds);
-    *cds_ptr = NULL;
+            MyFree(cds);
+            cds = NULL;
+        }
+    }
 }
 
 
@@ -681,7 +692,7 @@ PDBCdsArray
 {
     PDBCdsArray *pdbA = NULL;
 
-    pdbA = (PDBCdsArray *) malloc(sizeof(PDBCdsArray));
+    pdbA = (PDBCdsArray *) calloc(1, sizeof(PDBCdsArray));
     if (pdbA == NULL)
     {
         perror("\n  ERROR");
@@ -716,7 +727,7 @@ PDBCdsArrayAllocNum(PDBCdsArray *pdbA, const int cnum)
 {
     int             i;
 
-    pdbA->cds = (PDBCds **) malloc(cnum * sizeof(PDBCds *));
+    pdbA->cds = (PDBCds **) calloc(cnum, sizeof(PDBCds *));
     pdbA->cnum = cnum;
 
     for (i = 0; i < cnum; ++i)
@@ -756,41 +767,43 @@ PDBCdsArrayAllocLen(PDBCdsArray *pdbA, const int vlen)
 void
 PDBCdsArrayDestroy(PDBCdsArray **pdbA_ptr)
 {
-    PDBCdsArray *pdbA = *pdbA_ptr;
-    int             i;
+    if (pdbA_ptr)
+    {
+        PDBCdsArray *pdbA = *pdbA_ptr;
 
-    for (i = 0; i < pdbA->cnum; ++i)
-        PDBCdsDestroy(&(pdbA->cds[i]));
+        if (pdbA)
+        {
+            int             i;
 
-    PDBCdsDestroy(&(pdbA->avecds));
+            for (i = 0; i < pdbA->cnum; ++i)
+                PDBCdsDestroy(&(pdbA->cds[i]));
 
-    if (pdbA->upper != NULL)
-        free(pdbA->upper);
+            PDBCdsDestroy(&(pdbA->avecds));
 
-    if (pdbA->lower != NULL)
-        free(pdbA->lower);
+            MyFree(pdbA->upper);
+            MyFree(pdbA->lower);
 
-    if (pdbA->seq2pdb != NULL)
-        Seq2pdbDestroy(&(pdbA->seq2pdb));
+            //Seq2pdbDestroy(&(pdbA->seq2pdb)); DLT FIX
 
-    free(pdbA->cds);
-    free(pdbA);
+            MyFree(pdbA->cds);
+            MyFree(pdbA);
 
-    *pdbA_ptr = NULL;
+            pdbA = NULL;
+        }
+    }
 }
 
 
 PDBCds
 *PDBCdsInit(void)
 {
-    PDBCds      *pdbcds = NULL;
+    PDBCds     *pdbcds = NULL;
 
-    pdbcds = (PDBCds *) malloc(sizeof(PDBCds));
+    pdbcds = (PDBCds *) calloc(1, sizeof(PDBCds));
     pdbcds->vlen = 0; /* so that we know later if this has been allocated or not */
     pdbcds->scale = 1.0;
     pdbcds->translation = calloc(3, sizeof(double));
     pdbcds->matrix = MatAlloc(3, 3);
-    memset(&pdbcds->matrix[0][0], 0, 9 * sizeof(double));
 
     return(pdbcds);
 }
@@ -814,11 +827,15 @@ PDBCdsAlloc(PDBCds *pdbcds, const int vlen)
     pdbcds->chainID    = (char *)   calloc(vlen, sizeof(char));
     pdbcds->resSeq     = (int *)    calloc(vlen, sizeof(int));
     pdbcds->iCode      = (char *)   calloc(vlen, sizeof(char));
-    pdbcds->x          = (double *) calloc(vlen, sizeof(double));
-    pdbcds->y          = (double *) calloc(vlen, sizeof(double));
-    pdbcds->z          = (double *) calloc(vlen, sizeof(double));
-    pdbcds->occupancy  = (double *) calloc(vlen, sizeof(double));
-    pdbcds->tempFactor = (double *) calloc(vlen, sizeof(double));
+
+    pdbcds->c          = MatAlloc(5, vlen);
+
+    pdbcds->x          = pdbcds->c[0];
+    pdbcds->y          = pdbcds->c[1];
+    pdbcds->z          = pdbcds->c[2];
+    pdbcds->occupancy  = pdbcds->c[3];
+    pdbcds->tempFactor = pdbcds->c[4];
+
     pdbcds->segID      = (char **)  calloc(vlen, sizeof(char *));
     pdbcds->element    = (char **)  calloc(vlen, sizeof(char *));
     pdbcds->charge     = (char **)  calloc(vlen, sizeof(char *));
@@ -843,11 +860,7 @@ PDBCdsAlloc(PDBCds *pdbcds, const int vlen)
         || pdbcds->chainID       == NULL
         || pdbcds->resSeq        == NULL
         || pdbcds->iCode         == NULL
-        || pdbcds->x             == NULL
-        || pdbcds->y             == NULL
-        || pdbcds->z             == NULL
-        || pdbcds->occupancy     == NULL
-        || pdbcds->tempFactor    == NULL
+        || pdbcds->c             == NULL
         || pdbcds->segID         == NULL
         || pdbcds->element       == NULL
         || pdbcds->charge        == NULL
@@ -874,6 +887,13 @@ PDBCdsAlloc(PDBCds *pdbcds, const int vlen)
         pdbcds->segID[i]   = pdbcds->segID_space   + (i * 8); /* 4 */
         pdbcds->element[i] = pdbcds->element_space + (i * 4); /* 2 */
         pdbcds->charge[i]  = pdbcds->charge_space  + (i * 4); /* 2 */
+
+        pdbcds->record[i][6]  = '\0'; /* 6 */
+        pdbcds->name[i][3]    = '\0'; /* 3 */
+        pdbcds->resName[i][3] = '\0'; /* 3 */
+        pdbcds->segID[i][4]   = '\0'; /* 4 */
+        pdbcds->element[i][2] = '\0'; /* 2 */
+        pdbcds->charge[i][2]  = '\0'; /* 2 */
     }
 }
 
@@ -881,39 +901,42 @@ PDBCdsAlloc(PDBCds *pdbcds, const int vlen)
 void
 PDBCdsDestroy(PDBCds **pdbcds_ptr)
 {
-    PDBCds *pdbcds = *pdbcds_ptr;
+    if (pdbcds_ptr)
+    {
+        PDBCds *pdbcds = *pdbcds_ptr;
 
-    free(pdbcds->record_space);
-    free(pdbcds->name_space);
-    free(pdbcds->resName_space);
-    free(pdbcds->segID_space);
-    free(pdbcds->element_space);
-    free(pdbcds->charge_space);
+        if (pdbcds)
+        {
+            MyFree(pdbcds->record_space);
+            MyFree(pdbcds->name_space);
+            MyFree(pdbcds->resName_space);
+            MyFree(pdbcds->segID_space);
+            MyFree(pdbcds->element_space);
+            MyFree(pdbcds->charge_space);
 
-    MatDestroy(&(pdbcds->matrix));
-    free(pdbcds->translation);
+            MatDestroy(&(pdbcds->matrix));
+            MyFree(pdbcds->translation);
 
-    free(pdbcds->record);
-    free(pdbcds->serial);
-    free(pdbcds->Hnum);
-    free(pdbcds->name);
-    free(pdbcds->altLoc);
-    free(pdbcds->resName);
-    free(pdbcds->xchainID);
-    free(pdbcds->chainID);
-    free(pdbcds->resSeq);
-    free(pdbcds->iCode);
-    free(pdbcds->x);
-    free(pdbcds->y);
-    free(pdbcds->z);
-    free(pdbcds->occupancy);
-    free(pdbcds->tempFactor);
-    free(pdbcds->segID);
-    free(pdbcds->element);
-    free(pdbcds->charge);
+            MyFree(pdbcds->record);
+            MyFree(pdbcds->serial);
+            MyFree(pdbcds->Hnum);
+            MyFree(pdbcds->name);
+            MyFree(pdbcds->altLoc);
+            MyFree(pdbcds->resName);
+            MyFree(pdbcds->xchainID);
+            MyFree(pdbcds->chainID);
+            MyFree(pdbcds->resSeq);
+            MyFree(pdbcds->iCode);
+            MatDestroy(&(pdbcds->c));
+            MyFree(pdbcds->segID);
+            MyFree(pdbcds->element);
+            MyFree(pdbcds->charge);
 
-    free(pdbcds->nu);
+            MyFree(pdbcds->nu);
 
-    free(pdbcds);
-    *pdbcds_ptr = NULL;
+            MyFree(pdbcds);
+            pdbcds = NULL;
+        }
+    }
 }
+
