@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -163,14 +162,14 @@ SSMDestroy(SSM **ssm_ptr)
     for (i = 0; i < ssm->n * (ssm->n - 1) / 2; ++i)
         MatDestroy(&(ssm->mat[i]));
 
-    free(ssm->mat);
+    MyFree(ssm->mat);
     ssm->mat = NULL;
 
-    free(ssm->L);
+    MyFree(ssm->L);
     ssm->L = NULL;
 
-    free(ssm);
-    *ssm_ptr = NULL;
+    MyFree(ssm);
+    ssm = NULL;
 }
 
 
@@ -210,11 +209,12 @@ SSMCalc(SSM *ssm, CdsArray *cdsA)
 /*                           fflush(NULL); */
 
                             ssm->mat[k][p][q] =
-                            (invvar[m] * SqrCdsDist(cdsA->cds[i], m, cdsA->avecds, m) +
-                             invvar[n] * SqrCdsDist(cdsA->cds[i], m, cdsA->avecds, n) +
-                             invvar[m] * SqrCdsDist(cdsA->cds[j], n, cdsA->avecds, m) +
-                             invvar[n] * SqrCdsDist(cdsA->cds[j], n, cdsA->avecds, n) +
-                             2.0 * (lnvar[i] + lnvar[j])) * -0.25;
+                            log(SqrCdsDist(cdsA->cds[i], m, cdsA->cds[j], n));
+//                             (  invvar[m] * SqrCdsDist(cdsA->cds[i], m, cdsA->avecds, m)
+//                              + invvar[n] * SqrCdsDist(cdsA->cds[i], m, cdsA->avecds, n)
+//                              + invvar[m] * SqrCdsDist(cdsA->cds[j], n, cdsA->avecds, m)
+//                              + invvar[n] * SqrCdsDist(cdsA->cds[j], n, cdsA->avecds, n)
+//                              + 0.5 * (lnvar[i] + lnvar[j]));
 
 //                           printf("[%d][%d][%d]:%e\n", k, p, q, ssm->mat[k][p][q]);
 //                           fflush(NULL);
@@ -232,8 +232,8 @@ SSMCalc(SSM *ssm, CdsArray *cdsA)
         }
     }
 
-    free(lnvar);
-    free(invvar);
+    MyFree(lnvar);
+    MyFree(invvar);
 }
 
 
