@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -28,21 +27,12 @@
 static void
 Vars2Bfacts(CdsArray *cdsA);
 
-static void
-WriteVariance(CdsArray *cdsA, char *outfile_name);
-
-static void
-Vars2Bfacts(CdsArray *cdsA);
-
 static double
 SqrCoordMag(const Cds *cds, const int vec);
 
 static double
 RadiusGyration(Cds *cds, const double *weights);
 
-static void
-WriteResiduals(CdsArray *cdsA, char *outfile_name);
-
 static void
 MomentsCds(CdsArray *cdsA);
 
@@ -52,21 +42,6 @@ CalcAIC(CdsArray *cdsA);
 static void
 CalcBIC(CdsArray *cdsA);
 
-// static double
-// FrobTermDiag(CdsArray *cdsA);
-//
-// static double
-// FrobTerm2(CdsArray *cdsA);
-//
-// static double
-// CalcLogScaleJacob(CdsArray *cdsA);
-
-// static double **
-// CalcCov(CdsArray *cdsA);
-
-// static double *
-// CalcVar(CdsArray *cdsA);
-
 
 void
 CheckVars(CdsArray *cdsA)
@@ -94,133 +69,10 @@ CalcDf(CdsArray *cdsA)
         cdsA->df[i] = 0;
         for (j = 0; j < cdsA->cnum; ++j)
             cdsA->df[i] += cdsA->cds[j]->nu[i];
-
-        //cdsA->df[i] *= 3;
     }
 }
 
 
-double
-VarianceCds(CdsArray *cdsA)
-{
-    int             i, j;
-    double          tmpx, tmpy, tmpz;
-    double          variance;
-    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
-    const double    idf = 1.0 / (3.0 * cnum);
-    double         *var = cdsA->var;
-    const Cds     **cds = (const Cds **) cdsA->cds;
-    Cds            *cdsj = NULL;
-    const double   *avex = (const double *) cdsA->avecds->x,
-                   *avey = (const double *) cdsA->avecds->y,
-                   *avez = (const double *) cdsA->avecds->z;
-
-    memset(var, 0, vlen * sizeof(double));
-
-    for (j = 0; j < cnum; ++j)
-    {
-        cdsj = (Cds *) cds[j];
-
-        for (i = 0; i < vlen; ++i)
-        {
-            tmpx = cdsj->x[i] - avex[i];
-            tmpy = cdsj->y[i] - avey[i];
-            tmpz = cdsj->z[i] - avez[i];
-            var[i] += tmpx * tmpx + tmpy * tmpy + tmpz * tmpz;
-        }
-    }
-
-//     if (algo->scale > 0)
-//     {
-//         double lndet = 0.0;
-//         for (i = 0; i < vlen; ++i)
-//             lndet += log(var[i]);
-//
-//         lndet /= vlen;
-//         double det = exp(lndet);
-//
-//         for (i = 0; i < vlen; ++i)
-//             var[i] /= det;
-//
-//         variance = det;
-//         stats->var = variance;
-//         stats->stddev = sqrt(variance);
-//     }
-//     else
-    {
-        for (i = 0; i < vlen; ++i)
-            var[i] *= idf;
-
-        variance = 0.0;
-        for (i = 0; i < vlen; ++i)
-            variance += var[i];
-
-        variance /= vlen;
-        stats->var = variance;
-        stats->stddev = sqrt(variance);
-    }
-
-    return(stats->stddev);
-}
-
-
-/* Same as VarianceCds() but weights by occupancies */
-double
-VarianceCdsNu(CdsArray *cdsA)
-{
-    int             i, j;
-    double          sqrx, sqry, sqrz, sqrdist;
-    double          tmpx, tmpy, tmpz;
-    double          variance;
-    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
-    double         *var = cdsA->var;
-    const Cds     **cds = (const Cds **) cdsA->cds;
-    Cds            *cdsj = NULL;
-    const double   *avex = (const double *) cdsA->avecds->x,
-                   *avey = (const double *) cdsA->avecds->y,
-                   *avez = (const double *) cdsA->avecds->z;
-    double          nu, nusum;
-
-    variance = 0.0;
-    for (i = 0; i < vlen; ++i)
-    {
-        sqrx = sqry = sqrz = 0.0;
-        nusum = 0.0;
-        for (j = 0; j < cnum; ++j)
-        {
-            cdsj = (Cds *) cds[j];
-            nu = cdsj->nu[i];
-            tmpx = cdsj->x[i] - avex[i];
-            sqrx += nu * tmpx * tmpx;
-            tmpy = cdsj->y[i] - avey[i];
-            sqry += nu * tmpy * tmpy;
-            tmpz = cdsj->z[i] - avez[i];
-            sqrz += nu * tmpz * tmpz;
-
-            nusum += nu;
-
-/*          printf("\n%4d %4d %e - %e %e %e - %e %e %e", */
-/*                 i, j, nu, */
-/*                 cdsj->x[i], cdsj->y[i], cdsj->z[i], */
-/*                 avex[i], avey[i], avez[i]); */
-        }
-
-        sqrdist = sqrx + sqry + sqrz;
-
-        //var[i] = sqrdist / (3.0 * cdsA->df[i]);
-        var[i] = sqrdist / (3.0 * nusum); // should be the same as with df[i]
-        variance += var[i];
-/*         printf("\nvar[%3d]:%f\n", i, var[i]); */
-    }
-
-    //WriteVariance(cdsA, "jacob.log"); exit(1);
-    variance /= (double) vlen;
-    stats->stddev = sqrt(variance);
-
-    return(stats->stddev);
-}
-
-
 void
 WriteTransformations(CdsArray *cdsA, char *outfile_name)
 {
@@ -314,15 +166,19 @@ WriteTransformations(CdsArray *cdsA, char *outfile_name)
                 cdsA->avecds->z[i]);
     }
 
+//     fprintf(transfile, "\n# Phi and n\n");
+//
+//     fprintf(transfile, "PHI %18.12f\n", );
+
     fprintf(transfile, "\n\n");
     fflush(NULL);
 
-    free(v);
+    MyFree(v);
     fclose(transfile);
 }
 
 
-static void
+void
 WriteVariance(CdsArray *cdsA, char *outfile_name)
 {
     FILE           *varfile = NULL;
@@ -397,6 +253,35 @@ WriteVariance(CdsArray *cdsA, char *outfile_name)
 }
 
 
+void
+WriteResiduals(CdsArray *cdsA, char *outfile_name)
+{
+    FILE           *residualfile = NULL;
+    int             i;
+
+    residualfile = myfopen(outfile_name, "w");
+    if (residualfile == NULL)
+    {
+        perror("\n  ERROR");
+        fprintf(stderr,
+                "\n  ERROR99: could not open file '%s'. \n", outfile_name);
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    for (i = 0; i < cdsA->cnum * cdsA->vlen * 3; ++i)
+    {
+        fprintf(residualfile,
+                "%-3d %12.6f\n",
+                i+1,
+                cdsA->residuals[i]);
+    }
+    fputc('\n', residualfile);
+
+    fclose(residualfile);
+}
+
+
 static void
 Vars2Bfacts(CdsArray *cdsA)
 {
@@ -539,12 +424,26 @@ SqrCdsDist(const Cds *cds1, const int atom1,
            const Cds *cds2, const int atom2)
 {
     double          tmpx, tmpy, tmpz;
+//     const double *x1 = cds1->x;
+//     const double *y1 = cds1->y;
+//     const double *z1 = cds1->z;
+//     const double *x2 = cds2->x;
+//     const double *y2 = cds2->y;
+//     const double *z2 = cds2->z;
+//
+//     tmpx = x2[atom2] - x1[atom1];
+//     tmpy = y2[atom2] - y1[atom1];
+//     tmpz = z2[atom2] - z1[atom1];
 
     tmpx = cds2->x[atom2] - cds1->x[atom1];
     tmpy = cds2->y[atom2] - cds1->y[atom1];
     tmpz = cds2->z[atom2] - cds1->z[atom1];
 
+    #ifdef FP_FAST_FMA
+    return(fma(tmpx, tmpx, fma(tmpy, tmpy, tmpz * tmpz)));
+    #else
     return(tmpx*tmpx + tmpy*tmpy + tmpz*tmpz);
+    #endif
 }
 
 
@@ -591,35 +490,6 @@ RadiusGyration(Cds *cds, const double *weights)
 }
 
 
-static void
-WriteResiduals(CdsArray *cdsA, char *outfile_name)
-{
-    FILE           *residualfile = NULL;
-    int             i;
-
-    residualfile = myfopen(outfile_name, "w");
-    if (residualfile == NULL)
-    {
-        perror("\n  ERROR");
-        fprintf(stderr,
-                "\n  ERROR99: could not open file '%s'. \n", outfile_name);
-        PrintTheseusTag();
-        exit(EXIT_FAILURE);
-    }
-
-    for (i = 0; i < cdsA->cnum * cdsA->vlen * 3; ++i)
-    {
-        fprintf(residualfile,
-                "%-3d %12.6f\n",
-                i+1,
-                cdsA->residuals[i]);
-    }
-    fputc('\n', residualfile);
-
-    fclose(residualfile);
-}
-
-
 static void
 MomentsCds(CdsArray *cdsA)
 {
@@ -887,58 +757,151 @@ CalcMgLogL(CdsArray *cdsA)
 {
     const int       vlen = cdsA->vlen;
     const int       cnum = cdsA->cnum;
-    double         *var3N = cdsA->samplevar3N;
+    double         *var = cdsA->samplevar;
     double         *evals = cdsA->evals;
     double          mglogl = 0.0, term, lndet, sum;
     const double    phi = 2.0*stats->hierarch_p1;
-    double          nu = algo->covnu;
+    const double    ndf = algo->ndf;
     int             i;
 
-
-    CalcVar(cdsA);
+    //AveCdsNu(cdsA);
+    CalcSampleVar(cdsA);
 
     if (algo->leastsquares)
     {
         term = 0.5 * (3.0 * vlen * cnum - 2.0); // with Jeffreys prior on phi this is just 3KN/2
         sum = 0.0;
         for (i = 0; i < vlen; ++i)
-            sum += var3N[i];
+            sum += var[i];
 
         sum *= 0.5;
 
         mglogl = gsl_sf_lngamma(term)
                - 0.5 * (3.0 * vlen * cnum) * log(2.0*M_PI)
-               -term * log(sum);
+               -term * log(3.0*cnum*sum);
     }
     else if (algo->varweight)
     {
-        term = 0.5 * (3.0 * cnum + 1.0); /* n = 1 corresponds to c = 0.5 for inverse gamma */
+        term = 0.5 * (3.0 * cnum + ndf); /* n = 1 corresponds to c = 0.5 for inverse gamma */
 
         lndet = 0.0;
         for (i = 0; i < vlen; ++i)
-            lndet += log(var3N[i] + phi);
+            lndet += log(3.0*cnum*var[i] + phi);
 
         mglogl = vlen * gsl_sf_lngamma(term)
-               - term * vlen * log(M_PI)
-               + 0.5 * vlen * log(phi)
+               - term * vlen * gsl_sf_lngamma(0.5 * ndf)
+               + 0.5 * ndf * vlen * log(phi)
                - term * lndet;
 
+        mglogl -= phi; // gamma/exponential prior on phi, mean = 1
+
         //printf("mglogl: % 20.10f % 20.10f % 20.10e % 20.10f\n", term, lndet, phi, mglogl);
+        //fflush(NULL);
     }
-    else if (algo->covweight > 0)
+    else if (algo->covweight)
     {
-        term = 0.5 * (3.0 * cnum + nu);
+        term = 0.5 * (3.0 * cnum + ndf);
 
         mglogl = MultivarLnGamma(vlen, term)
-               - MultivarLnGamma(vlen, 0.5 * nu)
+               - MultivarLnGamma(vlen, 0.5 * ndf) // requires ndf >= vlen
                - 3.0 * cnum * vlen * log(M_PI)
-               + 0.5 * vlen * nu * log(phi);
+               + 0.5 * vlen * ndf * log(phi)
+               ;
 
         lndet = 0.0;
         for (i = 0; i < vlen; ++i)
             lndet += log(phi + 3.0 * cnum * evals[i]);
 
         mglogl -= term * lndet;
+        //mglogl += log(phi) - phi/1; // exponential prior on phi, mean = 1
+        mglogl -= phi/1;
+    }
+
+    stats->mlogL = mglogl;
+
+    return(mglogl);
+}
+
+
+double
+CalcMgLogLNu(CdsArray *cdsA)
+{
+    const int       vlen = cdsA->vlen;
+    const int       cnum = cdsA->cnum;
+    double         *svar = cdsA->samplevar;
+    double         *evals = cdsA->evals;
+    double          mglogl, term, lndet, sum, ri;
+    const double    phi = 2.0*stats->hierarch_p1;
+    const double    ndf = algo->ndf;
+    int             i;
+
+
+    CalcSampleVarNu(cdsA);
+
+    if (algo->leastsquares) // DLT FIX --- this has to incorporate incomplete info nu
+    {
+        term = 0.5 * (3.0 * vlen * cnum - 2.0); // with Jeffreys prior on phi this is just 3KN/2
+        sum = 0.0;
+        for (i = 0; i < vlen; ++i)
+            sum += svar[i];
+
+        sum *= 0.5;
+
+        mglogl = gsl_sf_lngamma(term)
+               - 0.5 * (3.0 * vlen * cnum) * log(2.0*M_PI)
+               -term * log(3.0*cnum*sum);
+    }
+    else if (algo->varweight)
+    {
+        mglogl = 0.0;
+        for (i = 0; i < vlen; ++i)
+        {
+            ri = cdsA->df[i];
+//             ri = 0.0;
+//             for (j = 0; j < cnum; ++j)
+//             {
+//                 nui = cdsA->cds[j]->nu[i];
+//                 ri += nui;
+//             }
+
+            term = 0.5 * (3.0 * ri + ndf); /* n = 1 corresponds to c = 0.5 for inverse gamma */
+
+            lndet = log(3.0*ri*svar[i] + phi);
+            //printf("%3d var:% e\n", i, svar[i]);
+
+            mglogl += gsl_sf_lngamma(term)
+                    - gsl_sf_lngamma(0.5 * ndf)
+                    - 1.5 * ri * log(M_PI)
+                    + 0.5 * ndf * log(phi)
+                    - term * lndet;
+        }
+
+        mglogl -= phi; // exponential prior on phi
+
+//         printf("mglogl: % 20.10f % 20.10f % 20.10e % 20.10f\n", term, lndet, phi, mglogl);
+//         fflush(NULL);
+    }
+    else if (algo->covweight)
+    {
+        term = 0.5 * (3.0 * cnum + ndf);
+
+        mglogl = MultivarLnGamma(vlen, term)
+               - MultivarLnGamma(vlen, 0.5 * ndf) // requires ndf >= vlen
+               - 3.0 * cnum * vlen * log(M_PI)
+               + 0.5 * vlen * ndf * log(phi)
+               ;
+
+        lndet = 0.0;
+        for (i = 0; i < vlen; ++i)
+            lndet += log(phi + 3.0 * cnum * evals[i]);
+
+        mglogl -= term * lndet;
+        //mglogl += log(phi) - phi/1; // exponential prior on phi, mean = 1
+        mglogl -= phi/1;
+    }
+    else
+    {
+        mglogl = 0.0;
     }
 
     stats->mlogL = mglogl;
@@ -962,7 +925,7 @@ CalcParamNum(CdsArray *cdsA)
     if (algo->leastsquares)
         params += 1.0;
 
-//     if (algo->varweight)
+//     if (algo->varweight) // no longer needed with marginal likelihood
 //         params += vlen;
 
     if (algo->covweight)
@@ -1088,9 +1051,56 @@ CalcCov(CdsArray *cdsA)
 }
 
 
-/* NB: Calculates var * 3 * N */
 void
-CalcVar(CdsArray *cdsA)
+CalcSampleVar(CdsArray *cdsA)
+{
+    double         newx, newy, newz;
+    double         stddev, fact;
+    int            i, k;
+    const int      cnum = cdsA->cnum;
+    const int      vlen = cdsA->vlen;
+    const Cds    **cds = (const Cds **) cdsA->cds;
+    const Cds     *cdsk = NULL;
+    const double   *avex = (const double *) cdsA->avecds->x,
+                   *avey = (const double *) cdsA->avecds->y,
+                   *avez = (const double *) cdsA->avecds->z;
+    double         *svar = cdsA->samplevar;
+
+    memset(svar, 0, vlen * sizeof(double));
+
+	for (k = 0; k < cnum; ++k)
+	{
+		cdsk = cds[k];
+		for (i = 0; i < vlen; ++i)
+		{
+            newx = cdsk->x[i] - avex[i];
+            newy = cdsk->y[i] - avey[i];
+            newz = cdsk->z[i] - avez[i];
+
+            #ifdef FP_FAST_FMA
+            svar[i] += fma(newx, newx, fma(newy, newy, newz * newz));
+            #else
+            svar[i] += (newx * newx) + (newy * newy) + (newz * newz);
+            #endif
+        }
+    }
+
+    fact = 1.0 / (3.0 * cnum);
+
+    stddev = 0.0;
+    for (i = 0; i < vlen; ++i)
+    {
+        svar[i] *= fact;
+        stddev += svar[i];
+    }
+
+    stats->var = stddev/vlen;
+    stats->stddev = sqrt(stddev/vlen);
+}
+
+
+void
+CalcSampleVarNu2(CdsArray *cdsA)
 {
     double         newx, newy, newz;
     double         varsum, stddev;
@@ -1102,34 +1112,108 @@ CalcVar(CdsArray *cdsA)
     const double   *avex = (const double *) cdsA->avecds->x,
                    *avey = (const double *) cdsA->avecds->y,
                    *avez = (const double *) cdsA->avecds->z;
-
-    //if (algo->doave)
-        //AveCds(cdsA);
+    double          nu, nusum;
+    double         *svar = cdsA->samplevar;
+//    const int   *df = cdsA->df;
 
     stddev = 0.0;
     for (i = 0; i < vlen; ++i)
     {
         varsum = 0.0;
+        nusum = 0.0;
         for (k = 0; k < cnum; ++k)
         {
             cdsk = cds[k];
 
+            nu = cdsk->nu[i];
+            nusum += nu;
+
             newx = cdsk->x[i] - avex[i];
             newy = cdsk->y[i] - avey[i];
             newz = cdsk->z[i] - avez[i];
 
             #ifdef FP_FAST_FMA
-            varsum += fma(newx, newx, fma(newy, newy, newz * newz));
+            varsum += nu * fma(newx, newx, fma(newy, newy, newz * newz));
             #else
-            varsum += (newx * newx) + (newy * newy) + (newz * newz);
+            varsum += nu * ((newx * newx) + (newy * newy) + (newz * newz));
             #endif
         }
 
-        cdsA->samplevar3N[i] = varsum;
+        svar[i] = varsum / (3.0 * nusum); // df[i]);
         stddev += varsum;
     }
 
-    stats->stddev = sqrt(stddev/(3.0*cnum*vlen));
+// printf("\n");
+//     for (i = 0; i < vlen; ++i)
+//         printf("ROUND:%5d %4d %16.6f %16.6f\n", algo->rounds, i, svar[i], cdsA->var[i]);
+// printf("\n");
+// fflush(NULL);
+
+    stats->var = stddev/vlen;
+    stats->stddev = sqrt(stddev/vlen);
+}
+
+
+// More efficient version than above (~2.5X ?)
+void
+CalcSampleVarNu(CdsArray *cdsA)
+{
+    double         newx, newy, newz;
+    double         stddev, nu;
+    int            i, k;
+    const int      cnum = cdsA->cnum;
+    const int      vlen = cdsA->vlen;
+    const Cds    **cds = (const Cds **) cdsA->cds;
+    const Cds     *cdsk = NULL;
+    const double   *avex = (const double *) cdsA->avecds->x,
+                   *avey = (const double *) cdsA->avecds->y,
+                   *avez = (const double *) cdsA->avecds->z;
+    double         *svar = cdsA->samplevar;
+    const int      *df = (const int *) cdsA->df;
+
+    memset(svar, 0, vlen * sizeof(double));
+
+	for (k = 0; k < cnum; ++k)
+	{
+		cdsk = cds[k];
+		for (i = 0; i < vlen; ++i)
+		{
+			nu = cdsk->nu[i];
+            if (nu > 0.0)
+            {
+				newx = cdsk->x[i] - avex[i];
+				newy = cdsk->y[i] - avey[i];
+				newz = cdsk->z[i] - avez[i];
+
+				#ifdef FP_FAST_FMA
+				svar[i] += fma(newx, newx, fma(newy, newy, newz * newz));
+				#else
+				svar[i] += (newx * newx) + (newy * newy) + (newz * newz);
+				#endif
+
+// 				if (svar[i] < DBL_MIN) // DLT DEBUG
+// 				    printf("^^^ %5d %5d: % 8.3f % 8.3f % 8.3f --- % 8.3f % 8.3f % 8.3f\n",
+// 				           k, i, cdsk->x[i], cdsk->y[i], cdsk->z[i], avex[i], avey[i], avez[i]);
+			}
+		}
+	}
+
+    stddev = 0.0;
+    for (i = 0; i < vlen; ++i)
+    {
+        svar[i] /= (3.0 * df[i]);
+        stddev += svar[i];
+    }
+
+// DLT DEBUG
+// printf("\n");
+//     for (i = 0; i < vlen; ++i)
+//         printf("ROUND:%5d %4d %4d %16.6f %16.6f -- %8.3f %8.3f %8.3f\n", algo->rounds, i, df[i], svar[i], cdsA->var[i], avex[i], avey[i], avez[i]);
+// printf("\n");
+// fflush(NULL);
+
+    stats->var = stddev/vlen;
+    stats->stddev = sqrt(stddev/vlen);
 }
 
 
@@ -1271,7 +1355,7 @@ CalcStats(CdsArray *incdsA)
             cdsA->CovMat = MatAlloc(vlen, vlen);
 
     //if (algo->alignment)
-        CalcDf(cdsA);
+    CalcDf(cdsA);
 
     if (algo->doave)
     {
@@ -1291,11 +1375,13 @@ CalcStats(CdsArray *incdsA)
             UnbiasMean(cdsA);
     }
 
-    CalcVar(cdsA);
+    //CalcSampleVar(cdsA);
 
     if (algo->docovars)
         CalcCovariances(cdsA);
 
+    memcpy(cdsA->var, cdsA->samplevar, vlen * sizeof(double));
+
     if (algo->dohierarch)
     {
         if (algo->varweight || algo->covweight)
@@ -1310,14 +1396,8 @@ CalcStats(CdsArray *incdsA)
     else
         CalcNormResiduals(cdsA);
 
-    if (algo->write_file)
-    {
-        char *residuals_name = mystrcat(algo->rootname, "_residuals.txt");
-        WriteResiduals(cdsA, residuals_name);
-        free(residuals_name);
-    }
-
-    CalcMgLogL(cdsA);
+    //CalcMgLogL(cdsA);
+    CalcMgLogLNu(cdsA);
     CalcAIC(cdsA);
     CalcBIC(cdsA);
     CalcMLRMSD(cdsA);
@@ -1327,6 +1407,8 @@ CalcStats(CdsArray *incdsA)
 
     MomentsCds(cdsA);
 
+    n = (double) vlen * cnum * 3.0;
+
     if (algo->leastsquares)
     {
         stats->omnibus_chi2 = stats->chi2; // DLT FIX
@@ -1334,22 +1416,15 @@ CalcStats(CdsArray *incdsA)
     }
     else
     {
-        stats->omnibus_chi2 = (vlen * stats->hierarch_chi2 + vlen * cnum * 3 * stats->chi2) / (vlen * cnum * 3 + vlen);
-        stats->omnibus_chi2_P = chisqr_sdf(vlen * stats->hierarch_chi2 + vlen * cnum * 3 * stats->chi2,
-                                                 vlen * cnum * 3 + vlen, 0);
+        stats->omnibus_chi2 =
+            (vlen * stats->hierarch_chi2 + n * stats->chi2) / (n + vlen);
+        stats->omnibus_chi2_P =
+            chisqr_sdf(vlen * stats->hierarch_chi2 + n * stats->chi2, n + vlen, 0);
     }
 
-    n = (double) vlen * cnum * 3;
     stats->SES = sqrt((6.0 * n * (n-1)) / ((n-2) * (n+1) * (n+3))); /* exact formulas */
     stats->SEK = sqrt((24.0 * n * (n-1) * (n-1)) / ((n-3) * (n-2) * (n+3) * (n+5)));
 
-    if (algo->write_file)
-    {
-        char *variances_name = mystrcat(algo->rootname, "_variances.txt");
-        WriteVariance(cdsA, variances_name);
-        free(variances_name);
-    }
-
     if (algo->covweight && (algo->write_file > 0 || algo->info) && algo->pca == 0)
     {
 /*         if (algo->alignment) */
@@ -1358,8 +1433,8 @@ CalcStats(CdsArray *incdsA)
 /*             CalcCovMat(cdsA); */
         char *cov_name = mystrcat(algo->rootname, "_cov.mat");
         char *cor_name = mystrcat(algo->rootname, "_cor.mat");
-        double nu = algo->covnu;
-        double fact = (3.0*cnum+nu)/(3.0*cnum+nu-vlen-1.0);
+        double ndf = algo->ndf;
+        double fact = (3.0*cnum+ndf)/(3.0*cnum+ndf-vlen-1.0);
 
         for (i = 0; i < vlen; ++i)
             for (j = 0; j < vlen; ++j)
@@ -1367,12 +1442,12 @@ CalcStats(CdsArray *incdsA)
 
         PrintCovMatGnuPlot((const double **) cdsA->CovMat, vlen, cov_name);
 
-        write_C_mat((const double **) cdsA->CovMat, vlen, 4, 10);
+        //write_C_mat((const double **) cdsA->CovMat, vlen, 4, 10);
 
         CovMat2CorMat(cdsA->CovMat, vlen);
         PrintCovMatGnuPlot((const double **) cdsA->CovMat, vlen, cor_name);
-        free(cov_name);
-        free(cor_name);
+        MyFree(cov_name);
+        MyFree(cor_name);
     }
 
     if (algo->fullpca)
@@ -1426,7 +1501,7 @@ CalcStats(CdsArray *incdsA)
         }
     }
 
-    free(evals);
+    MyFree(evals);
     MatDestroy(&rotmat);
     MatDestroy(&lastmat);
 }
