@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -62,7 +61,7 @@ CdsCopy(Cds *cds1, const Cds *cds2) // DLT FIX MAT COORDS
 
 
 void
-CdsArrayCopy(CdsArray *cdsA1, const CdsArray *cdsA2) // DLT FIX MAT COORDS
+CdsArrayCopy(CdsArray *cdsA1, const CdsArray *cdsA2)
 {
     int             i;
 
@@ -81,31 +80,29 @@ CdsArrayCopy(CdsArray *cdsA1, const CdsArray *cdsA2) // DLT FIX MAT COORDS
 
 
 void
-CdsCopyAll(Cds *cds1, const Cds *cds2) // DLT FIX MAT COORDS
+CdsCopyAll(Cds *cds1, const Cds *cds2) // DLT FIX MAT COORDS --- did it
 {
     int             i;
+    const int       vlen = cds2->vlen;
 
-    cds1->vlen = cds2->vlen;
+    cds1->vlen = vlen;
     cds1->aalen = cds2->aalen;
     cds1->model  = cds2->model;
 
     strncpy(cds1->filename, cds2->filename, FILENAME_MAX - 1);
 
-    memcpy(cds1->resName_space, cds2->resName_space, cds2->vlen * 4 * sizeof(char));
-    memcpy(cds1->chainID, cds2->chainID, cds2->vlen * sizeof(char));
-    memcpy(cds1->resSeq, cds2->resSeq, cds2->vlen * sizeof(int));
-    memcpy(cds1->x, cds2->x, cds2->vlen * sizeof(double));
-    memcpy(cds1->y, cds2->y, cds2->vlen * sizeof(double));
-    memcpy(cds1->z, cds2->z, cds2->vlen * sizeof(double));
-    memcpy(cds1->o, cds2->o, cds2->vlen * sizeof(double));
-    memcpy(cds1->b, cds2->b, cds2->vlen * sizeof(double));
-    memcpy(cds1->nu, cds2->nu, cds2->vlen * sizeof(int));
-    memcpy(cds1->mu, cds2->mu, cds2->vlen * sizeof(int));
-    memcpy(cds1->residual_x, cds2->residual_x, cds2->vlen * sizeof(double));
-    memcpy(cds1->residual_y, cds2->residual_y, cds2->vlen * sizeof(double));
-    memcpy(cds1->residual_z, cds2->residual_z, cds2->vlen * sizeof(double));
+    memcpy(cds1->resName_space, cds2->resName_space, vlen * 4 * sizeof(char));
+    memcpy(cds1->chainID, cds2->chainID, vlen * sizeof(char));
+    memcpy(cds1->resSeq, cds2->resSeq, vlen * sizeof(int));
+    memcpy(cds1->wc[0], cds2->wc[0], 5 * vlen * sizeof(double));
+//     memcpy(cds1->x, cds2->x, vlen * sizeof(double));
+//     memcpy(cds1->y, cds2->y, vlen * sizeof(double));
+//     memcpy(cds1->z, cds2->z, vlen * sizeof(double));
+//     memcpy(cds1->o, cds2->o, vlen * sizeof(double));
+//     memcpy(cds1->b, cds2->b, vlen * sizeof(double));
+    memcpy(cds1->nu, cds2->nu, vlen * sizeof(int));
 
-    MatCpySym(cds1->matrix, (const double **) cds2->matrix, 3);
+    MatCpySqr(cds1->matrix, (const double **) cds2->matrix, 3);
 
     memcpy(cds1->center, cds2->center, 3 * sizeof(double));
     memcpy(cds1->last_center, cds2->last_center, 3 * sizeof(double));
@@ -114,7 +111,7 @@ CdsCopyAll(Cds *cds1, const Cds *cds2) // DLT FIX MAT COORDS
     cds1->RMSD_from_mean  = cds2->RMSD_from_mean;
     cds1->wRMSD_from_mean = cds2->wRMSD_from_mean;
 
-    for (i = 0; i < 2; ++i)
+    for (i = 0; i < 4; ++i)
         cds1->evals[i]  = cds2->evals[i];
 }
 
@@ -122,34 +119,30 @@ CdsCopyAll(Cds *cds1, const Cds *cds2) // DLT FIX MAT COORDS
 void
 PDBCdsCopyAll(PDBCds *cds1, const PDBCds *cds2)
 {
-    cds1->vlen = cds2->vlen;
-    cds1->model  = cds2->model;
+    const int       vlen = cds1->vlen = cds2->vlen;
 
+    cds1->model  = cds2->model;
     strncpy(cds1->filename, cds2->filename, FILENAME_MAX-1);
 
-    memcpy(cds1->record_space, cds2->record_space, cds2->vlen * 8 * sizeof(char));
-    memcpy(cds1->name_space, cds2->name_space, cds2->vlen * 4 * sizeof(char));
-    memcpy(cds1->resName_space, cds2->resName_space, cds2->vlen * 4 * sizeof(char));
-    memcpy(cds1->segID_space, cds2->segID_space, cds2->vlen * 8 * sizeof(char));
-    memcpy(cds1->element_space, cds2->element_space, cds2->vlen * 4 * sizeof(char));
-    memcpy(cds1->charge_space, cds2->charge_space, cds2->vlen * 4 * sizeof(char));
+    memcpy(cds1->record_space, cds2->record_space, vlen * 8 * sizeof(char));
+    memcpy(cds1->name_space, cds2->name_space, vlen * 4 * sizeof(char));
+    memcpy(cds1->resName_space, cds2->resName_space, vlen * 4 * sizeof(char));
+    memcpy(cds1->segID_space, cds2->segID_space, vlen * 8 * sizeof(char));
+    memcpy(cds1->element_space, cds2->element_space, vlen * 4 * sizeof(char));
+    memcpy(cds1->charge_space, cds2->charge_space, vlen * 4 * sizeof(char));
 
-    memcpy(cds1->serial, cds2->serial, cds2->vlen * sizeof(int));
-    memcpy(cds1->Hnum, cds2->Hnum, cds2->vlen * sizeof(char));
-    memcpy(cds1->altLoc, cds2->altLoc, cds2->vlen * sizeof(char));
-    memcpy(cds1->xchainID, cds2->xchainID, cds2->vlen * sizeof(char));
-    memcpy(cds1->chainID, cds2->chainID, cds2->vlen * sizeof(char));
-    memcpy(cds1->resSeq, cds2->resSeq, cds2->vlen * sizeof(int));
-    memcpy(cds1->iCode, cds2->iCode, cds2->vlen * sizeof(char));
-    memcpy(cds1->x, cds2->x, cds2->vlen * sizeof(double));
-    memcpy(cds1->y, cds2->y, cds2->vlen * sizeof(double));
-    memcpy(cds1->z, cds2->z, cds2->vlen * sizeof(double));
-    memcpy(cds1->occupancy, cds2->occupancy, cds2->vlen * sizeof(double));
-    memcpy(cds1->tempFactor, cds2->tempFactor, cds2->vlen * sizeof(double));
+    memcpy(cds1->serial, cds2->serial, vlen * sizeof(int));
+    memcpy(cds1->Hnum, cds2->Hnum, vlen * sizeof(char));
+    memcpy(cds1->altLoc, cds2->altLoc, vlen * sizeof(char));
+    memcpy(cds1->xchainID, cds2->xchainID, vlen * sizeof(char));
+    memcpy(cds1->chainID, cds2->chainID, vlen * sizeof(char));
+    memcpy(cds1->resSeq, cds2->resSeq, vlen * sizeof(int));
+    memcpy(cds1->iCode, cds2->iCode, vlen * sizeof(char));
+    memcpy(cds1->c[0], cds2->c[0], 5 * vlen * sizeof(double));
 
-    memcpy(cds1->nu, cds2->nu, cds2->vlen * sizeof(int));
+    memcpy(cds1->nu, cds2->nu, vlen * sizeof(int));
 
-    MatCpySym(cds1->matrix, (const double **) cds2->matrix, 3);
+    MatCpySqr(cds1->matrix, (const double **) cds2->matrix, 3);
     memcpy(cds1->translation, cds2->translation, 3 * sizeof(double));
 }
 
@@ -234,7 +227,7 @@ CopyCds2PDB(PDBCds *pdbcds, const Cds *cds)
     pdbcds->model = cds->model;
     strncpy(pdbcds->filename, cds->filename, FILENAME_MAX-1);
 
-    MatCpySym(pdbcds->matrix, (const double **) cds->matrix, 3);
+    MatCpySqr(pdbcds->matrix, (const double **) cds->matrix, 3);
 
     for (i = 0; i < 3; ++i)
         pdbcds->translation[i]  = cds->translation[i];
@@ -287,6 +280,46 @@ CopyCds2PDB(PDBCds *pdbcds, const Cds *cds)
 }
 
 
+int
+NMRValidPDBCdsArray(PDBCdsArray *pdbA)
+{
+    int             ii, jj;
+    int             vlen = pdbA->cds[0]->vlen;
+    char            ch0, ch1, ch2;
+
+    // Do they all have the same length? If not, return zero.
+    for (ii = 1; ii < pdbA->cnum; ++ii)
+        if (pdbA->cds[ii]->vlen != vlen)
+        {
+            fprintf(stderr,
+                    "\n  ERROR20: PDB coordinates 0 and %d are unequal length [%d vs %d]. \n\n",
+                    ii, vlen, pdbA->cds[ii]->vlen);
+            exit(EXIT_FAILURE);
+        }
+
+    // Do they also all have the same sequence? If not, return zero.
+    for (ii = 0; ii < vlen; ++ii)
+    {
+        ch0 = pdbA->cds[0]->resName[ii][0];
+        ch1 = pdbA->cds[0]->resName[ii][1];
+        ch2 = pdbA->cds[0]->resName[ii][2];
+        for (jj = 1; jj < pdbA->cnum; ++jj)
+            if((pdbA->cds[jj]->resName[ii][0] != ch0)
+            || (pdbA->cds[jj]->resName[ii][1] != ch1)
+            || (pdbA->cds[jj]->resName[ii][2] != ch2))
+            {
+                fprintf(stderr,
+                        "\n  ERROR20: sequence %d is different than 0 ['%c%c%c' vs '%c%c%c']. \n\n", jj,
+                        pdbA->cds[jj]->resName[ii][0], pdbA->cds[jj]->resName[ii][1], pdbA->cds[jj]->resName[ii][2],
+                        ch0, ch1, ch2);
+                exit(EXIT_FAILURE);
+            }
+    }
+
+    return(vlen);
+}
+
+
 int
 NMRCheckPDBCdsArray(PDBCdsArray *pdbA)
 {
@@ -755,6 +788,7 @@ CenMassCov2(const double **c, double **cc, const double **wtmat, const int vlen,
 
     CalcCovCds2(cc, c, wtmat, vlen);
 
+    // Alternatively, we could normalize only once in HierarchVars
     wtsum = 0.0;
     for (i = 0; i < vlen; ++i)
         for (j = 0; j < vlen; ++j)
@@ -1179,7 +1213,7 @@ AveCdsTB(CdsArray *cdsA, int omit)
     const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     const Cds     **cds = (const Cds **) cdsA->cds;
     Cds            *cdsj = NULL;
-    double          invcnum = 1.0 / (double) (cnum-1);
+    double          invcnum = 1.0 / (double) (cnum-1.0);
 
     memset(avex, 0, vlen * sizeof(double));
     memset(avey, 0, vlen * sizeof(double));
@@ -1300,7 +1334,7 @@ AveCds_pth(CdsArray *cdsA, AveData **avedata, pthread_t *callThd,
 
 
 void
-AveCdsNu(CdsArray *cdsA)
+AveCdsNu2(CdsArray *cdsA)
 {
     int             i, j;
     double         *avex = cdsA->avecds->x,
@@ -1322,14 +1356,66 @@ AveCdsNu(CdsArray *cdsA)
         {
             cdsj = (Cds *) cds[j];
             nu = cdsj->nu[i];
-            nusum += nu;
-            avex[i] += nu * cdsj->x[i];
-            avey[i] += nu * cdsj->y[i];
-            avez[i] += nu * cdsj->z[i];
+            if (nu > 0.0)
+            {
+                nusum += nu;
+                avex[i] += cdsj->x[i];
+                avey[i] += cdsj->y[i];
+                avez[i] += cdsj->z[i];
+            }
         }
 
         invnu = 1.0 / nusum;
 
+//printf("nusum[%3d]: %f\n", i, nusum);
+
+        avex[i] *= invnu;
+        avey[i] *= invnu;
+        avez[i] *= invnu;
+    }
+}
+
+
+//  More efficient version than above (~5X as fast?)
+void
+AveCdsNu(CdsArray *cdsA)
+{
+    int             i, j;
+    double         *avex = cdsA->avecds->x,
+                   *avey = cdsA->avecds->y,
+                   *avez = cdsA->avecds->z;
+    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
+    const Cds     **cds = (const Cds **) cdsA->cds;
+    Cds            *cdsj = NULL;
+    double          nu, invnu;
+    const int      *df = (const int *) cdsA->df;
+
+    memset(avex, 0, vlen * sizeof(double));
+    memset(avey, 0, vlen * sizeof(double));
+    memset(avez, 0, vlen * sizeof(double));
+
+    for (j = 0; j < cnum; ++j)
+    {
+        cdsj = (Cds *) cds[j];
+
+        for (i = 0; i < vlen; ++i)
+        {
+            nu = cdsj->nu[i];
+            if (nu > 0.0)
+            {
+                avex[i] += cdsj->x[i];
+                avey[i] += cdsj->y[i];
+                avez[i] += cdsj->z[i];
+            }
+        }
+    }
+
+    for (i = 0; i < vlen; ++i)
+    {
+        invnu = 1.0 / df[i];
+
+//printf("nusum[%3d]: %f\n", i, nusum);
+
         avex[i] *= invnu;
         avey[i] *= invnu;
         avez[i] *= invnu;
