@@ -41,7 +41,7 @@
  *      in press, Journal of Computational Chemistry
  *
  *
- *  Copyright (c) 2009-2014 Pu Liu and Douglas L. Theobald
+ *  Copyright (c) 2009-2015 Pu Liu and Douglas L. Theobald
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without modification, are permitted
@@ -97,7 +97,7 @@ InnerProduct(double *A, double **coords1, double **coords2, const int len, const
 
     A[0] = A[1] = A[2] = A[3] = A[4] = A[5] = A[6] = A[7] = A[8] = 0.0;
 
-    if (weight != NULL)
+    if (weight)
     {
         for (i = 0; i < len; ++i)
         {
@@ -351,7 +351,7 @@ CenterCoords(double **coords, const int len, const double *weight)
 
     xsum = ysum = zsum = 0.0;
 
-    if (weight != NULL)
+    if (weight)
     {
         wsum = 0.0;
         for (i = 0; i < len; ++i)
