@@ -1,24 +1,23 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2014 Douglas L. Theobald
+    Copyright (C) 2004-2015 Douglas L. Theobald
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This file is part of THESEUS.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    THESEUS is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as
+    published by the Free Software Foundation, either version 3 of
+    the License, or (at your option) any later version.
+
+    THESEUS is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the:
-
-    Free Software Foundation, Inc.,
-    59 Temple Place, Suite 330,
-    Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public
+    License along with THESEUS in the file 'COPYING'. If not, see
+    <http://www.gnu.org/licenses/>.
 
     -/_|:|_|_\-
 */
@@ -31,13 +30,6 @@
     #error NOT a Standard C environment
 #endif
 
-/* Temporary workaround for broken OSX system headers, shipped in 
-   XCode 1.5. They're declaring stuff 'static inline', and this is 
-   incompatible with the '-ansi' flag I pass to gcc.
-   See Apple bug #3805571. */ 
-#if defined(__APPLE__) && !defined(inline) 
-    #define inline __inline__ 
-#endif
 
 #include <pthread.h>
 #include <stdlib.h>
@@ -56,6 +48,7 @@
     #include <sys/resource.h>
 #endif
 #include <sys/types.h>
+#include <gsl/gsl_math.h>
 #include <gsl/gsl_sf_gamma.h>
 #include "Error.h"
 #include "CovMat.h"
@@ -63,6 +56,7 @@
 #include "distfit.h"
 #include "Embed.h"
 #include "FragDist.h"
+#include "HierarchVars.h"
 #include "pdbUtils.h"
 #include "pdbMalloc.h"
 #include "pdbStats.h"
@@ -71,16 +65,12 @@
 #include "RandCds.h"
 #include "MultiPose.h"
 #include "MultiPose2MSA.h"
-#include "MultiPoseMix.h"
+#include "MultiPosePth.h"
 #include "QuarticHornFrag.h"
 #include "GibbsMet.h"
 #include "ProcGSLSVDNu.h"
+#include "StructAlign.h"
+#include "NWfill.h"
 #include "termcol.h"
 
-static void
-ParseCmdLine(int argc, char *argv[], CdsArray *baseA);
-
-static void
-PrintSuperposStats(CdsArray *cdsA);
-
 #endif
